#include "stdafx.h"
#include "ConnectedComponents.h"

#include <math.h>
#include <string.h>

#include "ConnectedComponent.h"
#include "ExtraMath.h"
#include "Run.h"

// CONSTRUCTION

CConnectedComponents::CConnectedComponents()
{
	m_bFourConnected = true;
	m_nCCs = 0;
	m_nCCsAlloc = 0;
	m_pCCs = NULL;
	m_nHeight = 0;
	m_nRuns = NULL;
	m_nRunsAlloc = NULL;
	m_pRuns = NULL;
}

CConnectedComponents::~CConnectedComponents()
{
	int i,y;
	
	for(y = 0; y < m_nHeight; y++)
	{
		for(i = 0; i < m_nRunsAlloc[y]; i++)
		{
			delete m_pRuns[y][i];
		}
		delete [] m_pRuns[y];
	}
	delete [] m_pRuns;
	delete [] m_nRuns;
	delete [] m_nRunsAlloc;

	for(i = 0; i < m_nCCsAlloc; i++)
	{
		delete m_pCCs[i];
	}
	
	delete [] m_pCCs;
}


// METHODS

bool CConnectedComponents::AddCC(CRun * Run)
{
	int i;
	
	while(m_nCCs + 1 > m_nCCsAlloc)
	{
		CConnectedComponent ** temp = new CConnectedComponent * [m_nCCsAlloc + CC_INCR];
		
		if(temp == NULL)
		{
			fprintf(stderr,"AddCC: Couldn't Allocate Memory for temp\n");
			return false;
		}
		
		memcpy(temp,m_pCCs,sizeof(CConnectedComponent *) * m_nCCsAlloc);
		
		for(i = 0; i < CC_INCR; i++)
		{
			temp[m_nCCsAlloc + i] = new CConnectedComponent();
			
			if(temp[m_nCCsAlloc + i] == NULL)
			{
				fprintf(stderr,"AddCC: Couldn't Allocate Memory for temp[%d]\n",m_nCCsAlloc + i);
				return false;
			}
		}
		
		delete [] m_pCCs;
		m_pCCs = temp;
		
		m_nCCsAlloc += CC_INCR;
	}
	
	if(m_pCCs[m_nCCs]->AddRun(Run) == false)
	{
		fprintf(stderr,"AddCC: AddRun Returned false\n");
		return false;
	}
	
	m_nCCs++;
	
	return true;
}

bool CConnectedComponents::AddRun(int y, int x1, int x2)
{
//	printf("AddRun (y,x1,x2) = (%d,%d,%d)\n\n",y,x1,x2);
	
	int i;
	
	if(m_nRuns == NULL)
		m_nRuns = new int[m_nHeight];
	
	while(m_nRuns[y] + 1 > m_nRunsAlloc[y])
	{
		CRun ** temp = new CRun * [m_nRunsAlloc[y] + RUN_INCR];
		
		if(temp == NULL)
		{
			fprintf(stderr,"AddRun: Couldn't Allocate Memory for temp\n");
			return false;
		}
		
		memcpy(temp,m_pRuns[y],sizeof(CRun *) * m_nRunsAlloc[y]);
		
		for(i = 0; i < RUN_INCR; i++)
		{
			temp[m_nRunsAlloc[y] + i] = new CRun();
			
			if(temp[m_nRunsAlloc[y] + i] == NULL)
			{
				fprintf(stderr,"AddRun: Couldn't Allocate Memory for temp[%d]\n",m_nRunsAlloc[y] + i);
				return false;
			}
		}
		
		delete [] m_pRuns[y];
		m_pRuns[y] = temp;
		
		m_nRunsAlloc[y] += RUN_INCR;
	}
	
	m_pRuns[y][m_nRuns[y]]->Create(y,x1,x2);
	
	m_nRuns[y]++;
	
	CRun * ThisRun, * CompRun;
	CConnectedComponent * CC;
	
	ThisRun = m_pRuns[y][m_nRuns[y] - 1];
	
	int NoOverlaps = 0;
	
	if(y > 0)
	{
		for(i = 0; i < m_nRuns[y - 1]; i++)
		{
			CompRun = m_pRuns[y - 1][i];
	
			if
			(
				CompRun->GetY()  == ThisRun->GetY() - 1
			
				&&
			
				(
					(
						CompRun->GetX1() <= ThisRun->GetX2() &&
						CompRun->GetX2() >= ThisRun->GetX1() &&
						m_bFourConnected
					)
			
					||
				
					(
						CompRun->GetX1() <= ThisRun->GetX2() + 1 &&
						CompRun->GetX2() >= ThisRun->GetX1() - 1 &&
						!m_bFourConnected
					)
				)
			)
			{
				NoOverlaps++;
				
				if(NoOverlaps == 1)
				{
					// Found First Overlap - Add Run to Above Run's CC
					
					CC = CompRun->GetCC();
					if(CC == NULL)
					{
						fprintf(stderr,"AddRun: CC of Run is NULL\n");
						return false;
					}
					
					CC->AddRun(ThisRun);
				}
				else
				{
					// Found Another Overlap - Merge CCs of two Runs
					if(ThisRun->GetCC() != CompRun->GetCC())
					{
						CConnectedComponent * Merged  = ThisRun->GetCC();
						CConnectedComponent * Deleted = CompRun->GetCC();
						Merged->Merge(Deleted);
//						Deleted->RemoveRunPointers();
						DeleteCC(Deleted);
					}
				}
			}
		}
		
		if(NoOverlaps == 0)
		{
			if(AddCC(ThisRun) == false)
			{
				fprintf(stderr,"AddRun: AddCC Returned false\n");
				return false;
			}
		}
	}
	else
	{
		if(AddCC(ThisRun) == false)
		{
			fprintf(stderr,"AddRun: AddCC Returned false (2)\n");
			return false;
		}
	}
	
	return true;
}

//CHistogram * CConnectedComponents::CreateBetweenLineNNDistHistogram(double RefAngle,double Tolerance)
//{
//	int i,j;
//	double Angle,Distance;
//	CConnectedComponent * CC1, * CC2;
//	CHistogram * Ret = new CHistogram(0);
//	
//	if(Tolerance < 0.0)
//		Tolerance = -Tolerance;
//	
//	for(i = 0; i < m_nCCs; i++)
//	{
//		CC1 = m_pCCs[i];
//		for(j = 0; j < m_pCCs[i]->GetNoNeighbours(); j++)
//		{
//			CC2 = CC1->GetNeighbour(j);
//
//			if(CC2 != NULL)
//			{
//				Angle = CExtraMath::AngleTwoPoints(CC1->GetCentroidX(),CC1->GetCentroidY(),CC2->GetCentroidX(),CC2->GetCentroidY());
//				Distance = CExtraMath::DistanceTwoPoints(CC1->GetCentroidX(),CC1->GetCentroidY(),CC2->GetCentroidX(),CC2->GetCentroidY());
//			
//				if(Angle >= RefAngle - Tolerance && Angle <= RefAngle + Tolerance)
//				{
//					if(int(Distance) >= Ret->GetNoValues())
//					{
//						Ret->Resize(int(Distance) + 1);
//					}
//					Ret->Increment(int(Distance));
//				}
//			}
//		}
//	}
//	return Ret;
//}
//
//CHistogram * CConnectedComponents::CreateComponentSizeHistogram()
//{
//	CHistogram * Hist = new CHistogram();
//	
//	int i;
//	int Size;
//	
//	if(Hist == NULL)
//	{
//		fprintf(stderr,"CreateComponentSizeHistogram: Couldn't Allocate Memory for Hist\n");
//		return NULL;
//	}
//	
//	for(i = 0; i < m_nCCs; i++)
//	{
///*		printf("X1\t%d\n",m_pCCs[i]->GetX1());
//		printf("X2\t%d\n",m_pCCs[i]->GetX2());
//		printf("Y1\t%d\n",m_pCCs[i]->GetY1());
//		printf("Y2\t%d\n",m_pCCs[i]->GetY2());*/
//		Size = m_pCCs[i]->GetSize();
////		printf("Size: %d\n",Size);
//		
////		printf("Hist->GetNoValues(): %d\n",Hist->GetNoValues());
//		if(Size >= Hist->GetNoValues())
//		{
//			if(Hist->Resize(Size + 1) == false)
//			{
//				fprintf(stderr,"CreateComponentSizeHistogram: Resize Returned false\n");
//				return false;
//			}
//		}
//		
//		if(Hist->Increment(Size) == false)
//		{
//			fprintf(stderr,"CreateComponentSizeHistogram: Increment Returned false\n");
//			return false;
//		}
//	}
//	
//	printf("COMPONENT SIZE HISTOGRAM\n-----------\n");
//	for(i = 0; i < Hist->GetNoValues(); i++)
//	{
//		printf("%d\t%d\n",i,Hist->GetValue(i));
//	}
//	
//	return Hist;
//}
//
//

bool CConnectedComponents::CreateFromMap(int ** Map, int mapwidth, int mapheight, bool FourConnected, int LookForColours)
{
	int Width = mapwidth;
	int Height = mapheight;
	
	int i,x,y;
	int LookForColour = LookForColours;
	bool InColour;

	int X1;

	// DELETE PREVIOUSLY ALLOCATED RUNS & CCS
	for(y = 0; y < m_nHeight; y++)
	{
		for(i = 0; i < m_nRuns[y]; i++)
		{
			delete m_pRuns[y][i];
		}
		delete [] m_pRuns[y];
	}
	delete [] m_pRuns;
	delete [] m_nRuns;
	delete [] m_nRunsAlloc;
	
	for(i = 0; i < m_nCCs; i++)
		delete m_pCCs[i];
	delete [] m_pCCs;
	m_nCCs = 0;
	
	// MAKE NEW ALLOCATIONS
	
	m_nHeight = Height;
	m_nWidth  = Width;
	
	m_pRuns = new CRun ** [Height];
	
	if(m_pRuns == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns\n");
		return false;
	}
	
	m_nRuns = new int[Height];
	
	if(m_nRuns == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRuns\n");
		return false;
	}
	
	m_nRunsAlloc = new int[Height];
	
	if(m_nRunsAlloc == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRunsAlloc\n");
		return false;
	}
	
	for(y = 0; y < m_nHeight; y++)
	{
		m_pRuns[y] = new CRun * [RUN_INIT];
		if(m_pRuns[y] == NULL)
		{
			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d]\n",y);
			return false;
		}

		for(i = 0; i < RUN_INIT; i++)
		{
			m_pRuns[y][i] = new CRun();
			if(m_pRuns[y][i] == NULL)
			{
				fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d][%d]\n",y,i);
				return false;
			}
		}
		
		m_nRuns[y] = 0;
		m_nRunsAlloc[y] = RUN_INIT;
	}
	
	m_pCCs = new CConnectedComponent * [CC_INIT];
	if(m_pCCs == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs\n");
		return false;
	}
	
	for(i = 0;i < CC_INIT; i++)
	{
		m_pCCs[i] = new CConnectedComponent();
		if(m_pCCs[i] == NULL)
		{
			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs[%d]\n",i);
			return false;
		}
	}
	m_nCCsAlloc = CC_INIT;

	m_bFourConnected = FourConnected;

//	fprintf(stderr,"\nERROR! NOT COVERING WHOLE IMAGE!!!\n\n");
	for(y = 0; y < m_nHeight; y++)
	{
//		printf("y = %d\n",y);
		InColour = false;
		
		for(x = 0; x < Width; x++)
		{
			//if(Image->IsBlack(x,y) == LookForColour && !InColour)
			if(Map[x][y] == LookForColour && !InColour)
			{
				// Found START OF RUN
				
				if(x == Width - 1)
				{
					// RUN of WIDTH 1 at END of LINE
					if(AddRun(y,x,x)==false)
					{
						fprintf(stderr,"CreateFromImage: AddRun(y,x,x) Returned false\n");
						return false;
					}
				}
				else
				{
					InColour = true;
					X1 = x;
				}
			}
			//else if(x == Width - 1 && InColour && Image->IsBlack(x,y) == LookForColour)
			else if(x == Width - 1 && InColour && Map[x][y] == LookForColour)
			{
				InColour = false;
				if(AddRun(y, X1, x)==false)
				{
					fprintf(stderr,"CreateFromImage: AddRun(y,X1,x) Returned false\n");
					return false;
				}
			}
			//else if(InColour && Image->IsBlack(x,y) != LookForColour)
			else if(InColour && Map[x][y] != LookForColour)
			{
				InColour = false;
				if(AddRun(y, X1, x - 1)==false)
				{
					fprintf(stderr,"CreateFromImage: AddRun(y,X1,x-1) Returned false\n");
					return false;
				}
			}
		}
	}

	return true;
}

//bool CConnectedComponents::CreateFromImage(CBiLevelImage * Image, bool FourConnected, bool LookForBlack)
//{
//	int Width = Image->GetWidth();
//	int Height = Image->GetHeight();
//	
//	int i,x,y;
//	bool LookForColour = LookForBlack;
//	bool InColour;
//
//	int X1;
//
//	// DELETE PREVIOUSLY ALLOCATED RUNS & CCS
//	for(y = 0; y < m_nHeight; y++)
//	{
//		for(i = 0; i < m_nRuns[y]; i++)
//		{
//			delete m_pRuns[y][i];
//		}
//		delete [] m_pRuns[y];
//	}
//	delete [] m_pRuns;
//	delete [] m_nRuns;
//	delete [] m_nRunsAlloc;
//	
//	for(i = 0; i < m_nCCs; i++)
//		delete m_pCCs[i];
//	delete [] m_pCCs;
//	
//	// MAKE NEW ALLOCATIONS
//	
//	m_nHeight = Height;
//	m_nWidth  = Width;
//	
//	m_pRuns = new CRun ** [Height];
//	
//	if(m_pRuns == NULL)
//	{
//		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns\n");
//		return false;
//	}
//	
//	m_nRuns = new int[Height];
//	
//	if(m_nRuns == NULL)
//	{
//		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRuns\n");
//		return false;
//	}
//	
//	m_nRunsAlloc = new int[Height];
//	
//	if(m_nRunsAlloc == NULL)
//	{
//		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRunsAlloc\n");
//		return false;
//	}
//	
//	for(y = 0; y < m_nHeight; y++)
//	{
//		m_pRuns[y] = new CRun * [RUN_INIT];
//		if(m_pRuns[y] == NULL)
//		{
//			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d]\n",y);
//			return false;
//		}
//
//		for(i = 0; i < RUN_INIT; i++)
//		{
//			m_pRuns[y][i] = new CRun();
//			if(m_pRuns[y][i] == NULL)
//			{
//				fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d][%d]\n",y,i);
//				return false;
//			}
//		}
//		
//		m_nRuns[y] = 0;
//		m_nRunsAlloc[y] = RUN_INIT;
//	}
//	
//	m_pCCs = new CConnectedComponent * [CC_INIT];
//	if(m_pCCs == NULL)
//	{
//		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs\n");
//		return false;
//	}
//	
//	for(i = 0;i < CC_INIT; i++)
//	{
//		m_pCCs[i] = new CConnectedComponent();
//		if(m_pCCs[i] == NULL)
//		{
//			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs[%d]\n",i);
//			return false;
//		}
//	}
//	m_nCCsAlloc = CC_INIT;
//
//	m_bFourConnected = FourConnected;
//
////	fprintf(stderr,"\nERROR! NOT COVERING WHOLE IMAGE!!!\n\n");
//	for(y = 0; y < m_nHeight; y++)
//	{
////		printf("y = %d\n",y);
//		InColour = false;
//		
//		for(x = 0; x < Width; x++)
//		{
//			if(Image->IsBlack(x,y) == LookForColour && !InColour)
//			{
//				// Found START OF RUN
//				
//				if(x == Width - 1)
//				{
//					// RUN of WIDTH 1 at END of LINE
//					if(AddRun(y,x,x)==false)
//					{
//						fprintf(stderr,"CreateFromImage: AddRun(y,x,x) Returned false\n");
//						return false;
//					}
//				}
//				else
//				{
//					InColour = true;
//					X1 = x;
//				}
//			}
//			else if(x == Width - 1 && InColour && Image->IsBlack(x,y) == LookForColour)
//			{
//				InColour = false;
//				if(AddRun(y, X1, x)==false)
//				{
//					fprintf(stderr,"CreateFromImage: AddRun(y,X1,x) Returned false\n");
//					return false;
//				}
//			}
//			else if(InColour && Image->IsBlack(x,y) != LookForColour)
//			{
//				InColour = false;
//				if(AddRun(y, X1, x - 1)==false)
//				{
//					fprintf(stderr,"CreateFromImage: AddRun(y,X1,x-1) Returned false\n");
//					return false;
//				}
//			}
//		}
//	}
//
//	return true;
//}
//
//CHistogram * CConnectedComponents::CreateNNAngleHist(double BinAngle)
//{
//	CConnectedComponent * CC1, * CC2;
//	CHistogram * Hist;
//	double Dist;
//	int i,j;
//	
//	if(BinAngle <= 0.0)
//	{
//		fprintf(stderr,"BinAngle <= 0.0\n");
//		return NULL;
//	}
//	else if(BinAngle > 180.0)
//	{
//		fprintf(stderr,"BinAngle <= 0.0\n");
//		return NULL;
//	}
//
//	Hist = new CHistogram(int(180.0 / BinAngle));
//
//	for(i = 0; i < m_nCCs; i++)
//	{
//		CC1 = m_pCCs[i];
//		
////		for(j = 0; j < CC1->GetNoNeighbours(); j++)
//		for(j = 0; j < 1; j++)
//		{
//			CC2 = CC1->GetNeighbour(j);
//			if(CC2 != NULL)
//			{
//				Dist = CExtraMath::AngleTwoPoints(CC1->GetCentroidX(),CC1->GetCentroidY(),CC2->GetCentroidX(),CC2->GetCentroidY());
//				while(Dist < 0.0)
//					Dist += 180.0;
//				while(Dist >= 180.0)
//					Dist -= 180.0;
//				Hist->Increment(int((double(Hist->GetNoValues()) * Dist) / 180.0));
//			}
//		}
//	}
//	
//	return Hist;
//}
//
bool CConnectedComponents::CreateSubSet(CConnectedComponents * CC,int LowerBound, int UpperBound)
{
	int Width = CC->GetWidth();
	int Height = CC->GetHeight();
	int Size;
	CConnectedComponent * TempCC;
	CRun * TempRun;
	int y,i,j;

	// DELETE PREVIOUSLY ALLOCATED RUNS & CCS
	for(y = 0; y < m_nHeight; y++)
	{
		for(i = 0; i < m_nRuns[y]; i++)
		{
			delete m_pRuns[y][i];
		}
		delete [] m_pRuns[y];
	}
	delete [] m_pRuns;
	delete [] m_nRuns;
	delete [] m_nRunsAlloc;
	
	for(i = 0; i < m_nCCs; i++)
		delete m_pCCs[i];
	delete [] m_pCCs;
	
	// MAKE NEW ALLOCATIONS
	
	m_nHeight = Height;
	m_nWidth  = Width;
	
	m_pRuns = new CRun ** [Height];
	
	if(m_pRuns == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns\n");
		return false;
	}
	
	m_nRuns = new int[Height];
	
	if(m_nRuns == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRuns\n");
		return false;
	}
	
	m_nRunsAlloc = new int[Height];
	
	if(m_nRunsAlloc == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_nRunsAlloc\n");
		return false;
	}
	
	for(y = 0; y < m_nHeight; y++)
	{
		m_pRuns[y] = new CRun * [RUN_INIT];
		if(m_pRuns[y] == NULL)
		{
			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d]\n",y);
			return false;
		}

		for(i = 0; i < RUN_INIT; i++)
		{
			m_pRuns[y][i] = new CRun();
			if(m_pRuns[y][i] == NULL)
			{
				fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pRuns[%d][%d]\n",y,i);
				return false;
			}
		}
		
		m_nRuns[y] = 0;
		m_nRunsAlloc[y] = RUN_INIT;
	}
	
	m_pCCs = new CConnectedComponent * [CC_INIT];
	if(m_pCCs == NULL)
	{
		fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs\n");
		return false;
	}
	
	for(i = 0;i < CC_INIT; i++)
	{
		m_pCCs[i] = new CConnectedComponent();
		if(m_pCCs[i] == NULL)
		{
			fprintf(stderr,"CreateFromImage: Couldn't Allocate Memory For m_pCCs[%d]\n",i);
			return false;
		}
	}
	m_nCCsAlloc = CC_INIT;

	m_bFourConnected = CC->GetFourConnected();

	for(i = 0; i < CC->GetNoCCs(); i++)
	{
		TempCC = CC->GetCC(i);
		if(TempCC == NULL)
		{
			fprintf(stderr,"CreateSubset: GetCC(%d) Returned NULL\n",TempCC);
			return false;
		}
		Size = TempCC->GetSize();
		
		if(Size >= LowerBound && Size <= UpperBound)
		{
			for(j = 0; j < TempCC->GetNoRuns(); j++)
			{
				TempRun = TempCC->GetRun(j);
				AddRun(TempRun->GetY(),TempRun->GetX1(),TempRun->GetX2());
			}
		}
	}

	return true;
}

//CHistogram * CConnectedComponents::CreateWithinLineNNDistHistogram(double RefAngle,double Tolerance)
//{
//	int i,j;
//	double Angle,Distance;
//	CConnectedComponent * CC1, * CC2;
//	CHistogram * Ret = new CHistogram(0);
//	
//	if(Tolerance < 0.0)
//		Tolerance = -Tolerance;
//	
//	for(i = 0; i < m_nCCs; i++)
//	{
//		CC1 = m_pCCs[i];
//		
//		for(j = 0; j < m_pCCs[i]->GetNoNeighbours(); j++)
//		{
//			CC2 = CC1->GetNeighbour(j);
//			
//			if(CC2 != NULL)
//			{
//				Angle = CExtraMath::AngleTwoPoints(CC1->GetCentroidX(),CC1->GetCentroidY(),CC2->GetCentroidX(),CC2->GetCentroidY());
//				Distance = CExtraMath::DistanceTwoPoints(CC1->GetCentroidX(),CC1->GetCentroidY(),CC2->GetCentroidX(),CC2->GetCentroidY());
//			
//				if(Angle >= RefAngle - Tolerance && Angle <= RefAngle + Tolerance)
//				{
//					if(int(Distance) >= Ret->GetNoValues())
//					{
//						Ret->Resize(int(Distance) + 1);
//					}
//					Ret->Increment(int(Distance));
//				}
//			}
//		}
//	}
//	return Ret;
//}
//
void CConnectedComponents::DeleteCC(CConnectedComponent * CC)
{
	bool Found = false;
	int i,x;
	
	for(i = 0; i < m_nCCs && !Found; i++)
	{
		if(m_pCCs[i] == CC)
		{
			Found = true;
			delete m_pCCs[i];
			
			for(x = i + 1; x < m_nCCs; x++)
			{
				m_pCCs[x - 1] = m_pCCs[x];
			}
			
			m_nCCs--;
		}
	}
	
	m_pCCs[m_nCCs] = new CConnectedComponent();
}

bool CConnectedComponents::FindNearestNeighbours(int K)
{
	int i,j,x,y;
	bool FoundPos;
	int ThisX,ThisY,CompX,CompY,CompDist,MAXDIST;
	
	int * Dist = new int[K];
	CConnectedComponent ** Neigh = new CConnectedComponent * [K];
	
	MAXDIST = int(sqrt(double((m_nWidth * m_nWidth) + (m_nHeight * m_nHeight)))) + 1;
	
	for(i = 0; i < K; i++)
	{
		Dist[i] = MAXDIST;
		Neigh[i] = NULL;
	}
	
	SortComponentsX();
	
	for(i = 0; i < m_nCCs; i++)
	{
		ThisX = m_pCCs[i]->GetCentroidX();
		ThisY = m_pCCs[i]->GetCentroidY();
		
		if(m_pCCs[i]->SetNoNeighbours(K) == false)
		{
			fprintf(stderr,"FindNearestNeighbours: SetNoNeighbours Returned false\n");
			return false;
		}
		
		for(j = 0; j < m_nCCs; j++)
		{
			if(i != j)
			{
				CompX = m_pCCs[j]->GetCentroidX();
				CompY = m_pCCs[j]->GetCentroidY();
				CompDist = int(sqrt(double(((CompX - ThisX) * (CompX - ThisX)) + ((CompY - ThisY) * (CompY - ThisY)))));
				FoundPos = false;
				x = 0;
				
				while(x < K && !FoundPos)
				{
					if(CompDist < Dist[x])
					{
						FoundPos = true;
						
						for(y = x + 1; y < K; y++)
						{
							Dist[y] = Dist[y - 1];
							Neigh[y] = Neigh[y - 1];
						}
						
						Dist[x] = CompDist;
						Neigh[x] = m_pCCs[j];
					}
					
					x++;
				}
			}
		}
		
		for(j = 0; j < K; j++)
		{
			m_pCCs[i]->SetNeighbour(j,Neigh[j]);
		}
	}
	
	for(i = 0; i < m_nCCs; i++)
	{
	}
	
	delete [] Dist;
	delete [] Neigh;
}

CConnectedComponent * CConnectedComponents::GetCC(int Index)
{
	if(Index < 0 || Index >= m_nCCs)
		return NULL;
	else
		return m_pCCs[Index];
}

bool CConnectedComponents::GetFourConnected()
{
	return m_bFourConnected;
}

int CConnectedComponents::GetHeight()
{
	return m_nHeight;
}

int CConnectedComponents::GetNoCCs()
{
	return m_nCCs;
}

int CConnectedComponents::GetNoRuns(int Y)
{
	return m_nRuns[Y];
}

CRun * CConnectedComponents::GetRun(int Y, int Index)
{
	return m_pRuns[Y][Index];
}

int CConnectedComponents::GetWidth()
{
	return m_nWidth;
}

void CConnectedComponents::SetHeight(int NewHeight)
{
	int i;
	
	for(i = 0; i < m_nHeight; i++)
	{
		delete [] m_pRuns[i];
	}
	delete [] m_pRuns;
	delete [] m_nRuns;
	delete [] m_nRunsAlloc;
		
	m_nHeight = NewHeight;
	m_nRuns = new int[m_nHeight];
	m_nRunsAlloc = new int[m_nHeight];
	m_pRuns = new CRun ** [m_nHeight];
	memset(m_nRuns, 0, sizeof(int) * m_nHeight);
	memset(m_nRunsAlloc, 0, sizeof(int) * m_nHeight);
	memset(m_pRuns, 0, sizeof(CRun **) * m_nHeight);
}

void CConnectedComponents::SortComponentsX()
{
	bool Changed = true;
	int i;
	CConnectedComponent * temp, * CC1, * CC2;
	
	while(Changed)
	{
		Changed = false;
		
		for(i = 0; i < m_nCCs - 1; i++)
		{
			CC1 = m_pCCs[i    ];
			CC2 = m_pCCs[i + 1];
			
			if
			(
				(CC1->GetX2() + CC1->GetX1()) / 2 >
				(CC2->GetX2() + CC2->GetX1()) / 2
			)
			{
				temp = CC1;
				CC1  = CC2;
				CC2  = temp;
			}
		}
	}
}

