#pragma once
#include "diagreyscale.h"
#include "Distribution.h"
#include "DIAHistograms.h"
#include "DIABiLevelImage.h"
#include "ConnectedComponent.h"
#include "ConnectedComponents.h"



class CDocImAnDoc;
class CDIAHistograms;

class CDIACharacter : public CObject
{
	DECLARE_SERIAL(CDIACharacter);
public:
	CDIACharacter(void);
	virtual ~CDIACharacter(void);
	void CreateCharacter(int height, long FirstSeparator, long SecondSeparator,long Yposition, CImage * p_Image , int Linenum, int Charnum);
	void DeleteCharacter();
	void CreateCharImage(CImage * p_Image, bool flag);
	void CreateCharacterFromImage(CImage * CharImage);
	void CreateLocatedCharImage(CDIAGreyScale * Image, bool flag);
	void LocateChar(CDIAGreyScale * Image);
	void LocateChar2(CDIAGreyScale * Image);
	void LocateChar3(CDIAGreyScale * Image);
	void LocateChar4(CDIAGreyScale * Image, int threshold, int noiseSize);
	void LocateChar5(CDIAGreyScale * Image, int threshold, int noiseSize);
	void LocateChar6(CDIAGreyScale * Image, int threshold, int noiseSize);
	void BackgroundAnalysis(CImage * Image);
	void ForegroundAnalysis(CImage * Image, bool IsLocated);
	void FindStrokeIntensity();
	void ExportCharacter();
	void ExportCharacterProjectionV();
	void ExportCharacterProjectionH();
	void ApplyThreshold40(CImage * Image,int percent);
	void ApplyThreshold402(CImage * Image, int PercentFore, int PercentBackg,CRect rect);
	void ApplyThreshold(CImage * Image, int threshold);
	void ApplyThreshold(CImage * Image,CRect rect, int begin,int end, int colour);
	void ApplyThreshold(CImage * Image,CRect rect, int threshold);
	void CalculateForegroundPercentage(bool clean);
	int	 CalculateForegroundPercentage2(int percent, int nPixelsForeground, CImage * Image, CRect rect);
	void ClassifyCharacterType(int CharShortAverage[]);
	void ClassifyCharacterType2(int CharShortAverage[]);
	void ClassifyCharacterType3(int CharShortAverage[]);
	void ClassifyCharacterType32(int CharShortAverage[]);
	void ClassifyCharacterType4(int Width, int Height);
	
	//void ClassifyCharacterType5(int Width, int Height);
	void ClassifyCharacterType4();
	void ClassifyTallCharacter(int CharShortAverage[],int Tallest[2]);
	void ClassifyTallCharacter2(int CharShortAverage[],int Tallest[2]);
	void ClassifyTallCharacter2(int CharShortAverage[], int Tallest[2], int index);
	//double CalculateMean(CPoint point, int neighbourhood);
	//double CalculateVariance(CPoint point, int neighbourhood, double mean);
	CDIAGreyScale* ApplyHistoTransformation(CDIAGreyScale * Image);
	void ApplyHistoTransformation(CDIAGreyScale * Image,CDIAGreyScale* &ImageReturn);
	static bool IsBlackInRun(int row, CDIAGreyScale * Image);
	static int IsBlackInRun(int row, CDIAGreyScale * Image, int threshold, int vorh);
	int CloseupLocation(CDIAGreyScale * Image, int threshold, int noiseSize,int start, int end, int LoopMovement, int movementHV);
	void FindUncertValuablePoints();

	long GetHeight();
	int GetWidth();
	long GetPosX();
	long GetPosY();
	int** ComputeCharDivisionBynPixels(int size);
	int** ComputeCharDivisionBynBoxes(CPoint size);
	void CalculateStrongMaxInProjs();
	void CalculateCrossings();
	void CalculateWeight();
	
	virtual void Serialize(CArchive& ar);
	void FindConnectedComponentsInChar(CDIAGreyScale * image);
	void UpdateBWMap(CImageMap * map, CImage * image, CRect rect);
	void UpdateBWMap(CImageMap * map, CImage * image, CRect rect,int threshold);

	void FindCC(CImage * CharImage, int Type);
	//void FindCC(CDIAGreyScale * CharImage, BYTE Type);
	void FindCharacterEdges(CImage * Image, CImageMap * Map);
	void ExtractFeatures();
	void ExtractFeatures(int sizeDivisionX, int sizeDivisionY);
	void Clear();
	void GetContours();
	void GetRuns();
	void GetRuns2();
	void GetAngles();
	void GetEdge();
	void ExportRuns();
	void GetDivisions();
	void GetDivisions(int sizeDivisionX, int sizeDivisionY);
	void CalculateMaxsDistances();
	void CreateCharacterHisto(CDocImAnDoc* doc);
	CDocImAnDoc* GetDocument();
	void SetDocument(CDocImAnDoc *pDoc);
	bool IsPixelBetweenBW(CPoint point, int colour, char move);
	void CreateCharacterForegrBackgrHisto(CDocImAnDoc* doc);
	void FindNeighbToOrangeUncertainAreas();
	void FindGreyNeighbToUncertainAreas(int color);
	bool IsNeighbourGrey(CPoint point);
	bool IsNeighbourColor(CPoint point, int color);
	bool FindUncertainCC(int INcolour, int OUTcolourSmall, int OUTcolourBig);
	void SetUncertainCCPixels(int LookForColour, int SetToColour,CConnectedComponent * CC);
	void SetUncertainChanges(int LookForColour, int SetToColour);
	void SetUncertainChanges(CImage * image,int LookForColour, int SetToColour);
	void SetUncertainChangesCC(CImage * image,int LookForColour, int SetToColour, int CC);


	int m_Width;
	long m_Height;
	long m_Position_Y;
	long m_Position_X;
	int m_BackgroundBegining;
	int m_BackgroundEnd;
	int m_BackgroundHighest;
	int m_foreground40;
	int m_foreground25;
	CString m_ImageName;
	CRect m_CharLocation;
	CDistribution <long> * m_pCharacterDistributionH;
	CDistribution <long> * m_pCharacterDistributionV;
	CDocImAnDoc * m_pdoc;
	long * m_pCharacterProjV;
	long * m_pCharacterProjH;

	//
	long * m_pCharContourR;
	long * m_pCharContourL;
	CDistribution <long> * m_pContourDistributionL;
	CDistribution <long> * m_pContourDistributionR;
	//

	CDIAGreyScale * m_pCharImage;
	CDIAGreyScale * m_pLocatedCharImage;
	
	int m_CharNum;
	int m_CharLineNum;
	
//	int m_CharShortAverage[4]; //[0] = top // [1] = ntop // [2] = bottom // [3] = nbottom
	POINT m_CharCenter;
	unsigned long m_histo[256];
	int m_nComponents;
	CComp * m_pComponents;


	typedef struct Divisions
	{
		CPoint	size;
		int**	Value;
		float**	Normalized;
	
	};
	typedef struct CROSSINGS
	{
		int		Crossings;
		int		NoCrossings;
	};
	typedef struct THRESH_VALS
	{
		int		AB;
		int		BC;
		int		BC2;
	};

	typedef struct BEST_CHAR
	{	
		float rate;
		char charname;
		bool isCapital;

	};
	typedef struct POSSIBLE_CHARS
	{
		CDIAGreyScale * image;
		BEST_CHAR bestChar;
		int no_bestChars;
		int CCPosition;
	};

	typedef struct RECOGNISED_CHAR
	{	
		char charName;
		bool isBlue;
		char secondCharName;

	};
	char m_Groundtruthchar;
	RECOGNISED_CHAR m_FTRChar;
	RECOGNISED_CHAR m_Refined1Char;
	RECOGNISED_CHAR m_Refined2Char;
	POSSIBLE_CHARS * m_pCharPossibleImages;
	int m_PossibleChars;
	//Final and important Character Features
	//Located Char Width
	int m_LocatedWidth;
	//Located Char Height
	int m_LocatedHeight;
	//Char Size Type
	
	int m_CharType[5]; //[0] = short // [1] = high // [2] = low // [3] = wrong // [4] = punctuation
	//int m_CharType;
	int m_CharFinalType;

	//Char Name
	char m_CharName;
	
	//Char Vertical Maxima
	int* m_CharVMax;
	int m_CharVMaxCount;

	//Char Horizontal Maxima
	int* m_CharHMax;
	int m_CharHMaxCount;

	//Char Weight
	int m_CharWeight; // left, centre, right

	//Char Divisions

	Divisions m_CharDivisions;

	//Char Crossings

	CROSSINGS * m_CharCrossingsH; 
	int m_CharCrossingsIndexH;
	CROSSINGS * m_CharCrossingsV; 
	int m_CharCrossingsIndexV;
	

	//Char NoHoles
	int m_nHoles;
	
	bool m_bIsCapital;

	BEST_CHAR m_CharRecogninition[3];
	BEST_CHAR m_CharMatching;

	CROSSINGS * m_HRuns2;
	int * m_HRuns;
	int m_HNoRuns2;
	int m_VNoRuns2;
	int * m_HNoRuns;
	
	CROSSINGS * m_VRuns2;
	int * m_VRuns;
	int * m_VNoRuns;

	int * m_DDRuns;
	int * m_DURuns;

	int	m_HMaxDistance;
	int	m_VMaxDistance;


	THRESH_VALS m_CharThreshVals;
	CDIAHistograms m_charHisto;
	CDIAHistograms m_charBackgrHisto;
	CDIAHistograms m_charForegrHisto;
	int m_charBackThreshVal;
	int m_charForegrThresVal;
	float m_charForegrPercent;
	float m_charBackgrPercent;
	int ** m_charUncertainAreas;

	CConnectedComponents m_CharUncertainCC;

	int m_horizontal;
	int m_vertical;
	int m_diagonalUp;
	int m_diagonalDown;
	int m_TotalEdge;
	CImage* m_EdgeImage;
};
