#include "stdafx.h"
#include "ImageLib.h"
#include "DIAGreyScale.h"
#include "DIABiLevelImage.h"


IMPLEMENT_SERIAL_MI(CDIAGreyScale, CGreyScaleImage, (UINT)(VERSIONABLE_SCHEMA|1))

CDIAGreyScale:: CDIAGreyScale(void)
{	
	m_Type = IT_DIA_GREYSCALE;
}

CDIAGreyScale::~ CDIAGreyScale(void)
{}



////////////////////////////////////////////////////////////////////////////
//        Recover Background: Recovers the background from Niblack's results
//
////////////////////////////////////////////////////////////////////////////
void CDIAGreyScale::Threshold_Gatos(int WindowSize, CRect& rect, CImage * pOutputImage, BYTE Flags)
{
       //Get a Niblack Thresholding of the image, and output results to the map
       Threshold_Niblack(5, rect, NULL, THR_OUTPUT_MAP);

       //Define the Input-Output arrays for the thresholding, based on the OutputImage attribute
       BYTE ** pDestLineArray;
       BYTE ForeColour, BackColour;
       if (pOutputImage == NULL)
       {
               pOutputImage = this;
               pDestLineArray = m_LineArray;
               ForeColour = 0x00; BackColour = 0xFF;
       }
       else if (pOutputImage->GetWidth() == m_Width && pOutputImage->GetHeight() == m_Height)
       {
               if (CDIAGreyScale * pGreyImage = dynamic_cast<CDIAGreyScale*>(pOutputImage))
               {
                       pDestLineArray = pGreyImage->m_LineArray;
                       ForeColour = 0x00; BackColour = 0xFF;
               }
               else if (CDIABiLevelImage * pBiImage = dynamic_cast<CDIABiLevelImage*>(pOutputImage))
               {
                       pDestLineArray = pBiImage->m_LineArray;
                       ForeColour = 0x00; BackColour = 0x01;
               }
               else return; //Image type not supported for output
       }
       else return; //output Image not of the same size as input

       //Check given rectangle
       if ((rect.left == -1) && (rect.right == -1) && (rect.top == -1) && (rect.bottom == -1))
       {
               rect.left = rect.top = 0;
               rect.right = m_Width;
               rect.bottom = m_Height;
       }
       else
       {
               if (rect.left < 0) rect.left = 0;
               if (rect.right > m_Width) rect.right = m_Width;
               if (rect.top < 0) rect.top = 0;
               if (rect.bottom > m_Height) rect.bottom = m_Height;
       }

       //Get Background Image in the m_BackupArray, reading from m_Array
       CreateBackup();
       CImageMap * pMap = SafeGetImageMap(PDM_STANDARD);
       BYTE Back, Fore;
       Back = pMap->GetPTByteLabel(PT_STR_BACKGROUND);

       int x, y;
       double AvgDifference = 0.0;
       double AvgBackground = 0.0;
       int PixelsCounted = 0;
       for (y = rect.top; y < rect.bottom; y ++)
       {
               BYTE * pDest = &(m_BackupLineArray[y][rect.left]);
               for (x = rect.left; x < rect.right; x++)
               {
                       if (pMap->GetMapByte(x, y) != Back)
                       {
                               long n = 0, Intensity = 0;
                               for (int ty = y - WindowSize; ty <= y + WindowSize; ty++)
                               {
                                       if (ty < rect.top) continue;
                                       if (ty >= rect.bottom) break;
                                       for (int tx = x - WindowSize; tx <= x + WindowSize; tx++)
                                       {
                                               if (tx < rect.left) continue;
                                               if (tx >= rect.right) break;
                                               n++;
                                               Intensity += m_LineArray[ty][tx];
                                       }
                               }
                               BYTE BackgroundValue = (BYTE)(Intensity / n);
                               AvgDifference += BackgroundValue - *pDest;
                               AvgBackground += BackgroundValue;
                               PixelsCounted ++;
                               *pDest = BackgroundValue;
                       }
                       pDest++;
               }
       }
       AvgDifference /= PixelsCounted;
       AvgBackground /= PixelsCounted;
       pMap->ClearMap();

       //Do the differentiation, consider the DestLineArray as the foreground (unchanged) and the sourceLineArray as the background (computed in the previous step)
       CImageMap * pOutputMap = pOutputImage->SafeGetImageMap(PDM_STANDARD);
       pOutputMap->ClearMap();
       Back = pOutputMap->GetPTByteLabel(PT_STR_BACKGROUND);
       Fore = pOutputMap->GetPTByteLabel(PT_STR_FOREGROUND);

       for (y = rect.top; y < rect.bottom; y ++)
       {
               BYTE * pFore = &(m_LineArray[y][rect.left]);
               BYTE * pBack = &(m_BackupLineArray[y][rect.left]);
               BYTE * pDest = &(pDestLineArray[y][rect.left]);
               for (x = rect.left; x < rect.right; x++)
               {
                       if (*pBack - *pFore > Threshold_Gatos_D(AvgDifference, AvgBackground, *pFore))
                       {
                               if (Flags & THR_OUTPUT_IMAGE) *pDest = ForeColour;
                               if (Flags & THR_OUTPUT_MAP) pOutputMap->SetMapByte(x, y, Fore);
                       }
                       else
                       {
                               if (Flags & THR_OUTPUT_IMAGE) *pDest = BackColour;
                               if (Flags & THR_OUTPUT_MAP) pOutputMap->SetMapByte(x, y, Back);
                       }
                       pBack++;
                       pFore++;
                       pDest++;
               }
       }
       DiscardBackup();

       //Invoke Post-processing
       if (Flags & THR_OUTPUT_IMAGE && Flags & THR_GATOS_PP)
       {
               if (CDIAGreyScale * pGreyImage = dynamic_cast<CDIAGreyScale*>(pOutputImage)) pGreyImage->Threshold_Gatos_PostProcessing(WindowSize, rect);
               else if (CDIABiLevelImage * pBiImage = dynamic_cast<CDIABiLevelImage*>(pOutputImage)) pBiImage->Threshold_Gatos_PostProcessing(WindowSize, rect);
       }
}


///////////////////
// Post-Processing //
// For this function to work, image output is required from the thresholding method
// This function is called only from inside Threshold_Gatos.
////////////////////////////////////////////////////////////////////////////////////

void CDIAGreyScale::Threshold_Gatos_PostProcessing(int WindowSize, CRect& rect)
{
       //The rectangle is suposed to be already checked by Threshold_Gatos();
       int WinSize2 = (int)(0.15*WindowSize/2.0); //WindowSize supposingly is the Height of the Characters
       int x, y;
       BYTE Fore = 0x00, Back = 0xFF;

       //1st and 2nd (Combined in a single run) Post-Processing Steps
       CreateBackup();
       for (y = rect.top; y < rect.bottom; y ++)
       {
               BYTE * pDest = &(m_LineArray[y][rect.left]);
               for (x = rect.left; x < rect.right; x++)
               {
                       if (*pDest == Fore) //Center pixel is Foreground
                       {
                               long n = 0, Psh = 0;
                               for (int ty = y - WinSize2; ty <= y + WinSize2; ty++)
                               {
                                       if (ty < rect.top) continue;
                                       if (ty >= rect.bottom) break;
                                       for (int tx = x - WinSize2; tx <= x + WinSize2; tx++)
                                       {
                                               if (tx < rect.left) continue;
                                               if (tx >= rect.right) break;
                                               n++;
                                               if (m_BackupLineArray[ty][tx] == Back) Psh++;
                                       }
                               }
                               if (Psh > 0.9 * n) *pDest = Back;
                       }
                       else //if (*pDest == Back) //Center pixel is Background
                       {
                               long n = 0, nFore = 0, Psw = 0, AvgX = 0, AvgY = 0;
                               for (int ty = y - WinSize2; ty <= y + WinSize2; ty++)
                               {
                                       if (ty < rect.top) continue;
                                       if (ty >= rect.bottom) break;
                                       for (int tx = x - WinSize2; tx <= x + WinSize2; tx++)
                                       {
                                               if (tx < rect.left) continue;
                                               if (tx >= rect.right) break;
                                               n++;
                                               if (m_BackupLineArray[ty][tx] == Fore)
                                               {
                                                       Psw++;
                                                       AvgX += tx;
                                                       AvgY += ty;
                                                       nFore++;
                                               }
                                       }
                               }
                               if (nFore > 0)
                               {
                                       AvgX /= nFore;
                                       AvgY /= nFore;
                                       if ((Psw > 0.5 * n) && (abs(x-AvgX) < 0.25*n) && (abs(y-AvgY) < 0.25*n)) *pDest = Fore;
                               }
                       }
                       pDest++;
               }
       }
       DiscardBackup();

       //3rd Post-processing step
       CreateBackup();
       for (y = rect.top; y < rect.bottom; y ++)
       {
               BYTE * pDest = &(m_LineArray[y][rect.left]);
               for (x = rect.left; x < rect.right; x++)
               {
                       if (*pDest == Back) //Center pixel is Background
                       {
                               long n = 0, Psw = 0;
                               for (int ty = y - WinSize2; ty <= y + WinSize2; ty++)
                               {
                                       if (ty < rect.top) continue;
                                       if (ty >= rect.bottom) break;
                                       for (int tx = x - WinSize2; tx <= x + WinSize2; tx++)
                                       {
                                               if (tx < rect.left) continue;
                                               if (tx >= rect.right) break;
                                               n++;
                                               if (m_BackupLineArray[ty][tx] == Fore) Psw++;
                                       }
                               }
                               if (Psw > 0.5 * n) *pDest = Fore;
                       }
                       pDest++;
               }
       }
       DiscardBackup();
}

double CDIAGreyScale::Threshold_Gatos_D(double AvgDifference, double AvgBackground, BYTE val)
{
//      static double q = 0.6, p1 = 0.5, p2 = 0.8; //Original Values
       static double q = 1.0, p1 = 0.5, p2 = 1.0;
       return q * AvgDifference * (p2 + ((1-p2)/(1+exp( (-4*val)/(AvgBackground*(1-p1)) ) + ( (2*(1+p1))/(1-p1) ) )) );
}
void CDIAGreyScale::UpdateMap(int T1, int T2,BYTE PixelType)
{
	int x, y;
	CImageMap * pMap = GetImageMap(PDM_STANDARD);
	//FillMemory(pMap->m_MapArray, m_Width*m_Height*sizeof(BYTE), pMap->GetPTByteLabel(PT_STR_FOREGROUND));
	//BYTE PTBackground = pMap->GetPTByteLabel(PT_STR_BACKGROUND);
	for (y = 0; y < m_Height; y++)
	{
		BYTE Byte;
		for (x = 0; x < m_Width; x++)
		{
			//BYTE * pByte = m_LineArray[y];
			Byte = this->GetBlue(x,y);
			if ((Byte >= T1)&& (Byte < T2)) 
			{
				pMap->SetMapByte(x,y,PixelType);
			}
		}
	}
}

void CDIAGreyScale::GetContour(int Left,int Top, int Width, int Height, long * LContour, long *RContour)
{
	if (LContour == NULL && RContour == NULL) return;
	if (LContour != NULL) ZeroMemory(LContour, Height * sizeof(long));
	if (RContour != NULL) ZeroMemory(RContour, Height * sizeof(long));
	int x, y;
	
	bool FirstBlack = false;
	long FirstVal = -1;
	bool LastBlack = false;
	long LastVal = -1;
	
	for (y = 0; y < Height; y++)
	{	FirstBlack = false;
		for (x = 0; x < Width; x++)
		{
			
			int val = (int) GetRValue(GetRGBColor(x,y));
			if (val == 0)
			{
				if (!FirstBlack)
				{
					LContour[y] = x;
					FirstBlack = true;
				}
				if (FirstBlack)
				{
					RContour[y] = x;
				}
			}
			
		}
	
	}

}
CPoint CDIAGreyScale::MoveinDirection(int direction, CPoint size, CPoint Location)
{
	CPoint newLocation(-1,-1);

	switch (direction)
	{
	case 0:
		{
			//move to the right
			if ((Location.x + 1) < size.x)
			{
				newLocation.x = Location.x + 1;
				newLocation.y = Location.y;
			}
			else
			{
				newLocation.x = -1;
				newLocation.y = -1;
			}
			break;
		}
	case 1:
		{
			//move to bottom
			if ((Location.y + 1) < size.y)
			{
				newLocation.y = Location.y + 1;
				newLocation.x = Location.x;
			}
			else
			{
				newLocation.x = -1;
				newLocation.y = -1;
			}
			break;
		}
	case 2:
		{
			//move in diagonal up
			if ((Location.x +1) < size.x)
			{
				if ((Location.y - 1) >= 0)
				{
					newLocation.x = Location.x + 1;
					newLocation.y = Location.y - 1;
				}
				else
				{
					newLocation.x = -1;
					newLocation.y = -1;
				}
			}
			else
				{
					newLocation.x = -1;
					newLocation.y = -1;
				}
				break;
		}
	case 3:
		{
			//move diagonal down
			if ((Location.x +1) < size.x)
			{
				if ((Location.y + 1) < size.y)
				{
					newLocation.x = Location.x + 1;
					newLocation.y = Location.y + 1;
				}
				else
				{
					newLocation.x = -1;
					newLocation.y = -1;
				}
			}
			else
				{
					newLocation.x = -1;
					newLocation.y = -1;
				}
				break;
		
		}

		

	}
	return newLocation;
}
int CDIAGreyScale::GetRun(int &lineNo, int &columnNo, int direction)
{
	//direction will be h = 0, v = 1, du = 2,dd = 3
	int run = 0;
	bool flag = false; //this wil change when finishing the black run//from white to black
	int pixel = 255;
	CPoint size(GetWidth(),GetHeight());
	CPoint point(columnNo, lineNo);
	while ((!flag) && (point.x >= 0) && (point.y >= 0))
	{
		pixel = (int) GetRValue(GetRGBColor(point.x,point.y));
		if (pixel == 0) //black run
		{
			run++;
			point = MoveinDirection(direction,size,point);

		}else 
		{
			if (run > 0) //end of run
			{
				flag = true;
				
			}else
			{
				//if run is still 0 then just move in the direction
				point = MoveinDirection(direction,size,point);
			}
		}
	}
	if ((point.x >=0) && (point.y >= 0))
	{
		lineNo = point.y;
		columnNo = point.x;
	}
	else
	{
		lineNo = size.y;
		columnNo = size.x;

	}
	return run;
}
//bool IsInRange(int number, int start, int end)
//{
//	return ((number >= start) && (number <= end) && (start < end))
//}
//void CDIAGreyScale::ApplyAdvancedTreshold(CDIAImage *image, RANGE_VALS* range, int size)
//{
//	
//	
//		for (int i = 0; i < GetWidth(); i++)
//		{
//			for (int j = 0; j < GetHeight(); i++)
//			{
//				int pixel = (int)GetRValue(GetRGBColor(i,j));
//				for (int x = 0; x < size; x++)
//				{
//					if (IsInRange(pixel,range[x].firstVal,range[x].lastVal))
//					{
//						SetRed(i,j,range[x].colour);
//					}
//				}
//
//
//			}
//		}
//	
//}


void CDIAGreyScale::Threshold_Bernsen(int WindowSize, CRect& rect, CImage * pOutputImage, BYTE Flags)
{
	///////
	//Threshold Bernsen
}