#include "StdAfx.h"
#include "DocImAn.h"
#include "DIAGreyScale.h"
#include "Image.h"
#include "DocImAnDoc.h"
#include "stdlib.h"
#include "DIACharacter.h"
#include ".\diahistograms.h"

CDIAHistograms::CDIAHistograms(void)
{

	//Initialization of inHistogram 
	//for (int x = 0; x <= 255; x++) m_inHistogram[x] = 0;
	//this is faster than the for...
	m_inHistogram = new unsigned long[256];
	m_outHistogram = new unsigned long[256];
	m_transformation = new unsigned long[256];
	m_LUT = new unsigned long[256];
	ZeroMemory(m_inHistogram, 256 * sizeof(unsigned long));

	//Initialization of inHistogram 
	ZeroMemory(m_outHistogram, 256 * sizeof(unsigned long));
	
	//Initialization of transformation 
	ZeroMemory(m_transformation, 256 * sizeof(unsigned long));

	//Initialization of LUT
	ZeroMemory(m_LUT, 256 * sizeof(unsigned long));
	
	m_maxvalueInHisto = 0;
	m_maxvalueOutHisto = 0;
	m_maxvalueLUT = 0;
	//m_showsigma = FALSE;
	//m_showouthisto = FALSE;
//	m_showoutimage = FALSE;

}

CDIAHistograms::~CDIAHistograms(void)
{
	delete m_inHistogram; 
	delete m_outHistogram;
	delete[] m_transformation;
	delete[] m_LUT;
}
void CDIAHistograms::ComputeHistogram(unsigned long *inHisto,unsigned long *maxval, CImage *image)
{
	//This computes the inHistogram from the original image 
	if (m_pdoc->GetActiveImage() != NULL)
	{
		int width = image->GetWidth();
		int height = image->GetHeight();
		
		//creation of inHistogram
		for (int y = 0; y < height; y++)
		{
			for (int x = 0; x < width; x++)
			{
				BYTE red = GetRValue(image->GetRGBColor(x,y));
				
				inHisto[red]++;
				if(inHisto[red]> *maxval) *maxval = inHisto[red];
			}
		}	
	}
}

void CDIAHistograms::ComputeHistogram(unsigned long *inHisto,unsigned long *maxval, CDIAGreyScale *image)
{
	//This computes the inHistogram from the original image 

	int width = image->GetWidth();
	int height = image->GetHeight();
	
	//creation of inHistogram
	for (int y = 0; y < height; y++)
	{
		for (int x = 0; x < width; x++)
		{
			BYTE red = GetRValue(image->GetRGBColor(x,y));
			
			inHisto[red]++;
			if(inHisto[red]> *maxval) *maxval = inHisto[red];
		}
	}	
}
//compute histogram for character image
void CDIAHistograms::ComputeHistogram(unsigned long *inHisto,unsigned long *maxval, CRect rect, CImage *Image)
{
	//This computes the inHistogram from the original image 

	
	
	//creation of inHistogram
	for (int x = rect.left; x < rect.right; x++)
	{
		for (int y = rect.top; y < rect.bottom; y++)
		{
			int red = (int)GetRValue(Image->GetRGBColor(x,y));
			inHisto[red]++;
		//	if(inHisto[red]> *maxval) *maxval = inHisto[red];
			
		}
	}	
}

void CDIAHistograms::ComputeTransformation(CImage * pImage, CImage * pOrigImage)
{
	if (m_pdoc == NULL) return;
	//CImage * pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);
	//CImage * pImage = GetImage();
	int width = pImage->GetWidth();
	int height = pImage->GetHeight();

	for (int y = 0; y < height; y++)
	{
		for (int x = 0; x < width; x++)
		{
			//any R,G,B for a greyscale
			
			BYTE red = GetRValue(pOrigImage->GetRGBColor(x,y));
			BYTE grey = (BYTE)m_transformation[red];
			pImage->SetRGBColor(x,y,RGB(grey,grey,grey));
			
		}
	}
	pImage->SetDirty();
}
void CDIAHistograms::ComputeTransformation(CImage * pImage, CImage * pOrigImage, CRect rect)
{
	if (m_pdoc == NULL) return;
	//CImage * pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);
	//CImage * pImage = GetImage();
	

	for (int x = rect.left; x < rect.right; x++)
	{
		for (int y = rect.top; y < rect.bottom; y++)
		{
			//any R,G,B for a greyscale
			
			BYTE red = GetRValue(pOrigImage->GetRGBColor(x,y));
			BYTE grey = (BYTE)m_transformation[red];
			pImage->SetRGBColor(x,y,RGB(grey,grey,grey));
			
		}
	}
	pImage->SetDirty();
}

void CDIAHistograms::ComputeAdaptiveTransformation(CImage * pImage, CImage * pOrigImage, CRect rect)
{
	if (m_pdoc == NULL) return;
	//CImage * pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);
	//CImage * pImage = GetImage();
	

	for (int x = rect.left; x < rect.right; x++)
	{
		for (int y = rect.top; y < rect.bottom; y++)
		{
			//any R,G,B for a greyscale
			
			BYTE red = GetRValue(pOrigImage->GetRGBColor(x,y));
			BYTE grey = (BYTE)m_transformation[red];
			pImage->SetRGBColor(x,y,RGB(grey,grey,grey));
			
		}
	}
	pImage->SetDirty();
}
void CDIAHistograms::ComputeEqualization(CImage * pImage, CImage * pOrigImage)
{
	if (m_pdoc == NULL) return;
	//CImage * pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);
	//CImage * pImage = GetImage();

	int width = pImage->GetWidth();
	int height = pImage->GetHeight();

	//m_transformation.Create(width, height, RGB(255,255,255), _T("Output Image")); 

	// Calculate the lookup table. 
	m_LUT[0] = m_inHistogram[0];

	// accumulate 
	for (int x = 1; x < 256; x++)
	{
		m_LUT[x] = m_LUT[x-1] + m_inHistogram[x];
	}

	// Normalize. 
    for (int x = 0; x < 256; x++)
	{
		m_LUT[x] = (long)((m_LUT[x] * 255.0) / (double)(height * width));
		if(m_LUT[x]> m_maxvalueLUT) m_maxvalueLUT = m_LUT[x];
	}

	m_maxvalueOutHisto=0;


	// Output the equalized frame. 
	for (int y = 0; y < height; y++)
	{
		for (int x = 0; x < width; x++)
		{
			//any R,G,B for a greyscale
			
			BYTE red = GetRValue(pImage->GetRGBColor(x,y));
			BYTE grey = (BYTE)m_LUT[red];
			pImage->SetRGBColor(x,y,RGB(grey,grey,grey));
			m_outHistogram[grey]++;
			if (m_outHistogram[grey] > m_maxvalueOutHisto) m_maxvalueOutHisto = m_outHistogram[grey];
		}
	}
	pImage->SetDirty();
}
void CDIAHistograms::CalculateFunction(int TopPos, int DownPos)
{
	int x;
	int top, down;
	float solution;

	top = TopPos;//m_SliderTop.GetPos();
	down = DownPos;//m_SliderBottom.GetPos();
	
	CString Selection;
	
	m_sigma = -2.0;
	//m_sigma = -1.2;
	//m_ComboFunction.GetLBText(m_ComboFunction.GetCurSel(), Selection);
	//m_sigma = (float)_wtof(Selection);
	
	for (x = 0; x < down; x++) m_transformation[x] = 0;
	for (x = top+1; x<=255; x++) m_transformation[x] = 255;
	for (x = down; x <= top; x++) 
	{
		solution = float(x-down)/float(top - down);
		m_transformation[x] = (unsigned long) (Sigma(solution,m_sigma)* 255) ;
	}
}

void CDIAHistograms::CalculateFunctionContrast(unsigned long * Histogram, int AB, int BC, bool init)
{
	double Percent_A, Percent_B, Percent_C;
	long A = 0;
	long B = 0;
	long C = 0;
	int Top = 255;
	int Bottom = 0;
	for (int i = 0; i < 255; i++)
	{
		if (i <= AB)
		{
			A = A + Histogram[i];
		}
		else
		{
			if (i <= BC)
			{
				B = B + Histogram[i];
			}
			else 
			{
				C = C + Histogram[i];
			}
		}
	}
	double sum = A+B+C;
	Percent_A = ((double) A/sum)* 100;
	Percent_B = ((double) B/sum)* 100;
	Percent_C = ((double) C/sum)* 100;

	//compare percents
	if (Percent_A < 10)Bottom = 50;
	if (Percent_C < 60) Top = 200;
	if (Percent_B > 18)
	{
		Bottom += 10;
		Top -= 10;
	}
	if (init == true)CalculateFunction(Top,Bottom);
	else CalculateFunction(AB,BC);
	
	
}
int CDIAHistograms::GetMaximum()
{	
	int max = -1;

	for (int i = 255 ;i >= 0; i--)
	{
		if ((m_inHistogram[i] > 0) && ( max == -1))
		{
			max = i;
		}
	}
	return max;
}
int CDIAHistograms::GetMinimum()
{	
	int min = -1;

	for (int i = 0 ;i <= 255; i++)
	{
		if ((m_inHistogram[i] > 0) && ( min == -1))
		{
			min = i;
		}
	}
	return min;
}

void CDIAHistograms::AdaptiveStretchHisto(CImage * image, CRect rect, CImage * outImage)
{
	ComputeHistogram(m_inHistogram,&m_maxvalueInHisto,rect,image);
	CalculateFunction(GetMaximum(),GetMinimum());
	ComputeTransformation(outImage,image,rect);
}
void CDIAHistograms::StretchHisto(CImage * image, CRect rect, CImage * outImage)
{
	ComputeHistogram(m_inHistogram,&m_maxvalueInHisto,rect,image);
	CalculateFunction(0,255);
	ComputeTransformation(outImage,image,rect);
}