#include "StdAfx.h"
#include "Distribution.h"
#include "DIAHistograms.h"
#include ".\diasegmentation.h"

DIASegmentation::DIASegmentation(void)
{
	m_vert = FALSE;
	m_horz = FALSE;
	m_sepaV = FALSE;
	m_sepaH = FALSE;
	m_complete = FALSE;
	m_pHorzProfileDistribution = NULL;
	m_pVertProfileDistribution = NULL;
	m_pSeparatorsH = NULL;
	m_pSeparatorsV = NULL;
	m_pVertProj = NULL;
	m_pHorzProj = NULL;
	apply_separatorsV = FALSE;
	apply_separatorsH = FALSE;
	m_pStrongSeparators = NULL;
	m_pChart = NULL;

	
}

DIASegmentation::~DIASegmentation(void)
{
	if (m_pVertProj != NULL)
	{
		delete[] m_pVertProj;
		m_pVertProj = NULL;
	}
	if (m_pHorzProj != NULL)
	{
		delete[] m_pHorzProj;
		m_pHorzProj = NULL;
	}
	if (m_pHorzProfileDistribution != NULL)
	{
		delete[] m_pHorzProfileDistribution;
		m_pHorzProfileDistribution = NULL;
	}
	if (m_pVertProfileDistribution != NULL)
	{
		delete[] m_pVertProfileDistribution;
		m_pVertProfileDistribution = NULL;
	}
	if (m_pSeparatorsV != NULL)
	{
		delete[] m_pSeparatorsV;
		m_pSeparatorsV = NULL;
	}
	
	if (m_pSeparatorsH != NULL)
	{
		delete[] m_pSeparatorsH;
		m_pSeparatorsH = NULL;
	}
	if (m_pStrongSeparators != NULL)
	{
		delete[] m_pStrongSeparators;
		m_pStrongSeparators = NULL;
	}
	if (m_pChart != NULL)
	{
		delete[] m_pChart;
		m_pChart = NULL;
	}
	
	
	
}

long DIASegmentation::CalculateMaxVal(long * Histo, int length)
{
	long x = 0;

	for (int i = 0; i < length; i++)
	{
		if (Histo[i]>x) x= Histo[i];
	}
	return x;
}

void DIASegmentation::FindVerticalSeparators()
{
	CImage * pOrigImage;
	pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);
	
	//Creation of Distribution for Horz and vert
	//whatever the case, allways calculate first the vertical-projection separators
		m_sepaV = TRUE;
		
		if (!m_SeparatorsComputedV)
		{
			m_SeparatorsComputedV = TRUE;

			m_pVertProfileDistribution->RemoveSubbase();
			m_pVertProfileDistribution->ComputeExtrema();
	
			int * p_minArray;

			p_minArray = m_pVertProfileDistribution->GetMinimaPositions();
			long valueR,valueL;
			int posR, posL;
			long heightDistance;
			int widthDistance;

			//widthDistance and heightDistance are the thresholds
			//may vary from horz to vert profiles
			widthDistance = 5;
			heightDistance = 4500;

			for (int i = 0; i < m_pVertProfileDistribution->GetMinimaCount(); i++)
			{
				posR = m_pVertProfileDistribution->GetLocalMaximumPos(p_minArray[i],GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
				valueR = m_pVertProfileDistribution->GetAt(posR);
				posL = m_pVertProfileDistribution->GetLocalMaximumPos(p_minArray[i],GLM_SD_LEFT,GLM_OUTOFBOUNDS_VALUES);
				valueL = m_pVertProfileDistribution->GetAt(posL);
				if ((abs(p_minArray[i]-posR)>= widthDistance) || (abs(m_pVertProfileDistribution->GetAt(p_minArray[i])-valueR)>= heightDistance) || (abs(p_minArray[i]-posL)>= widthDistance) || (abs(m_pVertProfileDistribution->GetAt(p_minArray[i])-valueL)>= heightDistance))
					{
						m_pSeparatorsV[m_separatorIndexV].Value = p_minArray[i];
						m_separatorIndexV++;
					}
			}
			SelectionateStrongSeparators7V(m_pSeparatorsV,m_separatorIndexV, m_pVertProfileDistribution);
		}

}
void DIASegmentation::SelectionateStrongSeparators7H(CTextLine::SEPARATOR * p_separator,int separatorIndex, CDistribution <long> * p_ProfileDistribution)
{

	//Horizontal Version, character segmentation
	//first find FirstSeparator
	//Find also the lowest and the highest minimum  to compare from that
	int LoMin,HiMin;
	int i = 0;
	int i2 = i + 1;
	int maxi;
	int MinPercent = 14;
	int MaxPercent = 45;
	int MinRange = 18;
	int MaxRange = 34;
	int HeightThreshold;
	int FirstSeparator = p_separator[i].Value;
	bool flag = false;
	LoMin = (p_ProfileDistribution->GetAt(p_separator[1].Value));
	HiMin = 0;
	for ( int j = 1; j < separatorIndex-2; j++)
	{
		if ((p_ProfileDistribution->GetAt(p_separator[j].Value) < LoMin)&&( p_ProfileDistribution->GetAt(p_separator[j].Value)!= 0))
			LoMin = (p_ProfileDistribution->GetAt(p_separator[j].Value));
		if (p_ProfileDistribution->GetAt(p_separator[j].Value) > HiMin) 
			HiMin = (p_ProfileDistribution->GetAt(p_separator[j].Value));
	}
	HeightThreshold =(HiMin-LoMin)/3;
	//HeightThreshold = HiMin/3;
	//once found, compare with lowest min to see if it is low enough
	//can check as well is previous min if much closer to lowest min.
	maxi = p_ProfileDistribution->GetLocalMaximumPos(FirstSeparator,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);

	while ((p_ProfileDistribution->GetAt(maxi)- p_ProfileDistribution->GetAt(FirstSeparator) < HeightThreshold) && (i < separatorIndex))
	{
		i++;
		FirstSeparator = p_separator[i].Value;
		maxi = p_ProfileDistribution->GetLocalMaximumPos(FirstSeparator,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
	}
	if (i < separatorIndex)
	{
		//if the founded separator is higher than a 14% from the LoMin, then look for previous
		if ((((p_ProfileDistribution->GetAt(FirstSeparator)-LoMin) * 100)/HiMin) > MinPercent)
		{
			while (((((p_ProfileDistribution->GetAt(FirstSeparator)-LoMin) * 100)/HiMin) > MinPercent)&& (i > 0))
			{
				i--;
				FirstSeparator = p_separator[i].Value;
			}
		}	
	}
	//now we have the FirstSeparator we can start finding the rest from it
	p_separator[i].Strong = TRUE;
	i2 = i+1;
	flag = true;
	while (i2 < separatorIndex-1)
	{
			while (((p_separator[i2].Value - p_separator[i].Value) <= MaxRange) && (i2 < separatorIndex))
			{
				i2++;
				flag = false;
			}
			if (!flag)
			{
				//if there is a value within the limits 28-34
				int LocalLowMin = (LoMin + p_ProfileDistribution->GetAt(p_separator[i].Value))/2; 
				if (((p_separator[i2-1].Value - p_separator[i].Value) <= MaxRange) && ((p_separator[i2-1].Value - p_separator[i].Value) >= MinRange))
				{
					//find if it is down the valley	
					int a = p_ProfileDistribution->GetAt(p_separator[i2-2].Value);
					int b = p_ProfileDistribution->GetAt(p_separator[i2-1].Value);
					int c = p_ProfileDistribution->GetAt(p_separator[i2].Value);
					int d = p_ProfileDistribution->GetAt(p_separator[i2+1].Value);
					int e = p_ProfileDistribution->GetAt(p_separator[i].Value);
					int LocalLowMin = (LoMin + p_ProfileDistribution->GetAt(p_separator[i].Value))/2; 

					if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) > MinPercent)
					{
						//compare to previous and next minimum in case it has a lower value
						if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent) 
						{
							p_separator[i2].Strong = TRUE;
							i = i2;
							i2++;
							flag = true;
						}
						else
						{
							if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent) 
							{
								p_separator[i2-2].Strong = TRUE;
								flag = true;
								if (i2-2 == i)
								{
									i = i2-1;
								}
								else
								{
									i = i2-2;
									i2--;
								}
								
							}
							else
							{
								if (i2+1 < separatorIndex)
								{
									if ((((p_ProfileDistribution->GetAt(p_separator[i2+1].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
									{
										p_separator[i2+1].Strong = TRUE;
										flag = true;
										i = i2+1;
										i2=i+2;
									}
								}
							}
							//if p_separator[i2-1] is the lowest value from i2 and i2+1 we set it to strong
							if (!flag)
							{
								if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2].Value))
								{
									if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
									{
										if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
										{
											p_separator[i2-1].Strong = TRUE;
											flag = true;
											i = i2-1;
										}
									}
									else 
									{
										if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
										{
											p_separator[i2-2].Strong = TRUE;
											flag = true;
											if (i2-2 == i)
											{
												i = i2-1;
											}
											else
											{
												i = i2-2;
												i2--;
											}
										}
									}
								}
								else
									{
										if (p_ProfileDistribution->GetAt(p_separator[i2].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
										{
											if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
											{
												p_separator[i2].Strong = TRUE;
												flag = true;
												i = i2;
												i2++;
											}
										}
										else 
										{
											if (p_ProfileDistribution->GetAt(p_separator[i2-2].Value) <= p_ProfileDistribution->GetAt(p_separator[i2+1].Value))
											{
												if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent)
												{
													p_separator[i2-2].Strong = TRUE;
													flag = true;
													if (i2-2 == i)
													{
														i = i2-1;
													}
													else
													{
														i = i2-2;
														i2--;
													}
												}
												
											}
											else
												{
													p_separator[i2+1].Strong = TRUE;
													flag = true;
													i = i2+1;
													i2=i+2;
												}
										
										}
									}
								}
							}
					}
					else
					{
						p_separator[i2-1].Strong = TRUE;
						flag = true;
						i = i2-1;
					}
				}
				else
				{
					//if is smaller than 28 or larger than 34
					if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
						{
							p_separator[i2-1].Strong = TRUE;
							flag = true;
							if (i == i2-1)
							{
								i = i2;
								i2= i2+1;
							}
							else
							{
								i = i2-1;
							}
						}
						else
						{
							if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
							{
								p_separator[i2].Strong = TRUE;
								flag = true;
								i = i2;
								i2++;
							}
							else
							{
								if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
								{
									p_separator[i2-2].Strong = TRUE;
									flag = true;
									if (i2-2 == i)
									{
										i = i2-1;
									}
									else
									{
										i = i2-2;
										i2--;
									}
								}
								else
								{
									if (i2+1 < separatorIndex)
									{
											p_separator[i2+1].Strong = TRUE;
											flag = true;
											i = i2+1;
											i2=+2;
										
									}
									else
									{
										
										//try to find the smallest of all and set it to strong
									}

								}
							}
						}
				}
			}else //this happens when none of the other cases 
				  //fit so the separator is set by default to the smaller value within maxrange
				{
					////test default value
					//	p_separator[i2-1].Strong = TRUE;
					//	
					//	i = i2;
					//	i2++;
					//	flag = true;

					
					long minimum = min(p_ProfileDistribution->GetAt(p_separator[i2-1].Value),p_ProfileDistribution->GetAt(p_separator[i2-2].Value));
					long minimum2 = min(p_ProfileDistribution->GetAt(p_separator[i2].Value),p_ProfileDistribution->GetAt(p_separator[i2+1].Value));
					long minimum3 = min(minimum,minimum2);
					if (minimum3 == (long)(p_ProfileDistribution->GetAt(p_separator[i2-1].Value)))
					{
					//case p_ProfileDistribution->GetAt(p_separator[i2-1].Value):
						p_separator[i2-1].Strong = TRUE;
						flag = true;
						if (i == i2-1)
							{
								i = i2;
								i2= i2+1;
							}
							else
							{
								i = i2-1;
							}
						//break;
					}else
						if (minimum3 == (long)(p_ProfileDistribution->GetAt(p_separator[i2].Value)))
						{
					//case p_ProfileDistribution->GetAt(p_separator[i2].Value):
						p_separator[i2].Strong = TRUE;
						flag = true;
						i = i2;
						i2++;
					//	break;
						}
						else
							if (minimum3 == (long)(p_ProfileDistribution->GetAt(p_separator[i2+1].Value)))
							{
					//case p_ProfileDistribution->GetAt(p_separator[i2+1].Value):
						p_separator[i2+1].Strong = TRUE;
						flag = true;
						i = i2+1;
						i2=+2;
						//break;
							}else
							{
					//case p_ProfileDistribution->GetAt(p_separator[i2-2].Value):
						p_separator[i2-2].Strong = TRUE;
						flag = true;
						if (i2-2 == i)
						{
							i = i2-1;
						}
						else
						{
							i = i2-2;
							i2--;
						}
						//break;
							}
				

					
				
				}
			//if at this point it has not go into any posibility then default case
			if (!flag)
			{
			/*	p_separator[i2-1].Strong = TRUE;
				i = i2;
				i2++;
				flag = true;*/
			
				long minimum = min(p_ProfileDistribution->GetAt(p_separator[i2-1].Value),p_ProfileDistribution->GetAt(p_separator[i2-2].Value));
					long minimum2 = min(p_ProfileDistribution->GetAt(p_separator[i2].Value),p_ProfileDistribution->GetAt(p_separator[i2+1].Value));
					long minimum3 = min(minimum,minimum2);
					if (minimum3 == p_ProfileDistribution->GetAt(p_separator[i2-1].Value))
					{
					//case p_ProfileDistribution->GetAt(p_separator[i2-1].Value):
						p_separator[i2-1].Strong = TRUE;
						flag = true;
						i = i2-1;
						//break;
					}else
						if (minimum3 == p_ProfileDistribution->GetAt(p_separator[i2].Value))
						{
					//case p_ProfileDistribution->GetAt(p_separator[i2].Value):
						p_separator[i2].Strong = TRUE;
						flag = true;
						i = i2;
						i2++;
					//	break;
						}
						else
							if (minimum3 == p_ProfileDistribution->GetAt(p_separator[i2+1].Value))
							{
					//case p_ProfileDistribution->GetAt(p_separator[i2+1].Value):
						p_separator[i2+1].Strong = TRUE;
						flag = true;
						i = i2+1;
						i2=+2;
						//break;
							}else
							{
					//case p_ProfileDistribution->GetAt(p_separator[i2-2].Value):
						p_separator[i2-2].Strong = TRUE;
						flag = true;
						if (i2-2 == i)
						{
							i = i2-1;
						}
						else
						{
							i = i2-2;
							i2--;
						}
						//break;
							}
			}
	}
			
}
//void DIASegmentation::SelectionateStrongSeparators7H(CTextLine::SEPARATOR * p_separator,int separatorIndex, CDistribution <long> * p_ProfileDistribution)
//{
//
//	//Horizontal Version, character segmentation
//	//first find FirstSeparator
//	//Find also the lowest and the highest minimum  to compare from that
//	int LoMin,HiMin;
//	int i = 0;
//	int i2 = i + 1;
//	int maxi;
//	int MinPercent = 14;
//	int MaxPercent = 45;
//	int MinRange = 18;
//	int MaxRange = 34;
//	int HeightThreshold;
//	int FirstSeparator = p_separator[i].Value;
//	bool flag = false;
//	LoMin = (p_ProfileDistribution->GetAt(p_separator[1].Value));
//	HiMin = 0;
//	for ( int j = 1; j < separatorIndex-2; j++)
//	{
//		if ((p_ProfileDistribution->GetAt(p_separator[j].Value) < LoMin)&&( p_ProfileDistribution->GetAt(p_separator[j].Value)!= 0))
//			LoMin = (p_ProfileDistribution->GetAt(p_separator[j].Value));
//		if (p_ProfileDistribution->GetAt(p_separator[j].Value) > HiMin) 
//			HiMin = (p_ProfileDistribution->GetAt(p_separator[j].Value));
//	}
//	HeightThreshold =(HiMin-LoMin)/3;
//	//HeightThreshold = HiMin/3;
//	//once found, compare with lowest min to see if it is low enough
//	//can check as well is previous min if much closer to lowest min.
//	maxi = p_ProfileDistribution->GetLocalMaximumPos(FirstSeparator,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
//
//	while ((p_ProfileDistribution->GetAt(maxi)- p_ProfileDistribution->GetAt(FirstSeparator) < HeightThreshold) && (i < separatorIndex))
//	{
//		i++;
//		FirstSeparator = p_separator[i].Value;
//		maxi = p_ProfileDistribution->GetLocalMaximumPos(FirstSeparator,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
//	}
//	if (i < separatorIndex)
//	{
//		//if the founded separator is higher than a 14% from the LoMin, then look for previous
//		if ((((p_ProfileDistribution->GetAt(FirstSeparator)-LoMin) * 100)/HiMin) > MinPercent)
//		{
//			while (((((p_ProfileDistribution->GetAt(FirstSeparator)-LoMin) * 100)/HiMin) > MinPercent)&& (i > 0))
//			{
//				i--;
//				FirstSeparator = p_separator[i].Value;
//			}
//		}	
//	}
//	//now we have the FirstSeparator we can start finding the rest from it
//	p_separator[i].Strong = TRUE;
//	i2 = i+1;
//	flag = true;
//	while (i2 < separatorIndex-1)
//	{
//			while (((p_separator[i2].Value - p_separator[i].Value) <= MaxRange) && (i2 < separatorIndex))
//			{
//				i2++;
//				flag = false;
//			}
//			if (!flag)
//			{
//				//if there is a value within the limits 28-34
//				int LocalLowMin = (LoMin + p_ProfileDistribution->GetAt(p_separator[i].Value))/2; 
//				if (((p_separator[i2-1].Value - p_separator[i].Value) <= MaxRange) && ((p_separator[i2-1].Value - p_separator[i].Value) >= MinRange))
//				{
//					//find if it is down the valley	
//					int a = p_ProfileDistribution->GetAt(p_separator[i2-2].Value);
//					int b = p_ProfileDistribution->GetAt(p_separator[i2-1].Value);
//					int c = p_ProfileDistribution->GetAt(p_separator[i2].Value);
//					int d = p_ProfileDistribution->GetAt(p_separator[i2+1].Value);
//					int e = p_ProfileDistribution->GetAt(p_separator[i].Value);
//					int LocalLowMin = (LoMin + p_ProfileDistribution->GetAt(p_separator[i].Value))/2; 
//
//					if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) > MinPercent)
//					{
//						//compare to previous and next minimum in case it has a lower value
//						if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent) 
//						{
//							p_separator[i2].Strong = TRUE;
//							i = i2;
//							i2++;
//							flag = true;
//						}
//						else
//						{
//							if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent) 
//							{
//								p_separator[i2-2].Strong = TRUE;
//								flag = true;
//								if (i2-2 == i)
//								{
//									i = i2-1;
//								}
//								else
//								{
//									i = i2-2;
//									i2--;
//								}
//								
//							}
//							else
//							{
//								if (i2+1 < separatorIndex)
//								{
//									if ((((p_ProfileDistribution->GetAt(p_separator[i2+1].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
//									{
//										p_separator[i2+1].Strong = TRUE;
//										flag = true;
//										i = i2+1;
//										i2=i+2;
//									}
//								}
//							}
//							//if p_separator[i2-1] is the lowest value from i2 and i2+1 we set it to strong
//							if (!flag)
//							{
//								if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2].Value))
//								{
//									if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
//									{
//										if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
//										{
//											p_separator[i2-1].Strong = TRUE;
//											flag = true;
//											i = i2-1;
//										}
//									}
//									else 
//									{
//										if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
//										{
//											p_separator[i2-2].Strong = TRUE;
//											flag = true;
//											if (i2-2 == i)
//											{
//												i = i2-1;
//											}
//											else
//											{
//												i = i2-2;
//												i2--;
//											}
//										}
//									}
//								}
//								else
//									{
//										if (p_ProfileDistribution->GetAt(p_separator[i2].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
//										{
//											if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent) 
//											{
//												p_separator[i2].Strong = TRUE;
//												flag = true;
//												i = i2;
//												i2++;
//											}
//										}
//										else 
//										{
//											if (p_ProfileDistribution->GetAt(p_separator[i2-2].Value) <= p_ProfileDistribution->GetAt(p_separator[i2+1].Value))
//											{
//												if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MaxPercent)
//												{
//													p_separator[i2-2].Strong = TRUE;
//													flag = true;
//													if (i2-2 == i)
//													{
//														i = i2-1;
//													}
//													else
//													{
//														i = i2-2;
//														i2--;
//													}
//												}
//												
//											}
//											else
//												{
//													p_separator[i2+1].Strong = TRUE;
//													flag = true;
//													i = i2+1;
//													i2=i+2;
//												}
//										
//										}
//									}
//								}
//							}
//					}
//					else
//					{
//						p_separator[i2-1].Strong = TRUE;
//						flag = true;
//						i = i2-1;
//					}
//				}
//				else
//				{
//					//if is smaller than 28 or larger than 34
//					if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
//						{
//							p_separator[i2-1].Strong = TRUE;
//							flag = true;
//							if (i == i2-1)
//							{
//								i = i2;
//								i2= i2+1;
//							}
//							else
//							{
//								i = i2-1;
//							}
//						}
//						else
//						{
//							if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
//							{
//								p_separator[i2].Strong = TRUE;
//								flag = true;
//								i = i2;
//								i2++;
//							}
//							else
//							{
//								if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LocalLowMin) * 100)/HiMin) < MinPercent)
//								{
//									p_separator[i2-2].Strong = TRUE;
//									flag = true;
//									if (i2-2 == i)
//									{
//										i = i2-1;
//									}
//									else
//									{
//										i = i2-2;
//										i2--;
//									}
//								}
//								else
//								{
//									if (i2+1 < separatorIndex)
//									{
//											p_separator[i2+1].Strong = TRUE;
//											flag = true;
//											i = i2+1;
//											i2=+2;
//										
//									}
//									else
//									{
//										
//										//try to find the smallest of all and set it to strong
//									}
//
//								}
//							}
//						}
//				}
//			}else //this happens when none of the other cases 
//				  //fit so the separator is set by default to the smaller value within maxrange
//				{
//					//test default value
//						p_separator[i2-1].Strong = TRUE;
//						
//						i = i2;
//						i2++;
//						flag = true;
//				}
//			//if at this point it has not go into any posibility then default case
//			if (!flag)
//			{
//				p_separator[i2-1].Strong = TRUE;
//				i = i2;
//				i2++;
//				flag = true;
//			
//			}
//	}
//			
//}
void DIASegmentation::SelectionateStrongSeparators8V(CTextLine::SEPARATOR * p_separator,int separatorIndex, CDistribution <long> * p_ProfileDistribution)
{
	//Vertical Version, Line Segmentation
	//First find first separator
	double LoMin,HiMin;
	//double HeightThreshold;
	int MinPercent = 10;
	int MaxPercent = 45;
	int MinRange = 40;
	int MaxRange = 55;
	//int maxi;
	int i = 0;
	int i2 = i + 1;
//	int x;
	int FirstSeparator = p_separator[i].Value;
	int NextSeparator;
	LoMin = (p_ProfileDistribution->GetMinimum());//modify to avoid 0 and 1 values
	HiMin = (p_ProfileDistribution->GetMaximum()); //probably the first or last positions due to cutting.
	//avoid 0 values (sometimes aperad at begining or end)
	/*if (FirstSeparator == 0) 
	{
		i++;
		FirstSeparator = p_separator[i].Value;
	}*/
	NextSeparator = p_separator [i+1].Value;
	////find maximum between FirstSeparator and NextSeparator
	//maxi = FindMinMaxBetweenTwo(FirstSeparator,NextSeparator,p_ProfileDistribution,1);
	//while ((maxi/p_ProfileDistribution->GetAt(FirstSeparator)) > MinPercent)
	//{
	//	i++;
	//	FirstSeparator = p_separator[i].Value;
	//	NextSeparator = p_separator [i+1].Value;
	//	maxi = FindMinMaxBetweenTwo(FirstSeparator,NextSeparator,p_ProfileDistribution,1);
	//}
	//Find diference for more than 10%
	while ( (((double)NextSeparator/(double)FirstSeparator ) *100 < 10)&& (i+1 < separatorIndex))
	{
		i++;
		FirstSeparator = p_separator[i].Value;
		NextSeparator = p_separator [i+1].Value;
	}
	if (((double)NextSeparator/(double)FirstSeparator ) *100 > 10)
	{
		p_separator[i].Strong = true;
		//now we have a firstSeparator that is 10% smaller than the next 
		
	}
	i2 = i+1;
	//to find next first find smallest value within range
	//if within minpercent then is strong 
	//otherwise find smallest within large range
	//if within minpercent then is strong
	//otherwise 
	//while (i2 < separatorIndex-1)
	//{
	//	//first find if there is a value within minrange and minpercentage
	//	x = FindIfValueIsInRange(i,p_separator,separatorIndex,MinRange,MaxRange,MinPercent,LoMin,HiMin);
	//	if (x == -1)
	//	{
	//		//find if good for maxpercent
	//		x = FindIfValueIsInRange(i,p_separator,separatorIndex,MinRange,MaxRange,MaxPercent,LoMin,HiMin);
	//		if (x == -1)
	//		{
	//			//not even within maxpercent

	//		}
	//	}
	//	else
	//	{
	//		//set p_separator[x] to strong and find next
	//		p_separator[x].Strong = true;
	//		if (x+1 <= separatorIndex)
	//		{
	//			i = x;
	//			i2 = x+1;
	//		}
	//	}

	//}

}
int DIASegmentation::FindIfValueIsInRange(int first,CTextLine::SEPARATOR * separators,int nseparators,int minrange,int maxrange, int percent,double lomin, double himin)
{

	int pos;
	int value;
	int second = first +1;
	bool flag = false;//indicates that at least there is one value within range
	while (((separators[second].Value - separators[first].Value) <= minrange) && (second < nseparators-1))
	{
		second++;
	}
	//find if values between minrange and maxrange are within percent
	value = separators[second].Value;
	pos = second;
	while (((separators[second].Value - separators[first].Value) <= maxrange) && (second < nseparators-1))
	{
		if ((((separators[second].Value-lomin) * 100)/himin) < percent)
		{
			if (separators[second].Value <= value)//just get the smallest value if more than one
			{
				value = separators[second].Value;
				pos = second;
				flag = true;
			}
		}
		second++;
	}

	if (flag == false)
	{
		if (second == nseparators-1)
		{
			return second;
		}
		else
		{
			return -1;
		}
	}
	else
	{
		return pos;
	}
	

}
void DIASegmentation::SelectionateStrongSeparators7V(CTextLine::SEPARATOR * p_separator,int separatorIndex, CDistribution <long> * p_ProfileDistribution)
{

	//Vertical Version, Line segmentation
	//first find FirstSeparator
	//Find also the lowest and the highest minimum  to compare from that
	double LoMin,HiMin;
	int i = 0;
	int i2 = i + 1;
//	int maxi;
//	double HeightThreshold;
	double MinPercent = 15;
	int Maxpercent = 45;
	int MinRange = 40;
	int MaxRange = 55;
	int FirstSeparator = p_separator[i].Value;
	int NextSeparator = p_separator [i+1].Value;
	//long maxval,minval;
	//bool flag = 0;
	LoMin = (p_ProfileDistribution->GetMinimum());//GetAt(p_separator[1].Value));
	HiMin = (p_ProfileDistribution->GetMaximum());//0;
	NextSeparator = p_separator [i+1].Value;
	
	//Find diference for more than 10%
	while ( (((double)NextSeparator/(double)FirstSeparator ) *100 < 10)&& (i+1 < separatorIndex))
	{
		i++;
		FirstSeparator = p_separator[i].Value;
		NextSeparator = p_separator [i+1].Value;
	}
	if (((double)NextSeparator/(double)FirstSeparator ) *100 > 10)
	{
		p_separator[i].Strong = true;
		//now we have a firstSeparator that is 10% smaller than the next 
	}
	i2 = i+1;
	//we have the FirstSeparator we can start finding the rest from it
	
	while (i2 < separatorIndex-1)
	{
			while ((p_separator[i2].Value - p_separator[i].Value <= MaxRange) && (i2 < separatorIndex))
			{
				i2++;
			}
			//if there is a value within the limits 28-34
			if ((p_separator[i2-1].Value - p_separator[i].Value <= MaxRange) && (p_separator[i2-1].Value - p_separator[i].Value >= MinRange))
			{
				//find if it is down the valley
				int a = p_ProfileDistribution->GetAt(p_separator[i2-1].Value);
				int b = p_ProfileDistribution->GetAt(p_separator[i2].Value);
				int c = p_ProfileDistribution->GetAt(p_separator[i2+1].Value);
				int d = p_ProfileDistribution->GetAt(p_separator[i2-2].Value);

				if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LoMin) * 100)/HiMin) > MinPercent)
				{
					//compare to previous and next minimum in case it has a lower value
					if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LoMin) * 100)/HiMin) < MinPercent)
					{
						p_separator[i2].Strong = TRUE;
						i = i2;
						i2++;
					}
					else
					{
						if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LoMin) * 100)/HiMin) < MinPercent)
						{
							p_separator[i2-2].Strong = TRUE;
							i = i2-2;
							i2--;
						}
						else
						{
							if (i2+1 < separatorIndex)
							{
								if ((((p_ProfileDistribution->GetAt(p_separator[i2+1].Value)-LoMin) * 100)/HiMin) < MinPercent)
								{
									p_separator[i2+1].Strong = TRUE;
									i = i2+1;
									i2=i+2;
								}
							}
						//if p_separator[i2-1] is the lowest value from i2 and i2+1 we set it to strong
						if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2].Value))
						{
							if (p_ProfileDistribution->GetAt(p_separator[i2-1].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
							{
								if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LoMin) * 100)/HiMin) < Maxpercent)
								{
									p_separator[i2-1].Strong = TRUE;
									i = i2-1;
								}
							}
							else 
							{
								if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LoMin) * 100)/HiMin) < Maxpercent)
								{
									p_separator[i2-2].Strong = TRUE;
									i = i2-2;
									i2--;
								}
							}
						}
						else
							{
								if (p_ProfileDistribution->GetAt(p_separator[i2].Value)<= p_ProfileDistribution->GetAt(p_separator[i2-2].Value))
								{
									if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LoMin) * 100)/HiMin) < Maxpercent)
									{
										p_separator[i2].Strong = TRUE;
										i = i2;
										i2++;
									}
								}
								else if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LoMin) * 100)/HiMin) < Maxpercent)
								{
									p_separator[i2-2].Strong = TRUE;
									i = i2-2;
									i2--;
								}
							}
						}
					}	
				}
				else
				{
					p_separator[i2-1].Strong = TRUE;
					i = i2-1;
				}
			}
			else
			{
				//if is smaller than 28 or larger than 34
				if ((((p_ProfileDistribution->GetAt(p_separator[i2-1].Value)-LoMin) * 100)/HiMin) < MinPercent)
					{
						p_separator[i2-1].Strong = TRUE;
						i = i2-1;
						i2++;
					}
					else
					{
						if ((((p_ProfileDistribution->GetAt(p_separator[i2].Value)-LoMin) * 100)/HiMin) < MinPercent)
						{
							p_separator[i2].Strong = TRUE;
							i = i2;
							i2++;
						}
						else
						{
							if ((((p_ProfileDistribution->GetAt(p_separator[i2-2].Value)-LoMin) * 100)/HiMin) < MinPercent)
							{
								p_separator[i2-2].Strong = TRUE;
								if ((i2 - 2) == i)
								{
									i = i2-1;
								}
								else
								{
									i = i2-2;
									i2--;
								}
							}
							else
							{
								if (i2+1 < separatorIndex)
								{
									p_separator[i2+1].Strong = TRUE;
									i = i2+1;
									i2=+2;
								}
							}
						}
					}
			}
	}
	if (p_separator[separatorIndex-1].Strong == FALSE) 
		p_separator[separatorIndex-1].Strong = TRUE;
			
}

void DIASegmentation::GetStrongSeparators(CTextLine::SEPARATOR * p_separator,int separatorIndex)
{
	
	int res = 0;
	m_StrongSeparatorIndex = 0;
	
	for (int i = 0; i <= separatorIndex; i++)
	{
		if (p_separator[i].Strong == TRUE) res++;
	}
	m_pStrongSeparators = new long [res];
	for (int j = 0; j <= separatorIndex; j++)
	{
		if (p_separator[j].Strong == TRUE) 
		{
			m_pStrongSeparators[m_StrongSeparatorIndex] = p_separator[j].Value;
			m_StrongSeparatorIndex++;
		}
	}
}

void DIASegmentation::GetStrongSeparators(CTextLine::SEPARATOR * p_separator,int separatorIndex,long * &p_StrongSeparators, int &StrongSeparatorsIndex)
{
	
	int res = 0;
	StrongSeparatorsIndex = 0;
	
	for (int i = 0; i <= separatorIndex; i++)
	{
		if (p_separator[i].Strong == TRUE) res++;
	}
	p_StrongSeparators = new long [res];
	for (int j = 0; j <= separatorIndex; j++)
	{
		if (p_separator[j].Strong == TRUE) 
		{
			p_StrongSeparators[StrongSeparatorsIndex] = p_separator[j].Value;
			StrongSeparatorsIndex++;
		}
	}
}
void DIASegmentation::CreateHorizontalLines()
{
	int minLineWidth = 40;
	int smalllines = 0;
	double MinValueBGr = 1.9;
	CImage * pOrigImage;
	pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-2);

	//for every vertical strong separator and the next,
	//find out if there is distance enough for a line
	//find a maximum high enough and create a CTextLine if so.

	GetStrongSeparators(m_pSeparatorsV,m_separatorIndexV);
	if (m_pStrongSeparators != NULL)
	{
		m_pdoc->m_pLineHolder = new CTextLine [m_StrongSeparatorIndex - 1];
		//next: find out the distance between separators
		for (int j = 0; j < m_StrongSeparatorIndex-1; j++)
		{
			if (abs((m_pStrongSeparators[j]-m_pStrongSeparators[j+1]))>= minLineWidth)//test value
			{
				//find out if there is a maximum between them "1.4"times bigger than the minimum value between separators
				int x = m_pStrongSeparators[j];
				//value is the difference between the max and the minimum
				//long value =  m_pVertProfileDistribution->GetAt(m_pVertProfileDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pVertProfileDistribution->GetAt(x);
				int minimumValue = (int)FindMinMaxBetweenTwo(m_pStrongSeparators[j], m_pStrongSeparators[j+1],m_pVertProfileDistribution ,0,false);
				int maximumValue = (int)FindMinMaxBetweenTwo(m_pStrongSeparators[j], m_pStrongSeparators[j+1],m_pVertProfileDistribution ,1,false);
				
				/*while ((x < m_pStrongSeparators[j+1]) && (value < 10000))
				{
					x = m_pVertProfileDistribution->GetLocalMaximumPos(x+1,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
					value =  m_pVertProfileDistribution->GetAt(m_pVertProfileDistribution->GetLocalMaximumPos(x+1,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pStrongSeparators[j];
				}*/
				if (minimumValue == 0) minimumValue = 1;
				if ((maximumValue/minimumValue)> MinValueBGr)
				//if ( value > 10000)//there is a maximum big enough//create textLine
				{
					m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),m_pStrongSeparators[j],m_pStrongSeparators[j+1],pOrigImage, FALSE);
					m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].m_ThinLine = false;
					m_pdoc->m_LineHolderIndex++;
					
				}
			}
			else
			{
				//if the line is not high enough (. _ )
				//find out if there is a maximum between them "10.000" bigger than the separator
				int x = m_pStrongSeparators[j];
				//value is the difference between the max and the separator
				//long value =  m_pVertProfileDistribution->GetAt(m_pVertProfileDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pVertProfileDistribution->GetAt(x);
				int minimumValue = (int)FindMinMaxBetweenTwo(m_pStrongSeparators[j], m_pStrongSeparators[j+1],m_pVertProfileDistribution ,0,false);
				int maximumValue = (int)FindMinMaxBetweenTwo(m_pStrongSeparators[j], m_pStrongSeparators[j+1],m_pVertProfileDistribution ,1,false);
				
				/*while ((x < m_pStrongSeparators[j+1]) && (value < 10000))
				{
					x = m_pVertProfileDistribution->GetLocalMaximumPos(x+1,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
					value = m_pVertProfileDistribution->GetAt(m_pVertProfileDistribution->GetLocalMaximumPos(x+1,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pStrongSeparators[j];
				}*/
				//if ( value > 10000)//there is a maximum big enough//create textLine
				if (minimumValue == 0) minimumValue = 1;
				if ((maximumValue/minimumValue)> MinValueBGr)
				{
					//set line to special ThinLine
					if (m_pdoc->m_LineHolderIndex == 0)
					{
						m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),m_pStrongSeparators[j],m_pStrongSeparators[j+2],pOrigImage, FALSE);
						m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].m_ThinLine = true;
						m_pdoc->m_LineHolderIndex++;
						
					}
					else
					{
						//it is a thin line, and is not the first 
						m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),m_pStrongSeparators[j],m_pStrongSeparators[j+1],pOrigImage, FALSE);
						m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].m_ThinLine = true;
						m_pdoc->m_LineHolderIndex++;

						//if ((m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex-1].m_Height + (m_pStrongSeparators[j+1] - m_pStrongSeparators[j]))<= 90)
						//{
						//	long firstSeparator = m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex-1].m_Position_Y;
						//	m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex-1].DeleteLine();
						//	m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex-1].CreateLine(pOrigImage->GetWidth(),firstSeparator,m_pStrongSeparators[j+1],pOrigImage, FALSE);
						//	//m_LineHolderIndex++;
						//}
						//else
						//{
						//	if ((j + 2) < (m_StrongSeparatorIndex))
						//	{
						//		if (((m_pStrongSeparators[j+1] - m_pStrongSeparators[j]) + (m_pStrongSeparators[j+2] - m_pStrongSeparators[j+1]))< 85)
						//		{
						//			m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),m_pStrongSeparators[j],m_pStrongSeparators[j+2],pOrigImage, FALSE);
						//			m_pdoc->m_LineHolderIndex++;
						//		}
						//		else 
						//		{
						//			m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),m_pStrongSeparators[j],m_pStrongSeparators[j+1],pOrigImage, TRUE);
						//			m_pdoc->m_LineHolderIndex++;
						//		}
						//	}
						//}
					}
				}
			}

		}
	}
//	Invalidate();
}

void DIASegmentation::CreateCharacters()
{
    //for each line compare character separators
	int MinCharWidth = 25;
	int MaxCharWidth = 35;
	double MinValueBGr = 1.4; //%
//	long value;
	CImage * pOrigImage;
	long * pStrongCharacterSeparators;
	int StrongCharacterSeparatorsIndex;
	//pOrigImage = m_pdoc->GetUndoImage(m_pdoc->m_index-4);//there was -2
	pOrigImage = m_pdoc->GetUndoImage(0);
	int LineNum, CharNum, ExtraNum;
	ExtraNum = 0;

	if (m_pdoc->m_LineHolderIndex != 0)//if there is at least a line
	{
		for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)// for each line
		{
			if (m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex != 0)//if there is at least a charcacter
			{
				GetStrongSeparators(m_pdoc->m_pLineHolder[i].m_pCharacterSeparators, m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex,pStrongCharacterSeparators,StrongCharacterSeparatorsIndex);
				m_pdoc->m_pLineHolder[i].m_pCharacterHolder = new CDIACharacter [StrongCharacterSeparatorsIndex];
				m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex = 0;	
				//
				//
				//
				////Find average distances between separators and use the average
				//int * DistanceArray = new int [46];
				//ZeroMemory(DistanceArray, 46 * sizeof(int));
				//for (int j = 0; j < StrongCharacterSeparatorsIndex-1; j++)
				//{
				//	if (j == 0)//if it is the first separator find distance to the begining
				//	{
				//		if ((pStrongCharacterSeparators[j] > MinCharWidth) && (pStrongCharacterSeparators[j] < MaxCharWidth))
				//		{
				//			DistanceArray[pStrongCharacterSeparators[j]]++;
				//		}
				//		//and distance to the next
				//		if ((abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) >= MinCharWidth) && (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) <= MaxCharWidth))
				//		{
				//			DistanceArray[abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1])]++;
				//		}
				//	}
				//	else
				//	{
				//		if (j == StrongCharacterSeparatorsIndex-1) 
				//		{	
				//			//if last separator then only compare it 
				//			//to the previous and to the end of the document
				//		
				//			if (((pOrigImage->GetWidth() - pStrongCharacterSeparators[j]) > MinCharWidth) && ((pOrigImage->GetWidth() - pStrongCharacterSeparators[j]) < MaxCharWidth))
				//			{
				//				DistanceArray[(pOrigImage->GetWidth() - pStrongCharacterSeparators[j])]++;
				//			}
				//			/*if ((abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1]) >= MinCharWidth) && (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1]) <= MaxCharWidth))
				//			{
				//				DistanceArray[abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1])]++;
				//			}*/
				//		}
				//		else
				//		{
				//			if ((abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) >= MinCharWidth) && (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) <= MaxCharWidth))
				//				{
				//					DistanceArray[abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1])]++;
				//				}
				//			/*if ((abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1]) >= MinCharWidth) && (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1]) <= MaxCharWidth))
				//			{
				//				DistanceArray[abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j-1])]++;
				//			}*/
				//				
				//		}
				//	}
				//	
				//}
				////have a look at DistanceArray and find highest
				//int MaxRepeatedValue = 0;
				//int MaxRepeatedNum = 0;
				//for (int x = 0; x < 46; x++)
				//{
				//	if (DistanceArray[x] > MaxRepeatedNum)
				//	{
				//		MaxRepeatedValue = x;
				//		MaxRepeatedNum = DistanceArray[x];		
				//	}
				//	else 
				//		if (DistanceArray[x] == MaxRepeatedNum)
				//		{
				//			if (MaxRepeatedNum == 0)
				//			{
				//				MaxRepeatedValue = x;
				//				MaxRepeatedNum = DistanceArray[x];
				//			}
				//			else 
				//				MaxRepeatedValue = (MaxRepeatedValue + x)/2;
				//		}
				//	
				//		
				//}
				//delete DistanceArray;

				for (int j = 0; j < StrongCharacterSeparatorsIndex-1; j++)
				{
					if (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) > MinCharWidth)
					{	
						if (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1]) > MaxCharWidth)
						{
							//divide the character in two
							long minimumValue = (int)FindMinMaxBetweenTwo(pStrongCharacterSeparators[j], pStrongCharacterSeparators[j+1],m_pdoc->m_pLineHolder[i].m_pTextLineDistribution ,0,true);
							long value = (abs(pStrongCharacterSeparators[j] - pStrongCharacterSeparators[j+1])/2) + pStrongCharacterSeparators[j];
							
							//num = 10 * i + j + NumExtra;
							LineNum =  i;
							CharNum = j + ExtraNum;
							ExtraNum++;
							m_pdoc->m_pLineHolder[i].m_pCharacterHolder[m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex].CreateCharacter(m_pdoc->m_pLineHolder[i].m_Height, pStrongCharacterSeparators[j], value, m_pdoc->m_pLineHolder[i].m_Position_Y, pOrigImage,  LineNum, CharNum);
							m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex++;
							
							//num++;
							/*NumExtra++;
							num = 10 * i + j + NumExtra;*/
							LineNum =  i;
							CharNum = j + ExtraNum;
							m_pdoc->m_pLineHolder[i].m_pCharacterHolder[m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex].CreateCharacter(m_pdoc->m_pLineHolder[i].m_Height,value, pStrongCharacterSeparators[j+1], m_pdoc->m_pLineHolder[i].m_Position_Y, pOrigImage,  LineNum, CharNum);
							m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex++;
							//num++;
							
						}
						else
						{
							//if the distance between them is big enough
							//find if there is a peak high enough so there is a character inside
							//find out if there is a maximum between them "2.000" bigger than the minimum
							int minimumValue = (int)FindMinMaxBetweenTwo(pStrongCharacterSeparators[j], pStrongCharacterSeparators[j+1],m_pdoc->m_pLineHolder[i].m_pTextLineDistribution ,0,false);
							int maximumValue = (int)FindMinMaxBetweenTwo(pStrongCharacterSeparators[j], pStrongCharacterSeparators[j+1],m_pdoc->m_pLineHolder[i].m_pTextLineDistribution ,1,false);
							//int x;	
							//int y;
							////value is the difference between the max and the minimumValue
							//long valuemax =  m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(x);
							////long valuey =  m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMinimumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(x);
							//value = (m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(x) + valuey)/valuex;
							//while ((x < pStrongCharacterSeparators[j+1]) && (value < MinValueBGr))
							//{
							//	x = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
							//	y = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMinimumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES);
							//	if ((m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - pStrongCharacterSeparators[j])> valuex)
							//	{
							//		valuex =  m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMaximumPos(x,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - pStrongCharacterSeparators[j];
							//	}
							//	if ((m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMinimumPos(y,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - pStrongCharacterSeparators[j])< valuey)
							//	{
							//		valuey =  m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetLocalMinimumPos(y,GLM_SD_RIGHT,GLM_OUTOFBOUNDS_VALUES)) - pStrongCharacterSeparators[j];
							//	}
							//	
							//	value = valuex - valuey;
							//	x++;
							//}
							//if ( value > MinValueBGr)//there is a maximum big enough//create textLine
							//{
							//	m_pdoc->m_pLineHolder[i].m_pCharacterHolder[m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex].CreateCharacter(m_pdoc->m_pLineHolder[i].m_Height, pStrongCharacterSeparators[j], pStrongCharacterSeparators[j+1], m_pdoc->m_pLineHolder[i].m_Position_Y, pOrigImage);
							//	m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex++;
							//}
							if (minimumValue == 0) minimumValue = 1;
							if (((maximumValue/minimumValue)> MinValueBGr)&& (maximumValue >= 600))
							//if ((maximumValue - minimumValue) > MinValueBGr)
							{
								//set num/
								//num = 10 * i + j + NumExtra;
								LineNum =  i;
								CharNum = j + ExtraNum;
								m_pdoc->m_pLineHolder[i].m_pCharacterHolder[m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex].CreateCharacter(m_pdoc->m_pLineHolder[i].m_Height, pStrongCharacterSeparators[j], pStrongCharacterSeparators[j+1], m_pdoc->m_pLineHolder[i].m_Position_Y, pOrigImage, LineNum, CharNum);
								m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex++;
						
							}
						}
					}
				}
			}
		}
		delete pStrongCharacterSeparators;
	}

//	Invalidate();
	
}

long DIASegmentation::FindMinMaxBetweenTwo(int firstpos, int secondpos, CDistribution <long> * Distri,bool Mm,bool PosVal)
{
	//Mm == 0 -> find minimum
	//Mm == 1 -> find maximum
	long firstvalue, secondvalue;
	long ResultPos,ResultVal;
	firstvalue = Distri->GetAt(firstpos);
	//if (firstvalue == 0) firstvalue = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(firstpos +1);
	secondvalue = Distri->GetAt(secondpos);
	ResultVal = firstvalue;
	ResultPos = firstpos;
	if (ResultVal <=1) ResultVal += 300;
	if (Mm == 0)
	{
		for (int i = firstpos; i <= secondpos; i++)
		{
			if ((Distri->GetAt(i) <= ResultVal) && (Distri->GetAt(i) > 300))
			{
				ResultVal = Distri->GetAt(i);
				ResultPos = i;
			}
		}
	}
	else
	{
		for (int i = firstpos; i <= secondpos; i++)
		{
			if (Distri->GetAt(i) >= ResultVal)
			{
				ResultVal = Distri->GetAt(i);
				ResultPos = i;
			}
		}
	}

	if (PosVal == false) return ResultVal;
	if (PosVal == true)  return ResultPos;
	else return ResultVal;
}
//long DIASegmentation::FindMinMaxBetweenTwo(int firstpos, int secondpos, int linepos,bool Mm)
//{
//	//Mm = 0 is find minimum
//	long firstvalue, secondvalue;
//	long result;
//	firstvalue = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(firstpos);
//	//if (firstvalue == 0) firstvalue = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(firstpos +1);
//	secondvalue = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(secondpos);
//	result = firstvalue;
//	if (result <=1) result += 600;
//	if (Mm == 0)
//	{
//		for (int i = firstpos; i <= secondpos; i++)
//		{
//			if ((m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(i) <= result) && (m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(i) >600))
//				result = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(i);
//		}
//	}
//	else
//	{
//		for (int i = firstpos; i <= secondpos; i++)
//		{
//			if (m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(i) >= result)
//				result = m_pdoc->m_pLineHolder[linepos].m_pTextLineDistribution->GetAt(i);
//		}
//	}
//	return result;
//}
void DIASegmentation::FindHorizontalSeparatorsFromLines2()
{
	//for each line
	
	for ( int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
	{
		if (m_pdoc->m_pLineHolder[i].m_ThinLine == FALSE)
		{
			m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex = 0;
			m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->RemoveSubbase();
			m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->ComputeExtrema();
			int * p_minArray;

			p_minArray = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetMinimaPositions();
			m_pdoc->m_pLineHolder[i].m_pCharacterSeparators = new CTextLine::SEPARATOR [m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetMinimaCount()];
			for (int j = 0; j < m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetMinimaCount(); j++)
			{
						m_pdoc->m_pLineHolder[i].m_pCharacterSeparators[m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex].Value = p_minArray[j];
						m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex++;
			}
			SelectionateStrongSeparators7H(m_pdoc->m_pLineHolder[i].m_pCharacterSeparators,m_pdoc->m_pLineHolder[i].m_CharacterSeparatorIndex, m_pdoc->m_pLineHolder[i].m_pTextLineDistribution);
		}	
	}
}

void DIASegmentation::ComputeCharacterStatistics()
{
	//of all the characters in the document
	long accumulator = 0;
	int count = 0;
	m_mean = 0;
	long mini, maxi;
	
	maxi = 0;
	
	if (m_pdoc->m_LineHolderIndex != 0)
	{
		for (int i = 0; i < m_pdoc->m_LineHolderIndex ; i++)
		{
			if (m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex != 0)
			{	
				mini = m_pdoc->m_pLineHolder[i].m_pCharacterHolder[0].m_Width;
				for (int j = 0; j < m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex; j++)
				{
					accumulator = accumulator + m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width;
					count++;
					if (maxi < m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width)
						maxi = m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width;
					if (mini > m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width)
						mini = m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width;
					//insert values sorted in chart
				}
			}
		}
	}
	//mean

	m_pChart= new INFO [maxi - mini +1];
	m_mean = (long)(accumulator / count);
	INFO ChartMax2;
	ChartMax2.Count = 0;
	ChartMax2.Value = 0;
	long Val = 0;
	for (int x = 0; x <= (maxi - mini);x++)
	{
		m_pChart[x].Value = mini + x;
		m_pChart[x].Count = 0;
	}
	//fill in the array with values and how many of each value are.
	for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
	{
		for (int j = 0; j < m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex; j++)
		{
			m_pChart[abs(m_pdoc->m_pLineHolder[i].m_pCharacterHolder[j].m_Width - mini)].Count++;
		}
	}
	//find out what is the most repeated value in array and store it in m_ChartMax
	for (int i = 0; i < (maxi - mini +1); i++)
	{	
		//if there are more than one Count with the same Val
		
		if (m_pChart[i].Count > Val)
		{
			Val = m_pChart[i].Count;
			m_ChartMax = m_pChart[i].Value;
		}
		if ( m_pChart[i].Count == Val)
		{
			ChartMax2.Value = m_pChart[i].Value;
			ChartMax2.Count = m_pChart[i].Count;
		}
	}
	//if finally there are 2 ChartMaxs we will use them both for the analysis
	if ((ChartMax2.Value != 0) && (ChartMax2.Value == m_ChartMax))
	{
		m_pdoc->m_pLineHolder->m_ChartMax2 = ChartMax2.Value;
	}
	m_pdoc->m_pLineHolder->m_mean = m_mean;
	m_pdoc->m_pLineHolder->m_ChartMax = m_ChartMax;
}

void DIASegmentation::ComputeFixedWidthSegmentation()
{
	//first part: find out the place to segment
	//given the fixed width obtained from statistics
	CTextLine::FIXINFO MinCostVal;
	long TempVal;
	int j;
	if (m_pdoc->m_LineHolderIndex != 0)
	{	
		for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
		{
			j = 0;
			int smalll;
			int large;
			int m = -3;
			
			if (m_pdoc->m_pLineHolder->m_ChartMax > m_pdoc->m_pLineHolder->m_mean)
			{
				large = m_pdoc->m_pLineHolder->m_ChartMax;
				smalll = m_pdoc->m_pLineHolder->m_mean;
			}
			else 
			{
				large = m_pdoc->m_pLineHolder->m_mean;
				smalll = m_pdoc->m_pLineHolder->m_ChartMax;
			}
			m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [(large-smalll+11)*15];
			while (m <=7)
			{
				for (int y = smalll-5; y <= large+5; y++) 
				{
					if (m_pdoc->m_pLineHolder[i].m_pCharacterHolder[0].m_Position_X + m >= 0)
					{
						m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].FirstPos = m_pdoc->m_pLineHolder[i].m_pCharacterHolder[0].m_Position_X + m;
					}
					else 
					{
						m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].FirstPos = m_pdoc->m_pLineHolder[i].m_pCharacterHolder[0].m_Position_X;
					}
						m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].WidthVal = y;
						m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].Score = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].FirstPos);

						TempVal = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].FirstPos + m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].WidthVal;
						for (int x = 0; x <= m_pdoc->m_pLineHolder[i].m_Width/y; x++)
						{
							TempVal += m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].WidthVal;
							m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[j].Score += m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt((int)TempVal);
						}
						j++;
				}
				 m++;
			}

			//find the smaller cost for segmentation from the m_pFixedWidthArray
			MinCostVal.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[0].Score;//initialise it to the first value
			for (int x = 0; x < j; x++)
			{
				if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score < MinCostVal.Score)
				{
					MinCostVal.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score;
					MinCostVal.FirstPos = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].FirstPos;
					MinCostVal.WidthVal = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].WidthVal;
				}else
					if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score == MinCostVal.Score)
					{
						MinCostVal.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score;
						MinCostVal.WidthVal = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].WidthVal;
						MinCostVal.FirstPos = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].FirstPos;
					}
			}
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = MinCostVal.Score;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = MinCostVal.WidthVal;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = MinCostVal.FirstPos;
		}
	}
	
	m_pdoc->m_ShowFix = true;
}
void DIASegmentation::ComputeFixedWidthLineSegmentation(int FixedWidth)
{
	//using a grid FixedWidth (50) pixels wide.
	//the way to make the grid fit is by calculating the cost and choosing the lowest one.
	//things to take into consideration:
	//starting point and Score
	
	
		
		//m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = new CTextLine::FIXINFO;
		CTextLine::FIXINFO * pFixedWidthArray;
		CTextLine::FIXINFO FixedSegmentationPoint;
		pFixedWidthArray = new CTextLine::FIXINFO [FixedWidth + 1];
		/*if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray == NULL)
		{
			TRACE("IS NULL");
		}*/
		CTextLine::FIXINFO LowerCostWidth[3];
		int looptimes = 0;

		long * pImageProjH;
		CDistribution <long> * pImageDistributionV;
		pImageProjH = new long [m_pdoc->GetActiveImage()->GetHeight()];
		m_pdoc->GetActiveImage()->GetProjection(0,0,m_pdoc->GetActiveImage()->GetWidth(),m_pdoc->GetActiveImage()->GetHeight(),pImageProjH, NULL ,PRJ_GREY);
		pImageDistributionV = new CDistribution <long>;
		pImageDistributionV->Create(pImageProjH,m_pdoc->GetActiveImage()->GetHeight());

		for (int z = FixedWidth - 1;z <= FixedWidth + 1; z++) 
		{
			//try first fixedWidth positions, loop 50 times
			for (int x = 0; x < (z + 1); x++)
			{
				pFixedWidthArray[x].WidthVal = z;
				pFixedWidthArray[x].FirstPos = x;
				pFixedWidthArray[x].Score = 0;
				int loop = 0;
				while (((loop * z) + x) < m_pdoc->GetActiveImage()->GetHeight())
				{
					pFixedWidthArray[x].Score += pImageDistributionV->GetAt((loop * z )+ x);
					//pFixedWidthArray[x].Score += (long) pow(pImageDistributionV->GetAt((loop * z )+ x),2);
					loop++;
				}
			}
			//find lower cost on array and apply segmentation
			FixedSegmentationPoint.Score = pFixedWidthArray[0].Score;
			for (int y = 0; y < (z + 1); y++)
			{
				if ( pFixedWidthArray[y].Score < FixedSegmentationPoint.Score)
				{
					FixedSegmentationPoint = pFixedWidthArray[y];
				}
			}
			LowerCostWidth[looptimes].WidthVal = z;
			LowerCostWidth[looptimes].FirstPos = FixedSegmentationPoint.FirstPos;
			LowerCostWidth[looptimes].Score = FixedSegmentationPoint.Score;
			looptimes++;
		}
		//find smaller from the 3 LowerCostWidth
		if (min(LowerCostWidth[0].Score,LowerCostWidth[1].Score) < LowerCostWidth[2].Score)
		{
			if (LowerCostWidth[0].Score < LowerCostWidth[1].Score)
			{
				FixedSegmentationPoint.Score = LowerCostWidth[0].Score;
				FixedSegmentationPoint.WidthVal = LowerCostWidth[0].WidthVal;
				FixedSegmentationPoint.FirstPos = LowerCostWidth[0].FirstPos;
			}
			else
			{
				FixedSegmentationPoint.Score = LowerCostWidth[1].Score;
				FixedSegmentationPoint.WidthVal = LowerCostWidth[1].WidthVal;
				FixedSegmentationPoint.FirstPos = LowerCostWidth[1].FirstPos;
			}

		}else
		{
			FixedSegmentationPoint.Score = LowerCostWidth[2].Score;
			FixedSegmentationPoint.WidthVal = LowerCostWidth[2].WidthVal;
			FixedSegmentationPoint.FirstPos = LowerCostWidth[2].FirstPos;
		}
		
		//Now we have the StartingPoint and the value of the FixedSegmentation
		//We can delete the m_pFixedWidthArray and use it to store definitive positions
		//for each separator, find if it is stepping on a charater and move it.
		//if it is stepping more than 2 or 3 ...
	
		//for (int x = 0; x < m_pdoc->m_LineHolderIndex; x++)
		//{
			//delete m_pdoc->m_pLineHolder[x].m_pFixedWidthArray;
			int FirstSeparator, NextSeparator;

			int loops = m_pdoc->GetActiveImage()->GetHeight() / FixedSegmentationPoint.WidthVal;
			m_pdoc->m_pLineHolder = new CTextLine [loops + 1];
			m_pdoc->m_LineHolderIndex = 0;

			//m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal];
			//now fixedWidth Array is going to contain the fixed width characters
			//As we are moving the separators we will create the characters 
			
			FirstSeparator = -1;
			for (int y = 0; y < loops + 1; y++)
			{
				int CurrentPosition = FixedSegmentationPoint.FirstPos + (FixedSegmentationPoint.WidthVal * y);
				int neighbValues[6];
				neighbValues[0] = pImageDistributionV->GetAt(CurrentPosition - 1);
				neighbValues[1] = pImageDistributionV->GetAt(CurrentPosition + 1);
				neighbValues[2] = pImageDistributionV->GetAt(CurrentPosition - 2);
				neighbValues[3] = pImageDistributionV->GetAt(CurrentPosition + 2);
				neighbValues[4] = pImageDistributionV->GetAt(CurrentPosition - 3);
				neighbValues[5] = pImageDistributionV->GetAt(CurrentPosition + 3);
				//find the smallest of the neighbValues
				int minimum , minimumValue;
				//= min(min(min(neighbValues[0],neighbValues[1]),min(neighbValues[2],neighbValues[3])),min(neighbValues[4],neighbValues[5]));

				//we can't use the min(a,b) because the values might be equal
				//We give more weight to the closest values to the separator
				if (min(neighbValues[0],neighbValues[1]) < pImageDistributionV->GetAt(CurrentPosition))
				{
					if (neighbValues[0] <= neighbValues[1])
					{
						minimum =neighbValues[0];
						minimumValue = -1;
					}
					else 
					{
						minimum =neighbValues[1];
						minimumValue = +1;
					}
				}
				else 
				{
					minimum = pImageDistributionV->GetAt(CurrentPosition);
					minimumValue = 0;
				}
				if (min(neighbValues[2],neighbValues[3]) < minimum)
				{
					if (neighbValues[2] <= neighbValues[3])
					{
						minimum =neighbValues[2];
						minimumValue = -2;
					}
					else 
					{
						minimum =neighbValues[3];
						minimumValue = +2;
					}
				}

				if (min(neighbValues[4],neighbValues[5]) < minimum)
				{
					if (neighbValues[4] <= neighbValues[5])
					{
						minimum =neighbValues[4];
						minimumValue = -3;
					}
					else 
					{
						minimum =neighbValues[5];
						minimumValue = +3;
					}
				}
					

				if (minimum < pImageDistributionV->GetAt(CurrentPosition))
				{//if any neighbValue is smaller than the separator
					if (FirstSeparator < 0)//first character
					{
						FirstSeparator = CurrentPosition + minimumValue;
						NextSeparator = FirstSeparator;
					}
					else
					{
						FirstSeparator = NextSeparator;
						NextSeparator = CurrentPosition + minimumValue;
						if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetHeight())
						{
							if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
							{
								//If returns true a character has been created and we increase the counter
								TRACE("Line created/n");
							}
						}
						else
						{
							NextSeparator = m_pdoc->GetUndoImage(0)->GetHeight();
							if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
							{
								//If returns true a character has been created and we increase the counter
								TRACE("Line created/n");
							}

						}
					}
				} //if the separator is the smallest
				else 
				{
					if (FirstSeparator < 0)//first character
					{
						FirstSeparator = CurrentPosition;
						NextSeparator = FirstSeparator;
					}
					else
					{
						FirstSeparator = NextSeparator;
						NextSeparator = CurrentPosition;
						if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetHeight())
						{
							if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
							//if (CreateFixedWidthCharacters(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
							{
								//If returns true a character has been created and we increase the counter
								TRACE("Line created/n");
							}
						}
					}
				}
				
				
			}
	
	
	
}
void DIASegmentation::ComputeFixedWidthSegmentation2(int FixedWidth)
{
	//using a grid FixedWidth (30) pixels wide. Applied only to each segmented line
	//(it will also be applied to each word separately...)
	//the way to make the grid fit is by calculating the cost and choosing the lowest one.
	//things to take into consideration:
		//starting point and Score
	
	//CTextLine::FIXINFO * FixedSegmentationPoint;
	//FixedWidth +=1;
	for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
	{
		//m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = new CTextLine::FIXINFO;
		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [FixedWidth + 1];
		if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray == NULL)
		{
			TRACE("IS NULL");
		}
		CTextLine::FIXINFO LowerCostWidth[3];
		int looptimes = 0;
		
		for (int z = FixedWidth - 1;z <= FixedWidth + 1; z++) 
		{
			//try first fixedWidth positions, loop 30 times
			for (int x = 0; x < (z + 1); x++)
			{
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].WidthVal = z;
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].FirstPos = x;
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score = 0;
				int loop = 0;
				while (((loop * z) + x) < m_pdoc->m_pLineHolder[i].m_Width)
				{
					m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score += (long) pow((double)m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt((loop * z )+ x),2);
					loop++;
				}
			}
			//find lower cost on array and apply segmentation
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[0].Score;
			for (int y = 0; y < (z + 1); y++)
			{
				if ( m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y].Score < m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score)
				{
					m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y];
				}
			}
			LowerCostWidth[looptimes].WidthVal = z;
			LowerCostWidth[looptimes].FirstPos = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos;
			LowerCostWidth[looptimes].Score = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score;
			looptimes++;
		}
		//find smaller from the 3 LowerCostWidth
		if (min(LowerCostWidth[0].Score,LowerCostWidth[1].Score) < LowerCostWidth[2].Score)
		{
			if (LowerCostWidth[0].Score < LowerCostWidth[1].Score)
			{
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[0].Score;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[0].WidthVal;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[0].FirstPos;
			}
			else
			{
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[1].Score;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[1].WidthVal;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[1].FirstPos;
			}

		}else
		{
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[2].Score;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[2].WidthVal;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[2].FirstPos;
		}
		
		//Now we have the StartingPoint and the value of the FixedSegmentation
		//We can delete the m_pFixedWidthArray and use it to store definitive positions
		//for each separator, find if it is stepping on a charater and move it.
		//if it is stepping more than 2 or 3 ...
	
		//for (int x = 0; x < m_pdoc->m_LineHolderIndex; x++)
		//{
			//delete m_pdoc->m_pLineHolder[x].m_pFixedWidthArray;
			int FirstSeparator, NextSeparator;
			int CharNum, LineNum;
			LineNum = i;
			CharNum = 0;

			int loops = m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal;
			m_pdoc->m_pLineHolder[i].m_pCharacterHolder = new CDIACharacter [loops + 1];
			m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex = 0;

			//m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal];
			//now fixedWidth Array is going to contain the fixed width characters
			//As we are moving the separators we will create the characters 
			
			FirstSeparator = -1;
			for (int y = 0; y < loops + 1; y++)
			{
				int CurrentPosition = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos + (m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal * y);
				if (CurrentPosition >= m_pdoc->GetUndoImage(0)->GetWidth())
				{
					CurrentPosition = m_pdoc->GetUndoImage(0)->GetWidth() - 4;
				}
				int neighbValues[6];
				neighbValues[0] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 1);
				neighbValues[1] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 1);
				neighbValues[2] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 2);
				neighbValues[3] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 2);
				neighbValues[4] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 3);
				neighbValues[5] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 3);
				//find the smallest of the neighbValues
				int minimum , minimumValue;
				//= min(min(min(neighbValues[0],neighbValues[1]),min(neighbValues[2],neighbValues[3])),min(neighbValues[4],neighbValues[5]));

				//we can't use the min(a,b) because the values might be equal
				//We give more weight to the closest values to the separator
				if (min(neighbValues[0],neighbValues[1]) < m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition))
				{
					if (neighbValues[0] <= neighbValues[1])
					{
						minimum =neighbValues[0];
						minimumValue = -1;
					}
					else 
					{
						minimum =neighbValues[1];
						minimumValue = +1;
					}
				}
				else 
				{
					minimum = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition);
					minimumValue = 0;
				}
				if (min(neighbValues[2],neighbValues[3]) < minimum)
				{
					if (neighbValues[2] <= neighbValues[3])
					{
						minimum =neighbValues[2];
						minimumValue = -2;
					}
					else 
					{
						minimum =neighbValues[3];
						minimumValue = +2;
					}
				}

				if (min(neighbValues[4],neighbValues[5]) < minimum)
				{
					if (neighbValues[4] <= neighbValues[5])
					{
						minimum =neighbValues[4];
						minimumValue = -3;
					}
					else 
					{
						minimum =neighbValues[5];
						minimumValue = +3;
					}
				}
					

				if (minimum < m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition))
				{//if any neighbValue is smaller than the separator
					if (FirstSeparator < 0)//first character
					{
						FirstSeparator = CurrentPosition + minimumValue;
						NextSeparator = FirstSeparator;
					}
					else
					{
						FirstSeparator = NextSeparator;
						NextSeparator = CurrentPosition + minimumValue;
						if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetWidth())
						{
							if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
							{
								//If returns true a character has been created and we increase the counter
								CharNum++;
							}
						}
						else
						{
							NextSeparator = m_pdoc->GetUndoImage(0)->GetWidth();
							if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
							{
								//If returns true a character has been created and we increase the counter
								CharNum++;
							}

						}
					}
				} //if the separator is the smallest
				else 
				{
					if (FirstSeparator < 0)//first character
					{
						FirstSeparator = CurrentPosition;
						NextSeparator = FirstSeparator;
					}
					else
					{
						FirstSeparator = NextSeparator;
						NextSeparator = CurrentPosition;
						if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetWidth())
						{
							if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
							{
								//If returns true a character has been created and we increase the counter
								CharNum++;
							}
						}
					}
				}
				//now we create the character between FirstSeparator and NextSeparator	
				//if (CreateFixedWidthCharacters(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//{
				//	//If returns true a character has been created and we increase the counter
				//	CharNum++;
				//}
				
			}
		//}

	}
}
bool DIASegmentation::CreateFixedWidthCharacters(int FirstSeparator, int NextSeparator, CTextLine * Line,CImage * pOrigImage, int LineNum,int CharNum)
{
	double MinValueBGr = 1.4; //%1.4
	
	//find if there is a peak high enough so there is a character inside
	//find out if there is a maximum between them "2.000" bigger than the minimum
	int minimumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,0,false);
	int maximumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,1,false);
	int valuemaxpos = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,1,true);
	if (minimumValue == 0) minimumValue = 1;
	if (((maximumValue/minimumValue)> MinValueBGr)&& (maximumValue >= 601))
	////if ((maximumValue - minimumValue) > MinValueBGr)
	{
		Line[LineNum].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
		Line[LineNum].m_CharacterHolderIndex++;
		return true;
	}
	else 
		return false;

	////All of the characters are created
	////like that we will not exclude very faded characters

	Line[LineNum].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
	Line[LineNum].m_CharacterHolderIndex++;
	return true;
	

}
bool DIASegmentation::CreateFixedWidthCharacters2(int FirstSeparator, int NextSeparator, CTextLine * Line,CImage * pOrigImage, int LineNum,int CharNum)
{
	double MinValueBGr = 1.4; //%1.4
	
	//find if there is a peak high enough so there is a character inside
	//find out if there is a maximum between them "2.000" bigger than the minimum
	int minimumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,0,false);
	int maximumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,1,false);
	int valuemaxpos = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,Line[LineNum].m_pTextLineDistribution ,1,true);
	if (minimumValue == 0) minimumValue = 1;
	//if (maximumValue >= 600)
	if ((maximumValue/minimumValue) > MinValueBGr)
		if (maximumValue > 3000)
		
	////if ((maximumValue - minimumValue) > MinValueBGr)
	{
		Line[LineNum].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
		Line[LineNum].m_CharacterHolderIndex++;
		return true;
	}
	else 
		return false;

	//All the characters are created
	//like that we will not exclude very faded characters

	//Line[LineNum].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
	//Line[LineNum].m_CharacterHolderIndex++;
	//return true;
	

}
bool DIASegmentation::CreateFixedWidthLines(int FirstSeparator, int NextSeparator,CImage * pOrigImage,CDistribution <long> * ImageDistribution)
{

	CDIAHistograms histo;
	CImage * outImage;
	outImage = new CImage;
	outImage->CreateFromImage(pOrigImage,0);
	histo.m_pdoc = m_pdoc;
	CRect rect(0,FirstSeparator,pOrigImage->GetWidth(),NextSeparator);
	
	histo.StretchHisto(pOrigImage, rect, outImage);
	histo.ComputeHistogram(histo.m_inHistogram,&histo.m_maxvalueInHisto,rect,outImage);
	if (histo.GetMinimum() < 100)//then there is no line
	{
		if (((NextSeparator-FirstSeparator) > 40) && ((NextSeparator-FirstSeparator) < 60))
		{
			m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),FirstSeparator,NextSeparator,pOrigImage, false);
		//line[line.].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
		m_pdoc->m_LineHolderIndex++;
		return true;
		}
		else return false;
	}
	else return false;
	delete outImage;
	//double MinValueBGr = 0.5;
	//
	//int minimumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,ImageDistribution ,0,false);
	//int maximumValue = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,ImageDistribution,1,false);
	//int valuemaxpos = (int)FindMinMaxBetweenTwo(FirstSeparator, NextSeparator,ImageDistribution ,1,true);
	//if (minimumValue == 0) minimumValue = 1;
	//if (((maximumValue/minimumValue)> MinValueBGr)&& (maximumValue >= 601))
	//////if ((maximumValue - minimumValue) > MinValueBGr)
	//{
	//	m_pdoc->m_pLineHolder[m_pdoc->m_LineHolderIndex].CreateLine(pOrigImage->GetWidth(),FirstSeparator,NextSeparator,pOrigImage, false);
	//	//line[line.].m_pCharacterHolder[Line[LineNum].m_CharacterHolderIndex].CreateCharacter(Line[LineNum].m_Height, FirstSeparator, NextSeparator, Line[LineNum].m_Position_Y, pOrigImage, LineNum, CharNum);
	//	m_pdoc->m_LineHolderIndex++;
	//	return true;
	//}
	//else 
	//	return false;
}

CTextLine::FIXINFO DIASegmentation::CalculateMinimumCostFixedSegmentation(int FixedWidth, CDistribution <long> *profile)
{
	CTextLine::FIXINFO LowerCostInfo;
	LowerCostInfo.FirstPos = -1;
	LowerCostInfo.WidthVal = -1;
	LowerCostInfo.Score = -1;

	CTextLine::FIXINFO TempCost;
	TempCost.Score = 0;
	TempCost.FirstPos = -1;
	TempCost.WidthVal = -1;
	
	for (int z = FixedWidth - 2;z <= FixedWidth + 2; z++) 
	{
		//try first fixedWidth positions, loop fixedWidth-times
		for (int x = 0; x < (z + 1); x++)
		{
			TempCost.WidthVal = z;
			TempCost.FirstPos = x;
			TempCost.Score = 0;
			int loop = 0;
			int count = profile->GetCount();
			while (((loop * z) + x) < profile->GetCount())
			{
				long getat = profile->GetAt((loop * z )+ x);
				//TempCost.Score += (long) pow(profile->GetAt((loop * z )+ x),2);
				TempCost.Score += profile->GetAt((loop * z )+ x);
				loop++;
			}
			if (LowerCostInfo.Score < 0)
			{
				LowerCostInfo.Score = TempCost.Score;
				LowerCostInfo.FirstPos = TempCost.FirstPos;
				LowerCostInfo.WidthVal = TempCost.WidthVal;
			}
			else if (TempCost.Score <= LowerCostInfo.Score)
			{
				LowerCostInfo.Score = TempCost.Score;
				LowerCostInfo.FirstPos = TempCost.FirstPos;
				LowerCostInfo.WidthVal = TempCost.WidthVal;
			}
		}
		
	}
	return LowerCostInfo;
}

void DIASegmentation::ComputeFixedWidthSegmentation3(int FixedWidthLines, int FixedWidthChars)
{
	FixedWidthLinesSeg(FixedWidthLines);
	//ComputeFixedWidthSegmentation2(FixedWidthChars);
	FixedWidthCharsSeg(FixedWidthChars);
	
}
void DIASegmentation::ComputeFixedWidthSegmentation4(int FixedWidth)
{

	//using a grid FixedWidth (30) pixels wide. Applied only to each segmented line
	//(it will also be applied to each word separately...)
	//the way to make the grid fit is by calculating the cost and choosing the lowest one.
	//things to take into consideration:
		//starting point and Score
	
	//CTextLine::FIXINFO * FixedSegmentationPoint;
	//FixedWidth +=1;
	for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
	{
		//m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = new CTextLine::FIXINFO;
		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [FixedWidth + 1];
		if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray == NULL)
		{
			TRACE("IS NULL");
		}
		CTextLine::FIXINFO LowerCostWidth[3];
		int looptimes = 0;
		
		for (int z = FixedWidth - 1;z <= FixedWidth + 1; z++) 
		{
			//try first fixedWidth positions, loop 30 times
			for (int x = 0; x < (z + 1); x++)
			{
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].WidthVal = z;
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].FirstPos = x;
				m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score = 0;
				int loop = 0;
				while (((loop * z) + x) < m_pdoc->m_pLineHolder[i].m_Width)
				{
					m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score += (long) pow((double)m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt((loop * z )+ x),2);
					loop++;
				}
			}
			//find lower cost on array and apply segmentation
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[0].Score;
			for (int y = 0; y < (z + 1); y++)
			{
				if ( m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y].Score < m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score)
				{
					m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y];
				}
			}
			LowerCostWidth[looptimes].WidthVal = z;
			LowerCostWidth[looptimes].FirstPos = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos;
			LowerCostWidth[looptimes].Score = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score;
			looptimes++;
		}
		//find smaller from the 3 LowerCostWidth
		if (min(LowerCostWidth[0].Score,LowerCostWidth[1].Score) < LowerCostWidth[2].Score)
		{
			if (LowerCostWidth[0].Score < LowerCostWidth[1].Score)
			{
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[0].Score;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[0].WidthVal;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[0].FirstPos;
			}
			else
			{
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[1].Score;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[1].WidthVal;
				m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[1].FirstPos;
			}

		}else
		{
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[2].Score;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[2].WidthVal;
			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[2].FirstPos;
		}
		
			int FirstSeparator, NextSeparator;
			int CharNum, LineNum;
			LineNum = i;
			CharNum = 0;

			int loops = m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal;
			m_pdoc->m_pLineHolder[i].m_pCharacterHolder = new CDIACharacter [loops + 1];
			m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex = 0;


			for (int y = 0; y < loops + 1; y++)
			{
				int CurrentPosition = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos + (m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal * y);

				FirstSeparator = CurrentPosition;
				NextSeparator = FirstSeparator + m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal;

				if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetWidth())
				{
					if (CreateFixedWidthCharacters(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
					{
						//If returns true a character has been created and we increase the counter
						CharNum++;
					}
				}

			}


	}

}
void DIASegmentation::FixedWidthCharsSeg(int FixedWidth)
{
	//using a grid FixedWidth (30) pixels wide. Applied only to each segmented line
	//(it will also be applied to each word separately...)
	//the way to make the grid fit is by calculating the cost and choosing the lowest one.
	//things to take into consideration:
		//starting point and Score
	
	//CTextLine::FIXINFO * FixedSegmentationPoint;
	//FixedWidth +=1;
	for (int i = 0; i < m_pdoc->m_LineHolderIndex; i++)
	{
		//m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = new CTextLine::FIXINFO;
		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [FixedWidth + 1];
		if (m_pdoc->m_pLineHolder[i].m_pFixedWidthArray == NULL)
		{
			TRACE("IS NULL");
		}
		
		int looptimes = 0;
		CTextLine::FIXINFO LowerCostWidth = CalculateMinimumCostFixedSegmentation(FixedWidth, m_pdoc->m_pLineHolder[i].m_pTextLineDistribution);
		
		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth.WidthVal;
		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth.Score;
		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth.FirstPos;
		//for (int z = FixedWidth - 1;z <= FixedWidth + 1; z++) 
		//{
		//	//try first fixedWidth positions, loop 30 times
		//	for (int x = 0; x < (z + 1); x++)
		//	{
		//		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].WidthVal = z;
		//		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].FirstPos = x;
		//		m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score = 0;
		//		int loop = 0;
		//		while (((loop * z) + x) < m_pdoc->m_pLineHolder[i].m_Width)
		//		{
		//			m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[x].Score += (long) pow(m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt((loop * z )+ x),2);
		//			loop++;
		//		}
		//	}
		//	//find lower cost on array and apply segmentation
		//	m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[0].Score;
		//	for (int y = 0; y < (z + 1); y++)
		//	{
		//		if ( m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y].Score < m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score)
		//		{
		//			m_pdoc->m_pLineHolder[i].FixedSegmentationPoint = m_pdoc->m_pLineHolder[i].m_pFixedWidthArray[y];
		//		}
		//	}
		//	LowerCostWidth[looptimes].WidthVal = z;
		//	LowerCostWidth[looptimes].FirstPos = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos;
		//	LowerCostWidth[looptimes].Score = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score;
		//	looptimes++;
		//}
		////find smaller from the 3 LowerCostWidth
		//if (min(LowerCostWidth[0].Score,LowerCostWidth[1].Score) < LowerCostWidth[2].Score)
		//{
		//	if (LowerCostWidth[0].Score < LowerCostWidth[1].Score)
		//	{
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[0].Score;
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[0].WidthVal;
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[0].FirstPos;
		//	}
		//	else
		//	{
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[1].Score;
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[1].WidthVal;
		//		m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[1].FirstPos;
		//	}

		//}else
		//{
		//	m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.Score = LowerCostWidth[2].Score;
		//	m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal = LowerCostWidth[2].WidthVal;
		//	m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos = LowerCostWidth[2].FirstPos;
		//}
		
		//Now we have the StartingPoint and the value of the FixedSegmentation
		//We can delete the m_pFixedWidthArray and use it to store definitive positions
		//for each separator, find if it is stepping on a charater and move it.
		//if it is stepping more than 2 or 3 ...
	
		//for (int x = 0; x < m_pdoc->m_LineHolderIndex; x++)
		//{
			//delete m_pdoc->m_pLineHolder[x].m_pFixedWidthArray;
			int FirstSeparator, NextSeparator;
			int CharNum, LineNum;
			LineNum = i;
			CharNum = 0;

			int loops = m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal;
			m_pdoc->m_pLineHolder[i].m_pCharacterHolder = new CDIACharacter [loops + 1];
			m_pdoc->m_pLineHolder[i].m_CharacterHolderIndex = 0;

			//m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal];
			//now fixedWidth Array is going to contain the fixed width characters
			//As we are moving the separators we will create the characters 
			
			FirstSeparator = -1;
			for (int y = 0; y < loops + 1; y++)
			{
				int CurrentPosition = m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.FirstPos + (m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal * y);

				//
				FirstSeparator = CurrentPosition;
				NextSeparator = CurrentPosition + m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal;
				if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
							{
								//If returns true a character has been created and we increase the counter
								CharNum++;
							}
				//

				//int neighbValues[6];
				//neighbValues[0] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 1);
				//neighbValues[1] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 1);
				//neighbValues[2] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 2);
				//neighbValues[3] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 2);
				//neighbValues[4] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition - 3);
				//neighbValues[5] = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition + 3);
				////find the smallest of the neighbValues
				//int minimum , minimumValue;
				////= min(min(min(neighbValues[0],neighbValues[1]),min(neighbValues[2],neighbValues[3])),min(neighbValues[4],neighbValues[5]));

				////we can't use the min(a,b) because the values might be equal
				////We give more weight to the closest values to the separator
				//if (min(neighbValues[0],neighbValues[1]) < m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition))
				//{
				//	if (neighbValues[0] <= neighbValues[1])
				//	{
				//		minimum =neighbValues[0];
				//		minimumValue = -1;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[1];
				//		minimumValue = +1;
				//	}
				//}
				//else 
				//{
				//	minimum = m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition);
				//	minimumValue = 0;
				//}
				//if (min(neighbValues[2],neighbValues[3]) < minimum)
				//{
				//	if (neighbValues[2] <= neighbValues[3])
				//	{
				//		minimum =neighbValues[2];
				//		minimumValue = -2;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[3];
				//		minimumValue = +2;
				//	}
				//}

				//if (min(neighbValues[4],neighbValues[5]) < minimum)
				//{
				//	if (neighbValues[4] <= neighbValues[5])
				//	{
				//		minimum =neighbValues[4];
				//		minimumValue = -3;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[5];
				//		minimumValue = +3;
				//	}
				//}
				//	

				//if (minimum < m_pdoc->m_pLineHolder[i].m_pTextLineDistribution->GetAt(CurrentPosition))
				//{//if any neighbValue is smaller than the separator
				//	if (FirstSeparator < 0)//first character
				//	{
				//		FirstSeparator = CurrentPosition + minimumValue;
				//		NextSeparator = FirstSeparator;
				//	}
				//	else
				//	{
				//		FirstSeparator = NextSeparator;
				//		NextSeparator = CurrentPosition + minimumValue;
				//		if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetWidth())
				//		{
				//			if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				CharNum++;
				//			}
				//		}
				//		else
				//		{
				//			NextSeparator = m_pdoc->GetUndoImage(0)->GetWidth();
				//			if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				CharNum++;
				//			}

				//		}
				//	}
				//} //if the separator is the smallest
				//else 
				//{
				//	if (FirstSeparator < 0)//first character
				//	{
				//		FirstSeparator = CurrentPosition;
				//		NextSeparator = FirstSeparator;
				//	}
				//	else
				//	{
				//		FirstSeparator = NextSeparator;
				//		NextSeparator = CurrentPosition;
				//		if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetWidth())
				//		{
				//			if (CreateFixedWidthCharacters2(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				CharNum++;
				//			}
				//		}
				//	}
				//}
				//now we create the character between FirstSeparator and NextSeparator	
				//if (CreateFixedWidthCharacters(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//{
				//	//If returns true a character has been created and we increase the counter
				//	CharNum++;
				//}
				
			}
		//}

	}
}
void DIASegmentation::FixedWidthLinesSeg(int FixedWidth)
{
	//using a grid FixedWidth (50) pixels wide.
	//the way to make the grid fit is by calculating the cost and choosing the lowest one.
	//things to take into consideration:
	//starting point and Score
	
	
		
		int looptimes = 0;

		long * pImageProjH;
		CDistribution <long> * pImageDistributionV;
		pImageProjH = new long [m_pdoc->GetActiveImage()->GetHeight()];
		m_pdoc->GetActiveImage()->GetProjection(0,0,m_pdoc->GetActiveImage()->GetWidth(),m_pdoc->GetActiveImage()->GetHeight(),pImageProjH, NULL ,PRJ_GREY);
		pImageDistributionV = new CDistribution <long>;
		pImageDistributionV->Create(pImageProjH,m_pdoc->GetActiveImage()->GetHeight());

		//
		CTextLine::FIXINFO LowerCostInfo = CalculateMinimumCostFixedSegmentation(FixedWidth, pImageDistributionV);
		//

		//Now we have the StartingPoint and the value of the FixedSegmentation
		//We can delete the m_pFixedWidthArray and use it to store definitive positions
		//for each separator, find if it is stepping on a charater and move it.
		//if it is stepping more than 2 or 3 ...
	
		//for (int x = 0; x < m_pdoc->m_LineHolderIndex; x++)
		//{
			//delete m_pdoc->m_pLineHolder[x].m_pFixedWidthArray;
			int FirstSeparator, NextSeparator;

			int loops = m_pdoc->GetActiveImage()->GetHeight() / LowerCostInfo.WidthVal;
			m_pdoc->m_pLineHolder = new CTextLine [loops + 1];
			m_pdoc->m_LineHolderIndex = 0;

			//m_pdoc->m_pLineHolder[i].m_pFixedWidthArray = new CTextLine::FIXINFO [m_pdoc->m_pLineHolder[i].m_Width / m_pdoc->m_pLineHolder[i].FixedSegmentationPoint.WidthVal];
			//now fixedWidth Array is going to contain the fixed width characters
			//As we are moving the separators we will create the characters 
			
			FirstSeparator = LowerCostInfo.FirstPos;
			NextSeparator = FirstSeparator;
			for (int y = 0; y < loops + 1; y++)
			{
				FirstSeparator = NextSeparator;
				NextSeparator = FirstSeparator + LowerCostInfo.WidthVal -1;
				if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetHeight())
				{
					if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
					{
						//If returns true a character has been created and we increase the counter
						TRACE("Line created/n");
					}
				}
				else
				{
					NextSeparator = m_pdoc->GetUndoImage(0)->GetHeight()-1;
					if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
					{
						//If returns true a character has been created and we increase the counter
						TRACE("Line created/n");
						
					}

				}

				//int neighbValues[6];
				//neighbValues[0] = pImageDistributionV->GetAt(CurrentPosition - 1);
				//neighbValues[1] = pImageDistributionV->GetAt(CurrentPosition + 1);
				//neighbValues[2] = pImageDistributionV->GetAt(CurrentPosition - 2);
				//neighbValues[3] = pImageDistributionV->GetAt(CurrentPosition + 2);
				//neighbValues[4] = pImageDistributionV->GetAt(CurrentPosition - 3);
				//neighbValues[5] = pImageDistributionV->GetAt(CurrentPosition + 3);
				////find the smallest of the neighbValues
				//int minimum , minimumValue;
				////= min(min(min(neighbValues[0],neighbValues[1]),min(neighbValues[2],neighbValues[3])),min(neighbValues[4],neighbValues[5]));

				////we can't use the min(a,b) because the values might be equal
				////We give more weight to the closest values to the separator
				//if (min(neighbValues[0],neighbValues[1]) < pImageDistributionV->GetAt(CurrentPosition))
				//{
				//	if (neighbValues[0] <= neighbValues[1])
				//	{
				//		minimum =neighbValues[0];
				//		minimumValue = -1;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[1];
				//		minimumValue = +1;
				//	}
				//}
				//else 
				//{
				//	minimum = pImageDistributionV->GetAt(CurrentPosition);
				//	minimumValue = 0;
				//}
				//if (min(neighbValues[2],neighbValues[3]) < minimum)
				//{
				//	if (neighbValues[2] <= neighbValues[3])
				//	{
				//		minimum =neighbValues[2];
				//		minimumValue = -2;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[3];
				//		minimumValue = +2;
				//	}
				//}

				//if (min(neighbValues[4],neighbValues[5]) < minimum)
				//{
				//	if (neighbValues[4] <= neighbValues[5])
				//	{
				//		minimum =neighbValues[4];
				//		minimumValue = -3;
				//	}
				//	else 
				//	{
				//		minimum =neighbValues[5];
				//		minimumValue = +3;
				//	}
				//}
				//	

				//if (minimum < pImageDistributionV->GetAt(CurrentPosition))
				//{//if any neighbValue is smaller than the separator
				//	if (FirstSeparator < 0)//first character
				//	{
				//		FirstSeparator = CurrentPosition + minimumValue;
				//		NextSeparator = FirstSeparator;
				//	}
				//	else
				//	{
				//		FirstSeparator = NextSeparator;
				//		NextSeparator = CurrentPosition + minimumValue;
				//		if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetHeight())
				//		{
				//			if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				TRACE("Line created/n");
				//			}
				//		}
				//		else
				//		{
				//			NextSeparator = m_pdoc->GetUndoImage(0)->GetHeight();
				//			if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				TRACE("Line created/n");
				//			}

				//		}
				//	}
				//} //if the separator is the smallest
				//else 
				//{
				//	if (FirstSeparator < 0)//first character
				//	{
				//		FirstSeparator = CurrentPosition;
				//		NextSeparator = FirstSeparator;
				//	}
				//	else
				//	{
				//		FirstSeparator = NextSeparator;
				//		NextSeparator = CurrentPosition;
				//		if (NextSeparator <= m_pdoc->GetUndoImage(0)->GetHeight())
				//		{
				//			if (CreateFixedWidthLines(FirstSeparator, NextSeparator,m_pdoc->GetUndoImage(0),pImageDistributionV))
				//			//if (CreateFixedWidthCharacters(FirstSeparator, NextSeparator,m_pdoc->m_pLineHolder,m_pdoc->GetUndoImage(0),LineNum, CharNum))
				//			{
				//				//If returns true a character has been created and we increase the counter
				//				TRACE("Line created/n");
				//			}
				//		}
				//	}
				//}
				
				
			}
	
	
	
}