#include "StdAfx.h"
#include "distance.h"

#include <string.h>
#include <malloc.h>

//****************************
// Get minimum of three values
//****************************

int CDistance::Minimum (int a, int b, int c)
{
int mi;

  mi = a;
  if (b < mi) {
    mi = b;
  }
  if (c < mi) {
    mi = c;
  }
  return mi;

}

//**************************************************
// Get a pointer to the specified cell of the matrix
//************************************************** 

int *CDistance::GetCellPointer (int *pOrigin, int col, int row, int nCols)
{
  return pOrigin + col + (row * (nCols + 1));
}

//*****************************************************
// Get the contents of the specified cell in the matrix 
//*****************************************************

int CDistance::GetAt (int *pOrigin, int col, int row, int nCols)
{
int *pCell;

  pCell = GetCellPointer (pOrigin, col, row, nCols);
  return *pCell;

}

//*******************************************************
// Fill the specified cell in the matrix with the value x
//*******************************************************

void CDistance::PutAt (int *pOrigin, int col, int row, int nCols, int x)
{
int *pCell;

  pCell = GetCellPointer (pOrigin, col, row, nCols);
  *pCell = x;

}

//*****************************
// Compute Levenshtein distance
//*****************************

int CDistance::LD (char const *s, char const *t)
{
	int *d; // pointer to matrix
	int n; // length of s
	int m; // length of t
	int i; // iterates through s
	int j; // iterates through t
	char s_i; // ith character of s
	char t_j; // jth character of t
	int cost; // cost
	int result; // result
	int cell; // contents of target cell
	int above; // contents of cell immediately above
	int left; // contents of cell immediately to left
	int diag; // contents of cell immediately above and to left
	int sz; // number of cells in matrix

  // Step 1	

	n = strlen (s);
	m = strlen (t);
	if (n == 0) 
	{
		return m;
	}
	if (m == 0) 
	{
		return n;
	}
	sz = (n+1) * (m+1) * sizeof (int);
	d = (int *) malloc (sz);

	// Step 2

	for (i = 0; i <= n; i++) 
	{
		PutAt (d, i, 0, n, i);
	}

	for (j = 0; j <= m; j++) 
	{
		PutAt (d, 0, j, n, j);
	}

	// Step 3

	for (i = 1; i <= n; i++) 
	{

		s_i = s[i-1];

		// Step 4

		for (j = 1; j <= m; j++) 
		{

		t_j = t[j-1];

		// Step 5

		if (s_i == t_j) {
			cost = 0;
		}
		else {
			cost = 1;
		}

		// Step 6 

		above = GetAt (d,i-1,j, n);
		left = GetAt (d,i, j-1, n);
		diag = GetAt (d, i-1,j-1, n);
		cell = Minimum (above + 1, left + 1, diag + cost);
		PutAt(d, i, j, n, cell);
		}
	}

	// Step 7

	result = GetAt (d, n, m, n);
	free (d);
	return result;
	
}


//*****************************
// Compute Levenshtein distance
//*****************************

int CDistance::LD (char const *s, char const *t,int &total, int &missing, int &noise, int &wrong)
{
	int *d; // pointer to matrix
	int n; // length of s
	int m; // length of t
	int i; // iterates through s
	int j; // iterates through t
	char s_i; // ith character of s
	char t_j; // jth character of t
	int cost; // cost
	int result; // result
	int cell; // contents of target cell
	int above; // contents of cell immediately above
	int left; // contents of cell immediately to left
	int diag; // contents of cell immediately above and to left
	int sz; // number of cells in matrix

	missing = 0;
	wrong = 0;
	noise = 0;
  // Step 1	

	n = strlen (s);
	
	m = strlen (t);
	if (n == 0) 
	{
		return m;
	}
	if (m == 0) 
	{
		return n;
	}
	sz = (n+1) * (m+1) * sizeof (int);
	d = (int *) malloc (sz);

	// Step 2

	for (i = 0; i <= n; i++) 
	{
		PutAt (d, i, 0, n, i);
	}

	for (j = 0; j <= m; j++) 
	{
		PutAt (d, 0, j, n, j);
	}

	// Step 3

	for (i = 1; i <= n; i++) 
	{

		s_i = s[i-1];

		// Step 4

		for (j = 1; j <= m; j++) 
		{

		t_j = t[j-1];

		// Step 5

		if (s_i == t_j) {
			cost = 0;
		}
		else {
			cost = 1;
		}

		// Step 6 

		above = GetAt (d,i-1,j, n); //deletion
		left = GetAt (d,i, j-1, n); //insertion
		diag = GetAt (d, i-1,j-1, n); //substitution
		cell = Minimum (above + 1, left + 1, diag + cost);
		/*if (cell == (above + 1))
		{
			noise++;
		}
		else if (cell == (diag + 1))
		{
			missing++;
		}
		else wrong++;*/
		PutAt(d, i, j, n, cell);
		}
	}

	// Step 7

	//now go back from the bottom right and get the number of missing, worng and noise
	int val;
	result = GetAt (d, n, m, n);
	val = result;
	int a = n;
	int b = m;
	int min;
	while ((a!= 0) || (a !=0))
	{
		if ((a-1)>= 0)
		{
			above = GetAt (d,a-1,b, n); //deletion
		}
		else above = 100000000;

		if ((b-1)>=0)
		{
			left = GetAt (d,a, b-1, n); //insertion
		}else
		{
			left = 1000000000;
		}

		if ((a-1 >=0) && ( b-1 >=0))
		{
			diag = GetAt (d, a-1,b-1, n); //substitution
		}
		else {
			diag = 100000000;
		}
		min = Minimum (diag, above, left);
		
		if (min == val)
		{
			a--;
			b--;
		}
		else
		{
			
			if (min == diag)
			{
				wrong++;
				
				a--;
				b--;
			}
			else
			if (min == above)
			{
				//then there was insertion
				missing++;
				a--;

			}else
			if (min == left)
			{
				//then there was deletion
				noise++;
				b--;

			}
			
		}
		if (min == diag) val = GetAt (d, a,b, n);
		else if (min == above) val = GetAt (d,a,b, n);
		else if (min == left) val = GetAt(d,a, b, n);
	}
	total = n;
	free (d);
	return result;
	
}