#pragma once
#ifndef DOCIMANDOC_H
#define DOCIMANDOC_H

// DocImAnDoc.h : interface of the CDocImAnDoc class
//
#include "Image.h"
#include "TextLine.h"
#include "DIACharacter.h"
#include "CharVector.h"
#include "ClassifierVector.h"
#include "ClassList.h"
#include <vector>
using namespace std;
//#include "DlgAddImages.h"

class CTextLine;
class CDocImAnDoc : public CDocument
{
protected: // create from serialization only
	CDocImAnDoc();
	DECLARE_DYNCREATE(CDocImAnDoc)

// Attributes
public:

// Operations
public:

// Overrides
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	void UpdateAllViews(CView* pSender, LPARAM lHint, CObject* pHint);

// Implementation
public:
	virtual ~CDocImAnDoc();

	CImage * GetActiveImage();
	void SetActiveImage(CImage * mImage);

//#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
//#endif

protected:

	CImage * m_pImage;
public:
	typedef struct INFO
	{	
		float	Val;
		char	Name;
		int		Cost;
		int		Pos;
		bool	Capital;
		float	Prob;
	};
	CImage ** m_pUndoImage;
	int m_index;
	void CreateNewUndoImage(DWORD Type = 0x00000000);
	void DeleteUndoList();
	CImage * GetUndoImage(int index);
//	void CreateNewCharacterVector();
	void ExportCharVectorToFile();
	void ExportCharVectorToFile(CCharVector** &VectorList, int &vectorIndex, CString Filepath, CString ImagePath);
	void AddNewCharacterVector(CCharVector &Vector);
	void AddNewCharacterVector(CCharVector &Vector, CCharVector** &VectorList, int &vectorIndex);
	void ImportCharVectorFromFile();
	void ImportCharCapitalVectorFromFile();
	//void ImportCharCapitalVectorFromFile2();
	void ImportCharVectorFromFile(CString filePath, CCharVector** &vectorList, int &vectorIndex);
	//void ExportCharacter(CDIACharacter character);
	void ExportCharacterCVS(CDIAGreyScale image, char name);
//	void ImportCharacter(CDIAGreyScale &image, char &name);
//	void CDocImAnDoc::ImportCharacter(CDIAGreyScale image, char name);
	void Exportimage(CImage* image, char name);
	void Exportimage(CImage * image, char name, CString path);
	void Exportimage(CImage * image, CString name, CString path);
	void DeleteVectorList();
	CString AddCharacterToClassifier(CCharVector &Vector);
	//CString AddCharacterToClassifier2(CCharVector &Vector);
	void ExportClassifier();
//	void ExportClassifier2();
	void ImportClassifier();
//	void ImportClassifier2();
	void ComputeContingencyTable();
	void AddFeatureToTable(int character,int featureNo, int val);
	void AddFeatureToTable(int character,int featureNo, float val);
	void TraceFeatureTable();
	void ExportFeatureTable();
	void DeleteContingencyTable();
	float * CalculateProbabilities(CCharVector &vector);
	float * CalculateProbabilities2(CCharVector &vector);
	void RecogniseUsingProbabilities();
//	void RecogniseUsingProbabilities2();
	float GetPobability(int charnum, int featureNo, int val);
	float GetPobability(int charnum, int featureNo, float val);
	int * FindHighest(float * prob, int size);
	int* FindHighest(float * prob, int size, int total);
//	int * FindHighest(float * prob, int size,int total);
	int GetCost(INFO highval,CImage *image);
	int GetFlexCost(INFO highval,CImage *image);
	int CompareImagesCentred(CImage* image1,CImage* image2);
	int CompareSSDImages(CImage* image1, CImage* image2,CPoint point1,CPoint point2);
	int CompareTubbs1Images(CImage* image1, CImage* image2,CPoint point1,CPoint point2);
	int CompareTubbs2Images(CImage* image1, CImage* image2,CPoint point1,CPoint point2);
	int CompareImagesFlexible(CImage* image1, CImage* image2,CPoint point1,CPoint point2, int totPix);
	int CompareImagesFlexible(CImage* image1,CImage* image2);
	int CompareImageInVectorWeight(CImage* image1,CImage* image2);
	int CompareWeightImages(CImage* image1, CImage* image2,CPoint point1,CPoint point2);
	char FindBestMatch(CImage* image);
	INFO FindBestMatch(CImage* image,int type,int MatchingType);
	int	ComputeTotalNumberofVectors();
	float CalculatePriorProbability(int charnum);
	float GetCombinedProbability(int charnum, int featureNo1, int val1, int featureNo2, int val2);
	float * CalculateDivisionsProbabilities(CCharVector &vector);
	float  CalculateDivisionsDistance(int charPos,CCharVector vector);
	float ComputeDistanceDiv(int charnum, int featureNo,int val);
	void ExportImagePlot();
	void ExportImagePlot2();
	void ExportImagePlot3();
	void ExportImagePlot4();
	void ExportImagePlot5();
	void CreateClassTree();
	void DeleteClassTree();
	void CreateCharLocationFile();
	bool IsNeighbourColor(CPoint point, int color, int** arrayvals);

	
	
	CTextLine * m_pLineHolder;
	int m_LineHolderIndex;

	//int m_FixSegmentation_Line_StartPos;
//	int m_FixSegmentation_Line_Size;


	bool m_ShowFix;
	bool m_ShowInBox;
	
	bool m_IsCharLocalized;
	bool m_IsCharSegmented;
	bool m_bIsPreprocessed;
	bool m_bIsLineSegmented;
	bool m_bIsImageRecognized;
	bool m_bIsImageImproved;
	bool m_bIsTableCreated;
	bool m_bIsHistoStretched;

	CCharVector** m_pCharVector;

	int m_CharVectorIndex;
	int m_CharAverageWidth;
	CCharVector** m_pCharCapitalVector;
	int m_CharCapitalVectorIndex;

	//vector<CClassifierVector> m_pProbabilityVector;
	CClassifierVector m_pProbabilityVector[100];
	int m_ProbabilityVectorIndex;
	typedef struct CCOUNT
	{
		int val;
		float val1;
		int count;
	};
	typedef struct CFEATURE
	{
		int		pos;
		CCOUNT	*Values;
		int		noOfValues;
		int		total;
	};
	typedef struct CTYPE
	{
		int			total;
		int			type[5];
	};
	typedef struct CTABLE
	{
		char		name;
		bool		capital;
		CFEATURE *	Features;
		int			TotalVectors;
	};

	typedef struct ItemType
	{
		char level;
		char value;
		char Name[10];
		int n_chars;
		char* pchars;
	}ItemType;

	typedef struct AddressType
	{
		char charName;
		CString charAddress;
		int size;
	};
	
	typedef struct STATISTICS
	{
		int total_chars;
		int total_correct;
		int total_incorrect;
		int total_noise;
		int correct_in_blue;
		int correct_in_red;
		int incorrect_in_blue;
		int incorrect_in_red;
		int incorrect_but_second;
		int missed_chars;
	};

	typedef struct WRONG_CHAR
	{
		char CharName;
		int total;
		WRONG_CHAR * OtherChars;
		int total_otherChars;
	};
	
	typedef struct WRONG_CHAR_STATS
	{
		WRONG_CHAR * worng_chars;
		int total_chars;
		int total_wrong_chars;
	};

	
	WRONG_CHAR_STATS m_WrongCharStats;
	WRONG_CHAR_STATS m_CharacterStats;
	STATISTICS m_FTR_Statistics;
	STATISTICS m_Refined1_Statistics;
	STATISTICS m_Refined2_Statistics;

	bool m_IsFTRStatistics;
	bool m_IsRefined1Statistics;
	bool m_IsRefined2Statistics;
	bool m_ImportedGroundTruth;
	bool is_charAddress;
	int m_ntree;
	ItemType TreeArray[6][6][6][6][6][6][6];
	int n_LocatedChars;
	AddressType m_CharAddresses[100];
	int		m_totalVectorsCT;
	CTABLE *  m_ppContingencyTable;
	bool m_bNeedSaveClassifier;
	CImage * m_pRecognizedImage;
	int ** m_2DCharPlot;
	CTYPE ** m_2DCharPlotType;
	int m_nFeatures;
	CString m_PerfectImagePath;
	CString m_ImagePath;
	CString m_CharFilePath;
	CString m_RecognizedImagePath;
	CString m_CapitalsFilePath;
	CString m_PerfectCapitalImagePath;
	CString m_ExportedCharsPath;
	CString m_ClassifiedCharsPath;
	CString m_ClassifierFilePath;
	CString m_PerfectCharactersLocation;
	CString m_TreePathName;
	CString m_CharLocationFile;
	
private:
	void ClearDocument();

protected:
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnFileImportimage();

	void OnFileImportimageToAdd();
	afx_msg void OnFileExportimage();
	afx_msg void OnUpdateEditUndo(CCmdUI *pCmdUI);
	afx_msg void OnEditUndo();
};


#endif // DOCIMANDOC_H
