// XMLDocument.cpp
// © 2004,2005 David Bridson
// Licenced under the GPL
#include "stdafx.h"
#include "XMLDocument.h"

#include "ExtraString.h"
#include "XMLDefinition.h"
#include "XMLDocType.h"
#include "XMLElement.h"

// CONSTRUCTION

CXMLDocument::CXMLDocument()
{
	m_pBaseElement = NULL;
	m_pDefinition = NULL;
	m_pDocType = NULL;
}

CXMLDocument::~CXMLDocument()
{
	delete m_pBaseElement;
	delete m_pDefinition;
	delete m_pDocType;
}

// METHODS

unsigned int CXMLDocument::ChangeState(unsigned int NewState)
{
/*
	if(NewState == ST_INITIAL_WHITE_SPACE)
		printf("Document: Changing State To ST_INITIAL_WHITE_SPACE\n");
	else if(NewState == ST_TAG_START_MARKER)
		printf("Document: Changing State To ST_TAG_START_MARKER\n");
	else if(NewState == ST_TAG_INITIAL_WHITE_SPACE)
		printf("Document: Changing State To ST_TAG_INITIAL_WHITE_SPACE\n");
	else if(NewState == ST_TAG_RECURSING)
		printf("Document: Changing State To ST_TAG_RECURSING\n");
	else if(NewState == ST_OTHER_CONTENT)
		printf("Document: Changing State To ST_OTHER_CONTENT\n");
	else if(NewState == ST_UNDEFINED)
		printf("Document: Changing State To ST_UNDEFINED\n");
*/
	return NewState;
}

bool CXMLDocument::CreateFromFile(FILE * File)
{
	char CurrentChar;
	int CurrentState = ST_INITIAL_WHITE_SPACE;
	int Backup;
	
	delete m_pBaseElement;
	m_pBaseElement = NULL;
	
	while(!feof(File))
	{
		CurrentChar = fgetc(File);
//		printf("CurrentChar: %c\t%d\n",CurrentChar,CurrentChar);
		
		if(CExtraString::IsWSpace(CurrentChar) || CurrentChar == '\n' || CurrentChar == 10 || CurrentChar == 13)
		{
			if(CurrentState == ST_INITIAL_WHITE_SPACE)
				; // Do Nothing
			else if(CurrentState == ST_TAG_START_MARKER)
			{
				CurrentState = ChangeState(ST_TAG_INITIAL_WHITE_SPACE);
				Backup++;
			}
			else if(CurrentState == ST_TAG_INITIAL_WHITE_SPACE)
			{
				Backup++;
			}
			else if(CurrentState == ST_TAG_RECURSING)
			{
				CurrentState = ChangeState(ST_OTHER_CONTENT);
			}
			else if(CurrentState == ST_OTHER_CONTENT)
			{
				// DO NOTHING
			}
			else if(CurrentState == ST_IGNORE_TAG)
			{
				// DO NOTHING
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == '<')
		{
			if(CurrentState == ST_INITIAL_WHITE_SPACE)
			{
				CurrentState = ChangeState(ST_TAG_START_MARKER);
				Backup = 1;
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CExtraString::IsAlphaNumeric(CurrentChar))
		{
			if(CurrentState == ST_TAG_INITIAL_WHITE_SPACE || CurrentState == ST_TAG_START_MARKER)
			{
				// BASE ELEMENT
				
				Backup++;
				CurrentState = ChangeState(ST_TAG_RECURSING);
				fseek(File,-Backup,SEEK_CUR);
				
				m_pBaseElement = new CXMLElement();
				if(m_pBaseElement->CreateFromFile(File) == false)
				{
					fprintf(stderr,"CXMLDocument::CreateFromFile: m_pBaseElement->CreateFromFile returned false\n");
					return false;
				}
			}
			else if(CurrentState == ST_IGNORE_TAG)
			{
				// DO NOTHING
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == '!')
		{
			if(CurrentState == ST_TAG_INITIAL_WHITE_SPACE || CurrentState == ST_TAG_START_MARKER)
			{
				CurrentState = ChangeState(ST_IGNORE_TAG);
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == '?' || CurrentChar == '!')
		{
			if(CurrentState == ST_TAG_START_MARKER || CurrentState == ST_IGNORE_TAG)
			{
				CurrentState = ChangeState(ST_IGNORE_TAG);
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == '>')
		{
			if(CurrentState == ST_IGNORE_TAG)
			{
				CurrentState = ChangeState(ST_INITIAL_WHITE_SPACE);
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == '=' || CurrentChar == '\"' || CurrentChar == '.' || CurrentChar == '-' || CurrentChar == ':' || CurrentChar == '/' || CurrentChar == '~' || CurrentChar == '_' || CurrentChar == '[' || CurrentChar == ']')
		{
			if(CurrentState == ST_IGNORE_TAG)
			{
				// DO NOTHING
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
				return false;
			}
		}
		else if(CurrentChar == EOF)
		{
			break;
		}
		else
		{
			CurrentState = ChangeState(ST_UNDEFINED);
			return false;
		}
	}
	
	if(m_pBaseElement == NULL)
		return false;
	else
		return true;
}

bool CXMLDocument::CreateFromFile(const char * FileName)
{
	FILE * File = fopen(FileName,"r");
	
	if(File == NULL)
	{
		fprintf(stderr,"CreateFromFile(char *): Couldn't Open File\n");
		return false;
	}
	
	if(CreateFromFile(File)==false)
	{
		fprintf(stderr,"CreateFromFile(char *): CreateFromFile(FILE *) Returned false\n");
		fclose(File);
		return false;
	}
	
	fclose(File);
	
	return true;
}

CXMLElement * CXMLDocument::GetBaseElement()
{
	return m_pBaseElement;
}

void CXMLDocument::PrintState(unsigned int State)
{
/*
	if(State == ST_INITIAL_WHITE_SPACE)
		printf("ST_INITIAL_WHITE_SPACE\n");
	else if(State == ST_TAG_START_MARKER)
		printf("ST_TAG_START_MARKER\n");
	else if(State == ST_UNDEFINED)
		printf("ST_UNDEFINED\n");
	else if(State == ST_TAG_INITIAL_WHITE_SPACE)
		printf("ST_TAG_INITIAL_WHITE_SPACE\n");
	else if(State == ST_TAG_RECURSING)
		printf("ST_TAG_RECURSING\n");
*/
}

void CXMLDocument::SetBaseElement(CXMLElement * NewBaseElement)
{
	m_pBaseElement = NewBaseElement;
}

void CXMLDocument::SetDefinition(CXMLDefinition * NewDefinition)
{
	m_pDefinition = NewDefinition;
}

void CXMLDocument::SetDocType(CXMLDocType * NewDocType)
{
	m_pDocType = NewDocType;
}

bool CXMLDocument::WriteToFile(FILE * File)
{

	if(m_pDefinition != NULL)
	{
		m_pDefinition->WriteToFile(File);
		fprintf(File,"\n");
	}

	if(m_pDocType != NULL)
	{
		m_pDocType->WriteToFile(File);
		fprintf(File,"\n\n");
	}
	
	if(m_pBaseElement != NULL)
	{
		m_pBaseElement->WriteToFile(File,0);
		fprintf(File,"\n");
	}

	return true;
}
bool CXMLDocument::WriteToFile(char * FileName)
{
	bool bRet;

	FILE * f = fopen(FileName, "w");
	bRet = WriteToFile(f);
	fclose(f);

	return bRet;
}



