// XMLElement.cpp
// © 2004,2005 David Bridson
// Licenced under the GPL
#include "stdafx.h"
#include "XMLElement.h"

#include <string.h>

#include "ExtraString.h"
#include "XMLAttribute.h"

// CONSTRUCTION

CXMLElement::CXMLElement()
{
	m_nAttributes = 0;
	m_pAttributes = NULL;
	m_nChildren   = 0;
	m_pChildren   = NULL;
	m_bEmpty      = false;
	m_sName       = NULL;
}

CXMLElement::CXMLElement(char * NewName)
{
	m_nAttributes = 0;
	m_pAttributes = NULL;
	m_nChildren   = 0;
	m_pChildren   = NULL;
	m_bEmpty      = false;
	m_sName       = NULL;
	SetName(NewName);
}

CXMLElement::~CXMLElement()
{
	int i;

	for(i = 0; i < m_nChildren; i++)
	{
		delete m_pChildren[i];
	}

	for(i = 0; i < m_nAttributes; i++)
	{
		delete m_pAttributes[i];
	}

	delete  []m_pAttributes;
	delete  []m_pChildren;
	delete  []m_sName;
}

// METHODS

bool CXMLElement::AddAttribute(CXMLAttribute * NewAtt)
{
	CXMLAttribute ** temp;
	temp = new CXMLAttribute * [m_nAttributes + 1];
	if(temp == NULL)
	{
		fprintf(stderr,"AddAttribute: Couldn't Increase Memory Allocation For m_pAttributes\n");
		return false;
	}
	memcpy(temp,m_pAttributes,sizeof(CXMLAttribute *) * m_nAttributes);
	temp[m_nAttributes] = NewAtt;
	delete [] m_pAttributes;
	m_pAttributes = temp;
	m_nAttributes++;
}

bool CXMLElement::AddChild(CXMLElement * NewChild)
{
	CXMLElement ** temp;
	temp = new CXMLElement * [m_nChildren + 1];
	if(temp == NULL)
	{
		fprintf(stderr,"AddChild: Couldn't Increase Memory Allocation For m_pChildren\n");
		return false;
	}
	
	memcpy(temp,m_pChildren,sizeof(CXMLElement *) * m_nChildren);
	temp[m_nChildren] = NewChild;
	delete [] m_pChildren;
	m_pChildren = temp;
	m_nChildren++;
}

unsigned int CXMLElement::ChangeState(unsigned int NewState)
{
/*
	if(NewState == ST_START)
		printf("Changing State To ST_START\n");
	else if(NewState == ST_OPEN_TAG_START_MARKER)
		printf("Changing State To ST_OPEN_TAG_START_MARKER\n");
	else if(NewState == ST_OPEN_TAG_INITIAL_WHITE_SPACE)
		printf("Changing State To ST_OPEN_TAG_INITIAL_WHITE_SPACE\n");
	else if(NewState == ST_OPEN_TAG_NAME)
		printf("Changing State To ST_OPEN_TAG_NAME\n");
	else if(NewState == ST_OPEN_TAG_INNER_WHITE_SPACE)
		printf("Changing State To ST_OPEN_TAG_INNER_WHITE_SPACE\n");
	else if(NewState == ST_OPEN_TAG_RECURSING_ATT)
		printf("Changing State To ST_OPEN_TAG_RECURSING_ATT\n");
	else if(NewState == ST_OPEN_TAG_EMPTY_MARKER)
		printf("Changing State To ST_OPEN_TAG_EMPTY_MARKER\n");
	else if(NewState == ST_OPEN_TAG_END_MARKER)
		printf("Changing State To ST_OPEN_TAG_END_MARKER\n");
	else if(NewState == ST_TAG_INNER_CONTENT)
		printf("Changing State To ST_TAG_INNER_CONTENT\n");
	else if(NewState == ST_OTHER_TAG_START_MARKER)
		printf("Changing State To ST_OTHER_TAG_START_MARKER\n");
	else if(NewState == ST_OTHER_TAG_INITIAL_WHITE_SPACE)
		printf("Changing State To ST_OTHER_TAG_INITIAL_WHITE_SPACE\n");
	else if(NewState == ST_OTHER_TAG_CLOSE_MARKER)
		printf("Changing State To ST_OTHER_TAG_CLOSE_MARKER\n");
	else if(NewState == ST_OTHER_TAG_INNER_WHITE_SPACE)
		printf("Changing State To ST_OTHER_TAG_INNER_WHITE_SPACE\n");
	else if(NewState == ST_OTHER_TAG_NAME)
		printf("Changing State To ST_OTHER_TAG_NAME\n");
	else if(NewState == ST_OTHER_TAG_ENDING_WHITE_SPACE)
		printf("Changing State To ST_OTHER_TAG_ENDING_WHITE_SPACE\n");
	else if(NewState == ST_OTHER_TAG_END_MARKER)
		printf("Changing State To ST_OTHER_TAG_END_MARKER\n");
	else if(NewState == ST_TAG_RECURSING_CHILD)
		printf("Changing State To ST_TAG_RECURSING_CHILD\n");
	else if(NewState == ST_UNDEFINED)
		printf("Changing State To ST_UNDEFINED\n");
*/
	return NewState;
}

bool CXMLElement::CreateFromFile(FILE * File)
{
	char CurrentChar;
	unsigned int CurrentState = ST_START;
	
	int Backup;
	int DataAlloc = DATA_INIT;
	char * Data = NULL;
	
	m_bEmpty = false;
	
	while(!feof(File))
	{
		CurrentChar = fgetc(File);
//		printf("CurrentChar: %c\t%d\n",CurrentChar,CurrentChar);
		
		if(CurrentChar == '<')
		{
			if(CurrentState == ST_START)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_START_MARKER);
			}
			else if(CurrentState == ST_OPEN_TAG_END_MARKER)
			{
				Backup = 1; // NEED TO FIND WHETHER CLOSE TAG OR CHILD
				CurrentState = ChangeState(ST_OTHER_TAG_START_MARKER);
			}
			else if(CurrentState == ST_TAG_RECURSING_CHILD)
			{
				Backup = 1;
				CurrentState = ChangeState(ST_OTHER_TAG_START_MARKER);
			}
			else if(CurrentState == ST_TAG_INNER_CONTENT)
			{
				Backup = 1; // NEED TO FIND WHETHER CLOSE TAG OR CHILD
				CurrentState = ChangeState(ST_OTHER_TAG_START_MARKER);
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
			}
		}
		else if(CurrentChar == '>')
		{
			if
			(
				CurrentState == ST_OPEN_TAG_INNER_WHITE_SPACE ||
				CurrentState == ST_OPEN_TAG_RECURSING_ATT
			)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_END_MARKER);
				if(m_bEmpty == true)
				{
					delete Data;
					return true;
				}
			}
			else if(CurrentState == ST_OPEN_TAG_NAME)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_END_MARKER);
				delete [] m_sName;
				m_sName = new char[strlen(Data) + 1];
				if(m_sName == NULL)
				{
					fprintf(stderr,"Couldn't Allocate Memory for m_sName\n");
					delete Data;
					return false;
				}
				memcpy(m_sName,Data,sizeof(char) * (strlen(Data) + 1));
				DataAlloc = 0;
				delete [] Data;
				Data = NULL;
//				printf("m_sName = %s\n",m_sName);
			}
			else if(CurrentState == ST_OPEN_TAG_EMPTY_MARKER)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_END_MARKER);
				delete Data;
				return true;
			}
			else if(CurrentState == ST_OTHER_TAG_NAME)
			{
				if(strcmp(Data,m_sName) != 0)
				{
					fprintf(stderr,"Some Kind Of Error: Start Tag (%s) Doesn't Match End Tag (%s)\n",m_sName,Data);
					delete Data;
					return false;
				}
				CurrentState = ChangeState(ST_OTHER_TAG_END_MARKER);
				delete [] Data;
				return true; // FOUND MATCHING CLOSING TAG
			}
			else if(CurrentState == ST_OTHER_TAG_INNER_WHITE_SPACE)
			{
				delete [] Data;
				return true;
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
			}
		}
		else if(CurrentChar == '/')
		{
			if
			(
				CurrentState == ST_OTHER_TAG_START_MARKER || 
				CurrentState == ST_OTHER_TAG_INITIAL_WHITE_SPACE
			)
			{
				CurrentState = ChangeState(ST_OTHER_TAG_CLOSE_MARKER);
			}
			else if(CurrentState == ST_OPEN_TAG_INNER_WHITE_SPACE)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_EMPTY_MARKER);
				m_bEmpty = true;
			}
			else if(CurrentState == ST_OPEN_TAG_RECURSING_ATT)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_EMPTY_MARKER);
			}
		}
		else if(CExtraString::IsWSpace(CurrentChar) || CurrentChar == '\n' || CurrentChar == 10 || CurrentChar == 13)
		{
			if(CurrentState == ST_OPEN_TAG_START_MARKER)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_INITIAL_WHITE_SPACE);
			}
			else if(CurrentState == ST_OPEN_TAG_INITIAL_WHITE_SPACE)
				; // DO NOTHING;
			else if(CurrentState == ST_OPEN_TAG_INNER_WHITE_SPACE)
				; // DO NOTHING;
			else if(CurrentState == ST_OPEN_TAG_NAME)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_INNER_WHITE_SPACE);
				delete [] m_sName;
				m_sName = new char[strlen(Data) + 1];
				if(m_sName == NULL)
				{
					fprintf(stderr,"Couldn't Allocate Memory for m_sName\n");
					delete Data;
					return false;
				}
				memcpy(m_sName,Data,sizeof(char) * (strlen(Data) + 1));
				DataAlloc = 0;
				delete [] Data;
				Data = NULL;
//				printf("m_sName = %s\n",m_sName);
			}
			else if(CurrentState == ST_OPEN_TAG_RECURSING_ATT)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_INNER_WHITE_SPACE);
			}
			else if(CurrentState == ST_OPEN_TAG_END_MARKER)
			{
				CurrentState = ChangeState(ST_TAG_INNER_CONTENT);
			}
			else if(CurrentState == ST_TAG_INNER_CONTENT)
			{
				; // DO NOTHING
			}
			else if(CurrentState == ST_OTHER_TAG_START_MARKER)
			{
				Backup++;
				CurrentState = ChangeState(ST_OTHER_TAG_INITIAL_WHITE_SPACE);
			}
			else if(CurrentState == ST_OTHER_TAG_INITIAL_WHITE_SPACE)
			{
				Backup++;
			}
			else if(CurrentState == ST_OTHER_TAG_CLOSE_MARKER)
			{
				CurrentState = ChangeState(ST_OTHER_TAG_INNER_WHITE_SPACE);
			}
			else if(CurrentState == ST_OTHER_TAG_INNER_WHITE_SPACE)
				; // DO NOTHING;
			else if(CurrentState == ST_OTHER_TAG_NAME)
			{
				CurrentState = ChangeState(ST_OTHER_TAG_INNER_WHITE_SPACE);
				if(strcmp(m_sName,Data) != 0)
				{
					fprintf(stderr,"Error: Start Tag (%s) and End Tag (%s) Don't Match\n",m_sName,Data);
					delete Data;
					return false;
				}
			}
			else if(CurrentState == ST_TAG_RECURSING_CHILD)
			{
				CurrentState = ChangeState(ST_TAG_INNER_CONTENT);
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
			}
		}
		else if(CExtraString::IsAlphaNumeric(CurrentChar) || CurrentChar == '_')
		{
			if
			(
				CurrentState == ST_OPEN_TAG_START_MARKER ||
				CurrentState == ST_OPEN_TAG_INITIAL_WHITE_SPACE
			)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_NAME);
				delete [] Data;
				DataAlloc = DATA_INIT;
				Data = new char[DataAlloc];
				if(Data == NULL)
				{
					fprintf(stderr,"CreateFromFile: Couldn't Allocate Memory for Name\n");
					delete Data;
					return false;
				}
				memset(Data,0,sizeof(char) * DataAlloc);
				Data[0] = CurrentChar;
//				printf("Data = %s\n",Data);
			}
			else if(CurrentState == ST_OPEN_TAG_NAME)
			{
				if(strlen(Data) == DataAlloc - 1)
				{
					char * temp = new char[DataAlloc + DATA_INCR];
					if(temp == NULL)
					{
						fprintf(stderr,"CreateFromFile: Couldn't ReAllocate Memory for Name\n");
						delete Data;
						return false;
					}
					memcpy(temp,Data,sizeof(char) * DataAlloc);
					memset(&(temp[DataAlloc]),0,sizeof(char) * DATA_INCR);
					DataAlloc += DATA_INCR;
					delete [] Data;
					Data = temp;
				}
				Data[strlen(Data)] = CurrentChar;
//				printf("Data = %s\n",Data);
			}
			else if(CurrentState == ST_OPEN_TAG_INNER_WHITE_SPACE)
			{
				CurrentState = ChangeState(ST_OPEN_TAG_RECURSING_ATT);
				fseek(File,-1,SEEK_CUR);
				CXMLAttribute ** temp = new CXMLAttribute * [m_nAttributes + 1];
				if(temp == NULL)
				{
					fprintf(stderr,"CreateFromFile: Couldn't Increase Memory Allocation For m_pAttributes\n");
					delete Data;
					return false;
				}
				memcpy(temp,m_pAttributes,sizeof(CXMLAttribute *) * m_nAttributes);
				delete [] m_pAttributes;
				temp[m_nAttributes] = new CXMLAttribute();
				if(temp[m_nAttributes] == NULL)
				{
					fprintf(stderr,"CreateFromFile: Couldn't Allocate Memory For m_pAttributes[%d]\n",m_nAttributes);
					delete Data;
					return false;
				}
				temp[m_nAttributes]->CreateFromFile(File);
				m_pAttributes = temp;
				m_nAttributes++;
			}
			else if
			(
				CurrentState == ST_OTHER_TAG_CLOSE_MARKER ||
				CurrentState == ST_OTHER_TAG_INNER_WHITE_SPACE
			)
			{
				CurrentState = ChangeState(ST_OTHER_TAG_NAME);
				delete [] Data;
				DataAlloc = DATA_INIT;
				Data = new char[DataAlloc];
				if(Data == NULL)
				{
					fprintf(stderr,"CreateFromFile: Couldn't Allocate Memory for Name\n");
					delete Data;
					return false;
				}
				memset(Data,0,sizeof(char) * DataAlloc);
				Data[0] = CurrentChar;
//				printf("Data = %s\n",Data);
			}
			else if(CurrentState == ST_OTHER_TAG_NAME)
			{
				if(strlen(Data) == DataAlloc - 1)
				{
					char * temp = new char[DataAlloc + DATA_INCR];
					if(temp == NULL)
					{
						fprintf(stderr,"CreateFromFile: Couldn't ReAllocate Memory for Name\n");
						delete Data;
						return false;
					}
					memcpy(temp,Data,sizeof(char) * DataAlloc);
					memset(&(temp[DataAlloc]),0,sizeof(char) * DATA_INCR);
					DataAlloc += DATA_INCR;
					delete [] Data;
					Data = temp;
				}
				Data[strlen(Data)] = CurrentChar;
//				printf("Data = %s\n",Data);
			}
			else if
			(
				CurrentState == ST_OTHER_TAG_START_MARKER ||
				CurrentState == ST_OTHER_TAG_INITIAL_WHITE_SPACE
			)
			{
				Backup ++;
				CurrentState = ChangeState(ST_TAG_RECURSING_CHILD);
				fseek(File,-Backup,SEEK_CUR);
				CXMLElement ** temp;
				temp = new CXMLElement * [m_nChildren + 1];
				memcpy(temp,m_pChildren,sizeof(CXMLElement *) * m_nChildren);
				delete [] m_pChildren;
				temp[m_nChildren] = new CXMLElement();

				if(temp[m_nChildren] == NULL)
				{
					fprintf(stderr,"CreateFromFile(FILE *): Couldn't Allocate Memory For temp[%d]\n",m_nChildren);
					delete Data;
					return false;
				}

				if(temp[m_nChildren]->CreateFromFile(File)==false)
				{
					fprintf(stderr,"CreateFromFile(FILE *): CreateFromFile(FILE *) returned false\n");
					delete Data;
					return false;
				}
				m_pChildren = temp;
				m_nChildren ++;
			}
			else
			{
				CurrentState = ChangeState(ST_UNDEFINED);
			}
		}
		else
		{
			CurrentState = ChangeState(ST_UNDEFINED);
		}
	}
/*
	static const unsigned int ST_START                         = 1354018976;
	static const unsigned int ST_OPEN_TAG_START_MARKER         =   87168123;
	static const unsigned int ST_OPEN_TAG_INITIAL_WHITE_SPACE  =  275325964;
	static const unsigned int ST_OPEN_TAG_NAME                 = 1393595380;
	static const unsigned int ST_OPEN_TAG_INNER_WHITE_SPACE    = 1684203569;
	static const unsigned int ST_OPEN_TAG_RECURSING_ATT        =  487082390;
	static const unsigned int ST_OPEN_TAG_EMPTY_MARKER         =  425989189;
	static const unsigned int ST_OPEN_TAG_END_MARKER           = 1484862058;
	static const unsigned int ST_TAG_INNER_CONTENT             = 1389249562;
	static const unsigned int ST_OTHER_TAG_START_MARKER        = 1810442670;
	static const unsigned int ST_OTHER_TAG_INITIAL_WHITE_SPACE =   64708011;
	static const unsigned int ST_OTHER_TAG_CLOSE_MARKER        = 1946472750;
	static const unsigned int ST_OTHER_TAG_INNER_WHITE_SPACE   = 1003976267;
	static const unsigned int ST_OTHER_TAG_NAME                = 1749658885;
	static const unsigned int ST_OTHER_TAG_ENDING_WHITE_SPACE  =  921418963;
	static const unsigned int ST_OTHER_TAG_END_MARKER          =  655949530;
	static const unsigned int ST_TAG_RECURSING_CHILD           =  449323449;
	static const unsigned int ST_UNDEFINED                     =  498509274;
*/

	delete [] Data;

	return true;
}

CXMLAttribute * CXMLElement::GetAttribute(int No)
{
	if(No < 0)
	{
		fprintf(stderr,"GetAttribute: No < 0\n");
		return NULL;
	}
	else if(No >= m_nAttributes)
	{
		fprintf(stderr,"GetAttribute: No >= m_nAttributes\n");
		return NULL;
	}
	else if(m_pAttributes == NULL)
	{
		fprintf(stderr,"GetAttribute: m_pAttributes == NULL\n");
		return NULL;
	}
		return m_pAttributes[No];
}

CXMLElement * CXMLElement::GetChild(int No)
{
	if(No < 0)
	{
		fprintf(stderr,"GetChild: No < 0\n");
		return NULL;
	}
	else if(No >= m_nChildren)
	{
		fprintf(stderr,"GetChild: No >= m_nChildren\n");
		return NULL;
	}
	else if(m_pChildren == NULL)
	{
		fprintf(stderr,"GetChild: m_pChildren == NULL\n");
		return NULL;
	}
	else
		return m_pChildren[No];
}

char * CXMLElement::GetName()
{
//	char * ret = new char[strlen(m_sName) + 1];
//	if(ret == NULL)
//	{
//		fprintf(stderr,"GetName: Couldn't Allocate Memory For ret\n");
//		return false;
//	}
//	memcpy(ret,m_sName,sizeof(char) * (strlen(m_sName) + 1));
//	return ret;
	return m_sName;
}

int CXMLElement::GetNoAttributes()
{
	return m_nAttributes;
}

int CXMLElement::GetNoChildren()
{
	return m_nChildren;
}

void CXMLElement::SetEmpty(bool NewEmpty)
{
	m_bEmpty = NewEmpty;
}

bool CXMLElement::SetName(char * NewName)
{
	delete [] m_sName;
	m_sName = new char[strlen(NewName) + 1];
	if(m_sName == NULL)
	{
		fprintf(stderr,"SetName: Couldn't Allocate Memory For m_sName\n");
		return false;
	}
	memcpy(m_sName,NewName,sizeof(char) * (strlen(NewName) + 1));
	return true;
}

bool CXMLElement::WriteToFile(FILE * File, int Indent)
{
	int i;
	
	for(i = 0; i < Indent; i++)
		fprintf(File,"\t");
	
	fprintf(File,"<%s",m_sName);
	
	if(m_pAttributes != NULL)
	{
		for(i = 0; i < m_nAttributes; i++)
		{
			fprintf(File," ");
			m_pAttributes[i]->WriteToFile(File);
		}
	}
	
	if(m_bEmpty)
	{
		fprintf(File," />");
	}
	else
	{
		fprintf(File,">\n");
		
		if(m_pChildren != NULL)
		{
			for(i = 0; i < m_nChildren; i++)
			{
				m_pChildren[i]->WriteToFile(File,Indent + 1);
				fprintf(File,"\n");
			}
		}
			
		for(i = 0; i < Indent; i++)
			fprintf(File,"\t");
		
		fprintf(File,"</%s>",m_sName);
	}

	return true;
}
