// BiLevelImage.h: interface for the CBiLevelImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BILEVELIMAGE_H__DFA1CBCA_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
#define AFX_BILEVELIMAGE_H__DFA1CBCA_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Image.h"

#include "LibraryDeclaration.h"

#define BILEVEL_BLACK	0x00
#define BILEVEL_WHITE	0x01

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CBiLevelImage : virtual public CImage
{
	DECLARE_SERIAL(CBiLevelImage)

	friend class CHiColorImage;
	friend class CLoColorImage;
	friend class CGreyScaleImage;
public:
	//Initialization
	CBiLevelImage();
	virtual ~CBiLevelImage();
	virtual BOOL	Create(int Width, int Height, COLORREF BackColour, LPCTSTR Name);
	virtual BOOL	CreateFromImage(CImage *pImage, BYTE Flags = 0x00);
	virtual BOOL	ExportTIFF(LPCTSTR FileName);
	virtual BOOL	CreateFromTIFF(TIFF * Tif, LPCTSTR Name = NULL);
	//Drawing Operations
protected:
	virtual void	FillBitmapInfo();
public:
	virtual BYTE*	UpdateNormal();
	virtual BYTE*	UpdateDimmed();
	//Operations
	virtual void	GetProjection(int Left, int Top, int Width, int Height, long * VertProfile = NULL, long * HorzProfile = NULL, BYTE Flags = 0x00);
	virtual void	Clear(int R = 255, int G = 255, int B = 255);
	virtual void	Rotate(double Angle, COLORREF BackColour = RGB(255,255,255), BOOL CropResultToOriginalSize = FALSE);
	virtual void	Rotate(CRect Rect, double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE);
	virtual void	Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL);
	virtual void	Crop(int Left, int Top, int Width, int Height);
#if defined (IMAGE_COMPONENT_SUPPORT)
	virtual BOOL	CCAnalysis(int Connectivity = FCC_C8);
	virtual BOOL	BBAnalysis(RECT ** pRect, int * nRect, CRect SearchRect = CRect(-1,-1,-1,-1), int Connectivity = FCC_C8);
	virtual void	DrawComponents(COLORREF Colour = RGB(0,0,0));
#endif
	virtual void	SaltPepperRemoval(int Size = 3);
	virtual void	GetNegative();
	virtual BOOL	ReSample(int NewWidth, int NewHeight, int NewDPI);
	virtual BOOL	CreateDuplicate(CImage *pImage, BOOL ReferenceArrays);
	virtual BOOL	DiscardBackup();
	virtual void	ApplyAdvancedTreshold( int * Range);
	virtual void	RestoreBackup();
	virtual BOOL	CreateBackup();
	virtual void	ApplyRankOrderFilter(BYTE Type, int Size = 3, BYTE Channel = CH_RGB);
	//Attributes
	virtual COLORREF	GetRGBColor(int x, int y);
	virtual void	SetRGBColor(int x, int y, COLORREF col);
	virtual BYTE	GetRed(int x, int y);
	virtual BYTE	GetGreen(int x, int y);
	virtual BYTE	GetBlue(int x, int y);
	virtual void	SetRed(int x, int y, BYTE val);
	virtual void	SetGreen(int x, int y, BYTE val);
	virtual void	SetBlue(int x, int y, BYTE val);

	//Utility functions
protected:
	inline int	GetBytesPerLine(int Width) {return ((Width * 8 + 31) & ~31) /8;} //This uses bpp = 8 because internally the image is stored as an array of bytes (not bits)
	inline int	GetPackedBytesPerLine(int Width) {return ((Width * m_BitsPerPixel + 31) & ~31) /8;} //This returns the expected bpp for the image if stored in a bit array

protected:
	//Initialization
	void			FreeMem();
	//Operations
	virtual void	OverlayColor(int x, int y, int Red, int Green, int Blue, BYTE *OutputArray, int WidthBytes);
	//Variables
	BYTE * m_Array;
	BYTE ** m_LineArray;
	BYTE * m_BackupArray;
	BYTE ** m_BackupLineArray;
public:
	virtual void Serialize(CArchive& ar);
private:
	void	UnPackBitArray(BYTE * BitArray);
	void	PackByteArray(BYTE * BitArray);
	void	PackByteArray_InvertY(BYTE * BitArray);

	inline BYTE PrivateGetByte(int x, int y)
		{return m_LineArray[y][x];}

	inline void PrivateSetByte(int x, int y, BYTE val)
		{m_LineArray[y][x] = val;}

};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_BILEVELIMAGE_H__DFA1CBCA_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
