// Clustering3D.h: interface for the CClustering3D class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CLUSTERING3D_H__INCLUDED_)
#define AFX_CLUSTERING3D_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define CLUSTERING_CLASSES_VERSION 0x0100 //High Version = 02 Low Version = 01, variable is WORD
#define CLUSTERING_ABSOLUTE_MAX_ITERATIONS (int)(65535)

//Auto Initialize Methods
#define CLUST_AIM_RANDOM_PTS	0x01
#define CLUST_AIM_RAMDOM		0x02
#define CLUST_AIM_POINT_GROUPS	0x04

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CClustering3D : public CObject  
{
	DECLARE_SERIAL(CClustering3D)

	typedef struct tagFPoint //angles given in radii
	{
		float x;
		float y;
		float z;
	} FPoint;

public:
	//Initialization
	CClustering3D();
	virtual ~CClustering3D();

	//Operations
	virtual int		AddPoint(float x, float y, float z);
	int				PerformClustering(float LeastChange = 1.0f, int MaxIterations = 100);
	virtual void	InitializeCluster(int ClusterID, float x, float y, float z);
	void			SetK(int K);
	int				GetK();
	virtual BOOL	GetClusterInfo(int ClusterID, float *x, float *y, float *z, int *size);
	virtual int		HypotheticAssignment(float x, float y, float z);
	void			AutoInitializeClusters(BYTE Method = CLUST_AIM_RANDOM_PTS);

	//Variables
	int			m_nPoints;
	FPoint *	m_Point;
	int			m_ArrayIncreamentStep;
	int			m_ArrayAllocated;
	int	*		m_Assignments;
	FPoint *	m_ClusterCenter;
	int	*		m_ClusterSize;
	int			m_nClusters;
private:
	void			AssignPointsToClusters();
	virtual void	ReCalculateClusterCenters();
	virtual float	DistanceSquared(FPoint p1, FPoint p2);

#ifdef _DEBUG
//Debug functions
public:
	virtual void	TraceClusterInfo(int c = -1);
	void			TracePoints();
	void			TracePointsToFile(LPCTSTR strFileName);
#endif
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif