#pragma once
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the COLOURSPACEDLL_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// COLOURSPACEDLL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef COLOURSPACEDLL_EXPORTS
#define COLOURSPACEDLL_API __declspec(dllexport)
#else
#define COLOURSPACEDLL_API __declspec(dllimport)
#endif

//Definitions
#define pi 3.1415926535897932384626433832795
#define CIE_NumberOfDataPoints 61
#define WD_NumberOfDataPoints 27
	//CheckVisualSimilarity check types
#define CVS_WAVELENGTH 1
#define CVS_LUMINANCE 2
#define CVS_ALL 4

#define CS_NONE					0x00
#define CS_OUTPUT_NORMALIZED	0x01
#define CS_OUTPUT_RANGED		0x02
#define CS_OUTPUT_RANGED255		0x04
#define CS_INPUT_NORMALIZED		0x08
#define CS_INPUT_RANGED			0x10
#define CS_INPUT_RANGED255		0x20

//internal structures
typedef struct
{
	double	Dl;
	double	Wl;
	int		StartHue;
	int		EndHue;
} HueDiscrimination;

//Global Variables
extern COLOURSPACEDLL_API int	 CIE_wl[CIE_NumberOfDataPoints];
extern COLOURSPACEDLL_API float	 CIE_x[CIE_NumberOfDataPoints];
extern COLOURSPACEDLL_API float	 CIE_y[CIE_NumberOfDataPoints];
extern COLOURSPACEDLL_API float	 CIE_z[CIE_NumberOfDataPoints];
extern COLOURSPACEDLL_API double CIE_angle[CIE_NumberOfDataPoints];
extern COLOURSPACEDLL_API int	 WD_wl[WD_NumberOfDataPoints];
extern COLOURSPACEDLL_API float	 WD_dl[WD_NumberOfDataPoints];
extern COLOURSPACEDLL_API HueDiscrimination	 HueD[256];
extern COLOURSPACEDLL_API float	WavelengthDiscriminationFactor;
extern COLOURSPACEDLL_API float	ChromaticityThresholds[256][3];
extern COLOURSPACEDLL_API int	LightnessThresholds[256];

//exported functions
COLOURSPACEDLL_API double CS_ComputeWavelength(float x, float y, float z);
COLOURSPACEDLL_API double CS_ComputeWavelength(double angle);
COLOURSPACEDLL_API double CS_ComputeAngleFromWavelength(double WL);
COLOURSPACEDLL_API void CS_WL2xyz(double WL, float *x, float *y, float *z);
COLOURSPACEDLL_API float CS_GetWavelengthDiscrimination(double WL);
COLOURSPACEDLL_API float CS_GetPosWavelengthDiscrimination(double WL);
COLOURSPACEDLL_API float CS_GetNegWavelengthDiscrimination(double WL);
COLOURSPACEDLL_API void CS_SetWhitePoint(float x, float y);
COLOURSPACEDLL_API void CS_GetWhitePoint(float *x, float *y);
COLOURSPACEDLL_API void CS_SetWhitePoint(float x, float y, float z);
COLOURSPACEDLL_API void CS_GetWhitePoint(float *x, float *y, float *z);
COLOURSPACEDLL_API void CS_RGB2XYZ(float R, float G, float B, float *X, float *Y, float *Z, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_XYZ2RGB(float X, float Y, float Z, float *R, float *G, float *B, BYTE Flags = CS_OUTPUT_RANGED);
COLOURSPACEDLL_API void CS_HLS2RGB(float h, float l, float s, float *r, float *g, float *b, BYTE Flags = CS_INPUT_RANGED | CS_OUTPUT_RANGED);
COLOURSPACEDLL_API void CS_RGB2HLS(float red, float green, float blue, float *Hue, float *Lum, float *Sat, BYTE Flags = CS_INPUT_RANGED | CS_OUTPUT_RANGED);
COLOURSPACEDLL_API void CS_xyz2sRGB(float x, float y, float z, float *R, float *G, float *B, BYTE Flags = CS_OUTPUT_RANGED);
COLOURSPACEDLL_API void CS_sRGB2xyz(float R, float G, float B, float *x, float *y, float *z, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_xyz2HLS(float x, float y, float z, float *H, float *L, float *S, BYTE Flags = CS_OUTPUT_RANGED);
COLOURSPACEDLL_API void CS_HLS2xyz(float H, float L, float S, float *x, float *y, float *z, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_sRGB2Luv(float R, float G, float B, float *L, float *u, float *v, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_sRGB2sLab(float R, float G, float B, float *L, float *a, float *b, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_sRGB2Lab(float R, float G, float B, float *L, float *a, float *b, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_sRGB2LaLbL(float R, float G, float B, float *L, float *aL, float *bL, BYTE Flags = CS_INPUT_RANGED);
COLOURSPACEDLL_API void CS_LaLbL2sRGB(float L, float aL, float bL, float *R, float *G, float *B, BYTE Flags = CS_OUTPUT_RANGED);
COLOURSPACEDLL_API BOOL CS_CheckVisualSimilarity(float x1, float y1, float z1, float x2, float y2, float z2, int CheckType = CVS_ALL);
COLOURSPACEDLL_API void CS_XYZ2xyz(float X, float Y, float Z, float *x, float *y, float *z, BYTE Flags = CS_NONE);
COLOURSPACEDLL_API void CS_xyz2XYZ(float x, float y, float z, float Yorig, float *X, float *Y, float *Z, BYTE Flags = CS_NONE);
COLOURSPACEDLL_API BOOL CS_CheckHueSimilarity(int BaseHue, int CheckHue, float factor = WavelengthDiscriminationFactor);
COLOURSPACEDLL_API BOOL CS_IsChromatic(int H, int L, int S);
COLOURSPACEDLL_API BOOL CS_IsAchromatic(int H, int L, int S);
COLOURSPACEDLL_API BOOL CS_CheckLightnessSimilarity(int Lightness1, int Lightness2);
COLOURSPACEDLL_API BOOL CS_CheckSaturationSimilarity(int H1, int L1, int S1, int H2, int L2, int S2);

COLOURSPACEDLL_API double CS_FindAngle(float x, float y, float x0, float y0);
COLOURSPACEDLL_API double CS_FindAngle(float x, float y);
COLOURSPACEDLL_API double CS_AngleDifference(double a1, double a2);
COLOURSPACEDLL_API double CS_Distance(double x1, double y1, double x2, double y2);

void InitTransMatrices();
BOOL InitCIETables(HANDLE hModule);
BOOL InitWLDiscriminationTables(HANDLE hModule);
BOOL InitHueDiscriminationTables(float factor = -1.0f, HANDLE hModule = NULL); //factor -1 means hard coded data are used
BOOL InitChromaticityThresholdTables(HANDLE hModule);
BOOL InitLightnessThresholdTables(HANDLE hModule);
float HLS_Value(float n1, float n2, float hue);
