// Comp.h: interface for the CComp class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMP_H__24A224A1_1882_11D5_8D34_00108302BD82__INCLUDED_)
#define AFX_COMP_H__24A224A1_1882_11D5_8D34_00108302BD82__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Image.h"

class CImage;

#define RR_FIRST -10
#define RR_LAST  -11

#define COMPTYPE_UNKNOWN		0x0001
#define COMPTYPE_CHARACTER		0x0002
#define COMPTYPE_BACKGROUND		0x0004
#define COMPTYPE_SELECTED		0x0008
#define COMPTYPE_FILTEREDOUT	0x0010
//Memorial Definitions
#define COMPTYPE_CONTENT		0x0020
	//Background Entities
#define COMPTYPE_CORNER		(0x0040 | COMPTYPE_BACKGROUND)
#define COMPTYPE_PUNCHHOLE	(0x0080 | COMPTYPE_BACKGROUND)
#define COMPTYPE_STAIN		(0x0100 | COMPTYPE_BACKGROUND)
#define COMPTYPE_OUTEREDGE	(0x0200 | COMPTYPE_BACKGROUND)
#define COMPTYPE_GROWNPAPER	(0x0400 | COMPTYPE_BACKGROUND)
#define COMPTYPE_PAPER		(0x0800 | COMPTYPE_BACKGROUND)	//used also for everything that doesn't fall in the other categories
	//Content (Foreground) Entities
#define COMPTYPE_LINESEGMENT	(0x0040 | COMPTYPE_CONTENT)
#define COMPTYPE_TEXT			(0x0080 | COMPTYPE_CONTENT)
#define COMPTYPE_PHOTO			(0x0100 | COMPTYPE_CONTENT)
#define COMPTYPE_SIGNATURE		(0x0200 | COMPTYPE_CONTENT)
#define COMPTYPE_STAMP			(0x0400 | COMPTYPE_CONTENT)
#define COMPTYPE_ANNOTATION		(0x0800 | COMPTYPE_CONTENT)
#define COMPTYPE_GRAPHICS		COMPTYPE_CONTENT			//used also for everything that doesn't fall in the other categories

#define RGB_GETALPHA(rgb) (((rgb) >> 24) & 0xff)

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CComp : public CObject  
{
	DECLARE_SERIAL(CComp)
private:
friend class CImage;
friend class CHiColorImage;
friend class CBiLevelImage;
friend class CLoColorImage;
friend class CGreyScaleImage;
#ifdef REGION_DEFINED
friend class CRegion;
#endif

//structures
public:
	typedef struct tagCompRun
	{
		int y;
		int xl;
		int xr;
	} CompRun;

//operators
public:
	void	operator =(CComp &srcComp);
	CComp	operator +(CComp &srcComp);
	void	operator +=(CComp &srcComp);
	CComp	operator -(CComp &srcComp);
	void	operator -=(CComp &srcComp);
	CComp	operator &(CComp &srcComp);
	void	operator &=(CComp &srcComp);
	CComp	operator |(CComp &srcComp);
	void	operator |=(CComp &srcComp);
	void	Merge(CComp &srcComp);

//access attributes functions
public:
	CRect		GetBoundingRect();
	inline CompRun *	GetRun(int index);
	int			GetInternalConnections();
	int			GetExternalConnections();
	int			GetMaxInternalBorder();
	int			GetSize();
	inline int	GetMaxRun();
	int			GetMaxExternalBorder();
	float		GetCenterY();
	float		GetCenterX();
	COLORREF	GetColorRGB();
#ifdef IMAGE_COLOURSPACE_SUPPORT
	COLORREF	GetColorHLS();
#endif //Colour Space
	void		SetVexed(CComp * pVexed);
	CComp *		GetVexed();
	BOOL		IsSelected();
	void		Select(BOOL Select = TRUE);
	POINT *		GetBoundary(BOOL Isothetic = TRUE);
	int			GetMaxBoundary();

//operational functions
public:
	void	Thin();
	int		AddRun(int y, int xl, int xr, BOOL ReCalculateAttributes = TRUE);
	int		RemoveRun(int index, BOOL ReCalculateAttributes = TRUE);
	BOOL	PtInComponent(int x, int y);
	BOOL	PtInComponent(CPoint & point);
	void	SetImageReference(CImage * pImage);
	BOOL	RequestRgn(CRgn * pRgn, float Magnification = 1.0f);
	void	ResetAttributes();
	void	Empty();
	void	Clear();
	void	Serialize(CArchive &ar);
	void	SetType(WORD newType, BOOL Combine = FALSE);
	WORD	GetType();
	int		GetHeight();
	int		GetWidth();
	void	DiscardVexedArea();
	int		AddPoint(int x, int y);
	int		AddPoint(CPoint Point);
	int		RemovePoint(int x, int y);
	int		RemovePoint(CPoint Point);
	void	SortRuns();
	//construction / deconstruction
	CComp();
	CComp(CComp &srcComp);
	CComp(CompRun * pRunArray, int nRuns);
	virtual ~CComp();

//Computations between components
public:
	int		ComputeOverlapping(CComp &srcComp);
	int		ComputeConnections(CComp &srcComp);
	int		ComputeConnections(CComp *srcComp);
	double	ComputeEuclidianDistance(CComp &srcComp);

#ifdef _DEBUG
//Debug functions
public:
	void TraceRuns();
	void TraceInfo();
#endif

private:
	WORD m_Type;
	static int _comparePOINT(const void * a, const void * b);
	static int _compareRUN(const void * a, const void * b);
	void FreeMem();

	//attributes
	CompRun *	m_Run;
	int			m_nRun;
	CImage *	m_pImage;
	int			m_Size;
	float		m_Center[2];
	COLORREF	m_ColorRGB;
	CRect		m_BoundingRect;
	int			m_nExternalBorder;	//External Border
	POINT *		m_ExternalBorder;	//External Border
	int			m_nInternalBorder;	//Internal Border
	POINT *		m_InternalBorder;	//Internal Border
	POINT *		m_Boundary;			//Boundary in terms of hor and ver edges, Clockwise
	int			m_nBoundary;
	int			m_nExternalConnections;
	int			m_nInternalConnections;
	CComp *		m_pVexed;
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_COMP_H__24A224A1_1882_11D5_8D34_00108302BD82__INCLUDED_)
