// Distribution.h: interface for the CDistribution class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DISTRIBUTION_H__26F30081_E497_4A87_AF18_B28B9F053003__INCLUDED_)
#define AFX_DISTRIBUTION_H__26F30081_E497_4A87_AF18_B28B9F053003__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//Computed Propeties
#define DIST_CP_NONE			0x0000
#define	DIST_CP_AVERAGE			0x0001
#define DIST_CP_EXTREMA			0x0002
#define	DIST_CP_MAXIMUM			0x0004
#define DIST_CP_MINIMUM			0x0008
//Plateu treatment when finding extrema
#define DIST_PT_PEAKMODE	0x01 //produces one maximum at the centre of the peak plateu, and two minimums are the start and end of the minimum plateu
#define	DIST_PT_BOTHENDS	0x02 //produces two extrema at the ends of the plateus
#define DIST_PT_CENTER		0x03 //produces one extreme at the center of the plateu
//Get Local Maximum/Minimum: Search Directions and Behaviour if out of bounds
#define GLM_OUTOFBOUNDS_VALUES	0x01	//Default Behaviour
#define GLM_OUTOFBOUNDS_ERROR	0x02
#define GLM_SD_RIGHT	0x01
#define GLM_SD_LEFT		0x02

#ifdef _DEBUG
//Debug
#define TRACE_POINTS	0x01
#define TRACE_MINIMA	0x02
#define TRACE_MAXIMA	0x04
#define TRACE_EXTREMES	TRACE_MINIMA|TRACE_MAXIMA
#define TRACE_ALL	TRACE_POINTS|TRACE_EXTREMES
#endif

#include "LibraryDeclaration.h"

template <class PointType> class IMG_DECL_CLASS CDistribution
{
public:
	typedef struct _Peak
	{
		int LeftMin;
		int RightMin;
		int Max;
	} Peak;
public:
				CDistribution();
	virtual		~CDistribution();
	BOOL		Create(PointType* Array, int Count);
	void		Reset();
	void		ResetProperties();
	void		SetCount(int Count);
	int			GetCount();
	void		SetPlateuTreatment(BYTE PlateuTreatment = DIST_PT_PEAKMODE);

	PointType	GetAverage();
	void		MovingAverage(int WindowSize);
	void		ComputeExtrema();
	PointType *	GetPointArray();
	int *		GetMaximaPositions();
	int *		GetMinimaPositions();
	int			GetLocalMaximumPos(int Position, BYTE SearchDir = GLM_SD_RIGHT, BYTE OutOfBoundsBehaviour = GLM_OUTOFBOUNDS_VALUES);
	int			GetLocalMinimumPos(int Position, BYTE SearchDir = GLM_SD_RIGHT, BYTE OutOfBoundsBehaviour = GLM_OUTOFBOUNDS_VALUES);
	int			GetMaximaCount() {if (!(m_ComputedProperties & DIST_CP_EXTREMA)) ComputeExtrema(); return m_nMaxima;}
	int			GetMinimaCount() {if (!(m_ComputedProperties & DIST_CP_EXTREMA)) ComputeExtrema(); return m_nMinima;}
	PointType	GetMaximum();
	PointType	GetMinimum();
	int			GetMaximumPos();
	int			GetMinimumPos();
	BOOL		GetPeak(int Position, Peak * pPeak);
	PointType	GetAt(int Index);
	void		AAPeakSmoothing(float Fhw = 1.0f, float Fwr = 0.11f, float Fpcr = 0.11f);
	int			DistanceAnalysis(int * Score, int Threshold = 15);
	void		RemoveSubbase();
	BOOL		IsZero();
	BOOL		IsConstant();
	double		LinearLSFitting(double * alpha, double * beta);

#ifdef _DEBUG
	//Debug functions
	void	TracePointsToFile(LPCTSTR strFileName, BYTE Flags = TRACE_POINTS);
#endif

private:
	void FreeMem();

private:
	PointType *	m_Point;
	int			m_Count;
	BYTE		m_PlateuTreatment;

	//Computed Properties
	WORD		m_ComputedProperties;
	PointType	m_Average;
	int		*	m_Maxima;
	int			m_nMaxima;
	int		*	m_Minima;
	int			m_nMinima;
	int			m_PosMaximum;
	int			m_PosMinimum;
};

#endif // !defined(AFX_DISTRIBUTION_H__26F30081_E497_4A87_AF18_B28B9F053003__INCLUDED_)
