// GreyScaleImage.h: interface for the CGreyScaleImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GREYSCALEIMAGE_H__31A15BCA_751E_4522_889C_A1664528052D__INCLUDED_)
#define AFX_GREYSCALEIMAGE_H__31A15BCA_751E_4522_889C_A1664528052D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "LoColorImage.h"
#include <math.h>

#if defined (IMAGE_SELAS_SPECIFIC)
//find dot definitions
#define	FD_LIMIT_I_REL	0.2f
#define FD_LIMIT_I_ABS	80
#define FD_LIMIT_RADIUS 100
#endif

//Separator Flags (Detect Text Lines)
#define SF_ISSTART		0x01
#define	SF_ISEND		0x02

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CGreyScaleImage : virtual public CImage  
{
	DECLARE_SERIAL(CGreyScaleImage)

	friend class CHiColorImage;
	friend class CLoColorImage;
	friend class CBiLevelImage;

public:
	//Initialization
	CGreyScaleImage();
	virtual ~CGreyScaleImage();
	virtual BOOL		Create(int Width, int Height, COLORREF BackColour, LPCTSTR Name = NULL);
	virtual BOOL		CreateFromTIFF(TIFF * Tif, LPCTSTR Name = NULL);
	virtual BOOL		CreateFromImage(CImage *pImage, BYTE Flags = 0x00);
	virtual BOOL		CreateDuplicate(CImage *pImage, BOOL ReferenceArrays = FALSE);
	virtual BOOL		ExportTIFF(LPCTSTR FileName);
	//Drawing Operations
protected:
	virtual void		FillBitmapInfo();
public:
	virtual BYTE*		UpdateNormal();
	virtual BYTE*		UpdateDimmed();
	//Operations
	virtual void		Rotate(double Angle, COLORREF BackColour = RGB(255, 255, 255), BOOL CropResultToOriginalSize = FALSE);
	virtual void		Rotate(CRect Rect, double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE);
	virtual void		Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL);
	virtual void		Crop(int Left, int Top, int Width, int Height);
	virtual void		ApplyFilter(FilterStruct *Filter);
	virtual BOOL		GetHistograms(long ** Histograms, DWORD ColorComponents = HCC_GREYSCALE, RECT& Rectangle = CRect(0,0,0,0), BYTE Action = GH_DEFAULT_ACTION, int MapIndex = -1, BYTE PixelType = 0x00);
	virtual BOOL		HistogramTransformation(int Transformation[256], DWORD ColorComponents = HCC_GREYSCALE);
	virtual void		GetProjection(int Left, int Top, int Width, int Height, long * VertProfile = NULL, long * HorzProfile = NULL, BYTE Flags = PRJ_GREY);
	virtual void		Serialize(CArchive& ar);
	virtual COLORREF	GetAverageColour(int MapIndex = -1, BYTE PixelType = 0x00);
	virtual void		HistogramEqualization(RECT &Rectangle = CRect(0,0,0,0) );
	//Operations: Greyscale specific
	virtual void		Threshold_Niblack(int Size = 5, CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL, BYTE Flags = THR_OUTPUT_MAP|THR_OUTPUT_IMAGE);
	virtual void		Threshold_WeszkaRosenfeld(CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL);
	virtual void		Threshold(int T = 128, CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL);
	//Attributes
	virtual BYTE		GetRed(int x, int y);
	virtual BYTE		GetGreen(int x, int y);
	virtual BYTE		GetBlue(int x, int y);
	virtual void		SetRed(int x, int y, BYTE val);
	virtual void		SetGreen(int x, int y, BYTE val);
	virtual void		SetBlue(int x, int y, BYTE val);
	virtual COLORREF	GetRGBColor(int x, int y);
	virtual void		SetRGBColor(int x, int y, COLORREF col);
	virtual BOOL		DiscardBackup();
	virtual void		RestoreBackup();
	virtual BOOL		CreateBackup();

private:
	inline float CGreyScaleImage::Sigma(float f)
		{ 
			f = f*2 - 1; //range [-1,1]
			return (float)(0.5+(f>0?1:-1)*0.5*pow( (double)(f>0?1:-1)*f, exp((double)(-0.93888)) ));//(m_Sigma==0?1:(m_Sigma>0?m_Sigma:1.0/(-m_Sigma))) ));
		}

protected:
	//Initialization
	void		FreeMem();
	//Operations
	//Attributes
	inline void PrivateSetByte(int x, int y, BYTE val)
		{m_LineArray[y][x] = val;}

	inline BYTE PrivateGetByte(int x, int y)
		{return m_LineArray[y][x];}

	//Variables
	BYTE *		m_Array;
	BYTE **		m_LineArray;
	BYTE *		m_BackupArray;
	BYTE **		m_BackupLineArray;
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_GREYSCALEIMAGE_H__31A15BCA_751E_4522_889C_A1664528052D__INCLUDED_)
