// HiColorImage.h: interface for the CHiColorImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HICOLORIMAGE_H__DFA1CBCC_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
#define AFX_HICOLORIMAGE_H__DFA1CBCC_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//Default tolerance for colours to be similar in Lab
#define DEFAULT_LAB_TOLERANCE 1.0

//Positions of components in array
#define POS_RED    2
#define POS_GREEN  1
#define POS_BLUE   0
#define POS_ALPHA  3

#include "Image.h"

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CHiColorImage : virtual public CImage  
{
	DECLARE_SERIAL(CHiColorImage)

	friend class CLoColorImage;
	friend class CGreyScaleImage;
	friend class CBiLevelImage;
public:

	//Drawing Functions
protected:
	virtual void		FillBitmapInfo();
public:
	virtual BYTE*		UpdateNormal();
	virtual BYTE*		UpdateDimmed();
	virtual void		Draw(HDC hDC, int x, int y, DWORD rop = SRCCOPY);
	virtual void		DrawLine(int x1, int y1, int x2, int y2);
	virtual void		DrawLine(float Rho, float Theta, int CenterX = 0, int CenterY = 0);
	//Initialization
	CHiColorImage();
	virtual ~CHiColorImage();
	virtual BOOL		Create(int Width, int Height, COLORREF BackColour, LPCTSTR Name);
	virtual BOOL		ExportTIFF(LPCTSTR FileName);
	virtual BOOL		CreateFromTIFF(TIFF * Tif, LPCTSTR Name = NULL);
	virtual BOOL		CreateFromImage(CImage *pImage, BYTE Flags = 0x00);
	virtual BOOL		CreateDuplicate(CImage *pImage, BOOL ReferenceArrays);
	//Operations
	virtual void		Rotate(double Angle, COLORREF BackColour = RGB(255,255,255), BOOL CropResultToOriginalSize = FALSE);
	virtual void		Rotate(CRect Rect, double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE);
	virtual void		Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL);
	virtual void		Crop(int Left, int Top, int Width, int Height);
	virtual void		Clear(int R, int G, int B);
	virtual void		ClearSimilar(BYTE Red, BYTE Green, BYTE Blue, int Threshold);
	virtual void		EdgePreservingSmoothing();
	virtual COLORREF	GetAverageColour(int MapIndex = -1, BYTE PixelType = 0x00);
#ifdef IMAGE_COMPONENT_SUPPORT
#ifdef IMAGE_COLOURSPACE_SUPPORT
	virtual void		ColorCC(int Connectivity = FCC_C8);
	virtual void		ExColorCC(int Connectivity = FCC_C8, double LabTolerance = 20.0);
#endif //colour space
	virtual BOOL		CCAnalysis(int Connectivity = FCC_C8);
#endif //component support

	virtual void		Serialize(CArchive& ar);
	virtual void		SetReferenceImage(CImage * pImage);
	virtual void		InvertMap();
#ifdef IMAGE_COLOURSPACE_SUPPORT
	virtual void		EdgeDetector(int Directions);
	virtual void		ApplyAdvancedTreshold( int * Range);
#endif //Colour space
	virtual void		GetNegative();
	virtual BOOL		ReSample(int NewWidth, int NewHeight, int NewDPI);
	virtual BOOL		DiscardBackup();
	virtual void		RestoreBackup();
	virtual BOOL		CreateBackup();
	virtual BOOL		GetHistograms(long ** Histograms, DWORD ColorComponents = HCC_RGB|HCC_HLS, RECT& Rectangle = CRect(0,0,0,0), BYTE Action = GH_DEFAULT_ACTION, int MapIndex = -1, BYTE PixelType = 0x00);
	virtual BOOL		HistogramTransformation(int Transformation[256], DWORD ColorComponents = HCC_RGB);
	virtual void		ColorReduction();
	virtual void		ApplyRankOrderFilter(BYTE Type, int Size = 3, BYTE Channel = CH_RGB);
	virtual void		ApplyFilter(FilterStruct *Filter);
	//Atributes
	virtual BYTE		GetRed(int x, int y);
	virtual BYTE		GetGreen(int x, int y);
	virtual BYTE		GetBlue(int x, int y);
	virtual void		SetRed(int x, int y, BYTE val);
	virtual void		SetGreen(int x, int y, BYTE val);
	virtual void		SetBlue(int x, int y, BYTE val);
	virtual COLORREF	GetRGBColor(int x, int y);
	virtual void		SetRGBColor(int x, int y, COLORREF col);
protected:
	inline BYTE PrivateGetR(int x, int y)
		{return m_LineArray[y][x*4+POS_RED];}
	inline BYTE PrivateGetG(int x, int y)
		{return m_LineArray[y][x*4+POS_GREEN];}
	inline BYTE PrivateGetB(int x, int y)
		{return m_LineArray[y][x*4+POS_BLUE];}

	inline void PrivateSetR(int x, int y, BYTE val)
		{m_LineArray[y][x*4+POS_RED] = val;}
	inline void PrivateSetG(int x, int y, BYTE val)
		{m_LineArray[y][x*4+POS_GREEN] = val;}
	inline void PrivateSetB(int x, int y, BYTE val)
		{m_LineArray[y][x*4+POS_BLUE] = val;}

	inline BYTE PrivateGetBackupR(int x, int y)
		{return m_BackupLineArray[y][x*4+POS_RED];}
	inline BYTE PrivateGetBackupG(int x, int y)
		{return m_BackupLineArray[y][x*4+POS_GREEN];}
	inline BYTE PrivateGetBackupB(int x, int y)
		{return m_BackupLineArray[y][x*4+POS_BLUE];}


protected:
	//Initialization
	void				FreeMem();
	//Operations
#ifdef IMAGE_COLOURSPACE_SUPPORT
	virtual BOOL		CheckVisualSimilarity(int x, int y, int r, int g, int b, int SimilarityTest = ST_ALL);
	virtual BOOL		CheckVisualSimilarity(float r1, float g1, float b1, float r, float g, float b, int SimilarityTest = ST_ALL);
	virtual double		CheckLabSimilarity(int r1, int g1, int b1, int r2, int g2, int b2);
	virtual double		CheckLabSimilarity(int x1, int y1, int x2, int y2);
	virtual double		CheckLabSimilarity(COLORREF col1, COLORREF col2);
	virtual double		CheckLabSimilarity(int x, int y, int r, int g, int b);
	virtual BOOL		IsLabSimilar(int r1, int g1, int b1, int r2, int g2, int b2, double tolerance = DEFAULT_LAB_TOLERANCE);
	virtual BOOL		IsLabSimilar(int x1, int y1, int x2, int y2, double tolerance = DEFAULT_LAB_TOLERANCE);
	virtual BOOL		IsLabSimilar(COLORREF col1, COLORREF col2, double tolerance = DEFAULT_LAB_TOLERANCE);
#endif
	virtual void		OverlayColor(int x, int y,int Red, int Green, int Blue, BYTE *OutputArray, int WidthBytes);
	//Attributes

	BYTE *	m_Array;
	BYTE ** m_LineArray; //Helper for faster access of lines
	BYTE *	m_BackupArray;
	BYTE ** m_BackupLineArray;
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_HICOLORIMAGE_H__DFA1CBCC_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
