// Image.h: interface for the CImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IMAGE_H__DFA1CBC9_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
#define AFX_IMAGE_H__DFA1CBC9_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_

//#define IMAGE_PROGRESS_SUPPORT
//#define IMG_ENABLE_PROGRESS_DIALOG
//#define IMAGE_FINEREADER_SUPPORT

#define IMAGE_COMPONENT_SUPPORT
#define IMAGE_COLOURSPACE_SUPPORT

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ImgGlobalUtilFunctions.h" //In order to use GetCycleCount
#include "tiffio.h" //Save - Load TIFF Files

#ifdef IMAGE_PROGRESS_SUPPORT
#ifdef IMG_ENABLE_PROGRESS_DIALOG
	#include "ProgressLib.h"
#else
	#define PROGDLG_PLAIN_BAR		0x00
	#define PROGDLG_STEP_BAR		0x01
	#define PROGDLG_CANCEL_BUTTON	0x02
	#define PROGDLG_TASK_LIST		0x04
#endif
#endif

#ifdef IMAGE_FINEREADER_SUPPORT
	//finereader support
	#import "FREngine.tlb"
#endif

#ifdef IMAGE_COLOURSPACE_SUPPORT
	#include "ColourSpaceDLL.h"
#endif

//Factor used for relaxing HUE threholds
#define DEFAULT_HUE_FACTOR -1.0f

#ifdef IMAGE_COMPONENT_SUPPORT
	#include "Comp.h"	// Added by ClassView
	class CComp;
#endif

#if defined (IMAGE_COMPONENT_SUPPORT)
//Return Structure for comparison results
typedef struct tagGroundTruthComparisonResults
{
	struct tagCharacterResult
	{
		int * CompArray;
		int nComp;
		int nPixels;
		int PixelsMatched;
		int PixelsOutSide;
	} * CharacterArray;
	int MaxCharacter;
	int nMissedCharacters;
	int nOverlappings;
} GTComparison;
#endif

#ifdef IMAGE_PROGRESS_SUPPORT
	//Progress Bar Options (Control of Progress Bar/Window Usage)
	#define PBO_PROGRESS_SHOWWINDOW	0x01
	#define PBO_PROGRESS_SHOWBAR	0x02
	#define PBO_PROGRESS_SHOWNONE	0x04
#endif

//Artifacts
#define AR_NONE					0x00
#define AR_LOOSY_COMPRESSION	0x01
#define AR_SCAN_BLURRING		0x02
#define AR_NOISE				0x04

//Title options
#define IM_TITLE_NAME		0x01
#define IM_TITLE_TYPE		0x02

//Image Type Attributes - Application Scopes (the first WORD)
#define IT_GENERIC							0x00000000
#define IT_AURORA_SPECIFIC					0x01000000
#define IT_MEMORIAL_SPECIFIC				0x02000000
#define IT_USERAPP1							0x04000000
#define IT_USERAPP2							0x08000000
#define IT_USERAPP3							0x10000000
#define IT_USERAPP4							0x20000000
#define IT_USERAPP5							0x40000000
#define IT_USERAPP6							0x80000000
//Image Type attributes - Image Types (the rest 3 WORDs)
#define IT_UNKNOWN							0x00000001
#define IT_BILEVEL							0x00000002
#define IT_LOCOLOR							0x00000004
#define IT_HICOLOR							0x00000008
#define IT_GREYSCALE						0x00000010 //| IT_LOCOLOR //It is a LoColor, specialized: 1 more bit is turned on
//Aurora Specific Types Definitions
#define IT_PARENT							IT_AURORA_SPECIFIC | 0x00000020
#define IT_HUE								IT_AURORA_SPECIFIC | 0x00000040
#define IT_LUM								IT_AURORA_SPECIFIC | 0x00000080
#define IT_CHROMATIC						IT_AURORA_SPECIFIC | 0x00000100
#define IT_ACHROMATIC						IT_AURORA_SPECIFIC | 0x00000200
#define IT_GROUND_TRUTH						IT_AURORA_SPECIFIC | 0x00000400
#define IT_CHAR_COMPS						IT_AURORA_SPECIFIC | 0x00000800
#define IT_CHAR_COMPS_CF_PIXEL_SIZE			IT_AURORA_SPECIFIC | 0x00001000
#define IT_CHAR_COMPS_CF_BOUNDING_BOX_SIZE	IT_AURORA_SPECIFIC | 0x00002000
#define IT_CHAR_COMPS_CF_WIDTH				IT_AURORA_SPECIFIC | 0x00004000
#define IT_CHAR_COMPS_CF_HEIGHT				IT_AURORA_SPECIFIC | 0x00008000
#define IT_CHAR_COMPS_CF_COMPACTNESS		IT_AURORA_SPECIFIC | 0x00010000
#define IT_CHAR_COMPS_CF_DIAGONAL			IT_AURORA_SPECIFIC | 0x00020000
#define IT_CHAR_COMPS_CF_ASPECT_RATIO		IT_AURORA_SPECIFIC | 0x00040000
#define IT_CHAR_COMPS_TEXT_LINE				IT_AURORA_SPECIFIC | 0x00080000
#define IT_CHAR_COMPS_UNSURE				IT_AURORA_SPECIFIC | 0x00100000
#define IT_CHAR_COMPS_NON_TEXT_LINE			IT_AURORA_SPECIFIC | 0x00200000

//Histogram Colour Components
#define HCC_GREYSCALE						0x00000001
#define HCC_RGB_RED							0x00000002
#define HCC_RGB_GREEN						0x00000004
#define HCC_RGB_BLUE						0x00000008
#define HCC_HLS_HUE							0x00000010
#define HCC_HLS_LIGHTNESS					0x00000020
#define HCC_HLS_SATURATION					0x00000040
#define HCC_LAB_LIGHTNESS					0x00000080
#define HCC_LAB_A							0x00000100
#define HCC_LAB_B							0x00000200
#define HCC_XYZ_X							0x00000400
#define HCC_XYZ_Y							0x00000800
#define HCC_XYZ_Z							0x00001000
#define HCC_LUV_LIGHTNESS					0x00002000
#define HCC_LUV_U							0x00004000
#define HCC_LUV_V							0x00008000
#define HCC_RGB								HCC_RGB_RED|HCC_RGB_GREEN|HCC_RGB_BLUE
#define HCC_HLS								HCC_HLS_HUE|HCC_HLS_LIGHTNESS|HCC_HLS_SATURATION
#define HCC_LAB								HCC_LAB_LIGHTNESS|HCC_LAB_A|HCC_LAB_B
#define HCC_XYZ								HCC_XYZ_X|HCC_XYZ_Y|HCC_XYZ_Z
#define HCC_LUV								HCC_LUV_LIGHTNESS|HCC_LUV_U|HCC_LUV_V

//filter types
#define FT_EROSION	0x01	//minimum for rank order
#define FT_DILATION	0x02	//maximum for rank order
#define FT_CONTOUR	0x04	//maximum-minimum for rank order
#define FT_MEDIAN	0x08	//median of sorted values
#define CH_RED		0x01	//channel red
#define CH_GREEN	0x02	//channel green
#define CH_BLUE		0x04	//channel blue
#define CH_RGB		0x08	//all channels

//Resolution Units
#define RU_INCH 0
#define RU_MM	1

//FindConnectedComponents AnalysisType and Conectivity
#define FCC_AT_BACKGROUND	0x01
#define FCC_AT_VEXED		0x02
#define FV_AT_LIGHTNESS		0x04
#define FV_AT_HUE			0x08
#define FV_AT_SATURATION	0x10	//Saturation only and all, is defined as the same, since saturation only, has no meaning
#define FV_AT_ALL			0x10	//Saturation only and all, is defined as the same, since saturation only, has no meaning

#define FCC_C4				0
#define FCC_C8				1

//SimilarityTests
#define	ST_WAVELENGTH	0
#define ST_LUMINANCE	1
#define ST_SATURATION	2
#define ST_ALL			100
#define ST_NONE			200

//Overlay component flags
#define OC_COMP				0x01
#define OC_VEXED			0x02
#define OC_HIGHLIGHT_COMP	0x04
#define OC_HIGHLIGHT_VEXED	0x08
//Thumbnail's Size
#define THUMB_WIDTH		100
#define THUMB_HEIGHT	100
//UpdateFlags
#define UD_NORMAL				0x01
#define UD_DIMMED				0x02
#define UD_THUMBNAIL			0x04
#define	UD_SHOW_COMPONENTS		0x08
#define UD_HIGHLIGHT_SELECTED	0x10
#define UD_BW_TEXT				0x20
#define UD_COLOUR_MAP			0x40
//Select Component Flags
#define SC_ADD			0x01
#define	SC_REMOVE		0x02
#define	SC_NEW			0x04
#define SC_NONE			0x08
#define SC_ALL			0x10
#define	SC_CHARACTERS	0x20
#define SC_BACKGROUND	0x40
#define SC_UNKNOWN		0x80
//Undefined Skew
#define UNDEFINED_SKEW	-999.99f
//Get Histogram Behaviours
#define GH_DEFAULT_ACTION	0	//exclude background pixels from standard image map
#define GH_INCLUDEALL		1
#define GH_INCLUDE			2
#define GH_EXCLUDE			3
//Skew Detection Methods
#define	DS_FINEREADER		0

//Projection Options
#define PRJ_GREY		0x01	//Get greyscale projection (use with greyscale images)
#define PRJ_BILEVEL		0x02	//Get Bilevel projection (use with greyscale images)
#define PRJ_PIXEL_COUNT	0x04	//Count Number of Pixels (default behaviour)
#define	PRJ_RUN_COUNT	0x08	//Count Number of Runs (only for Map Projections)
#define	PRJ_DISCARD_MAP	0x10	//Get All (background and foreground) pixels

//Create From Image Options
#define CFI_USE_COMPONENT_RED	0x01	//Only when creating from HiColor
#define CFI_USE_COMPONENT_GREEN	0x02	//Only when creating from HiColor
#define CFI_USE_COMPONENT_BLUE	0x04	//Only when creating from HiColor

//Tilting Flags
#define TILT_HORIZONTAL	0x01
#define	TILT_VERTICAL	0x02

//Rotation: Transparent Color
#define ROT_PRESERVE_BACKGROUND		0xFFFFFFFF

//Thresholding Flags
#define THR_OUTPUT_MAP		0x01	//Sets the background bit in the image map
#define THR_OUTPUT_IMAGE	0x02	//Changes the colours (black and white) in the image itself

#ifndef PI
#define PI 3.1415926535897932384626433832795f
#endif

#include "ImageMap.h"
class CImageMap;
class CBiLevelImage;

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CImage : public CObject
{
	DECLARE_SERIAL(CImage)

private:
friend class CImageMap;

	//------------------------------
	//          STRUCTURES
	//------------------------------
public:
	//Linear filter structure
	typedef struct tagFilterStruct
	{
		int Filter [7][7];
		int Div;
		int Size;
		TCHAR Name[25];
	} FilterStruct;

	//Image Update Information
	typedef struct tagImageUpdate
	{
		WORD Flags;
		float Magnification;
		COLORREF DimColor;
	} ImageUpdate;

	//------------------------------
	//       MEMBER VARIABLES
	//------------------------------
protected:
	//Image Attributes
	BOOL			m_ArraysByReference;
	int				m_BitsPerPixel;
	long			m_BytesPerLine;
	int				m_ResX;
	int				m_ResY;
	int				m_ResUnit;
	WORD			m_Artifacts;
	TCHAR *			m_Name;
    BOOL			m_BackupExists;
	int				m_Width;
	int				m_Height;
	long			m_NumberOfPixels;
	DWORD			m_Type;
	float			m_fSkew;
	BOOL			m_IsDirty;

	//Connected Components
#if defined (IMAGE_COMPONENT_SUPPORT)
	CRgn			m_SelectedCompRgn;
	int				m_SelectedCompMax;
	int *			m_SelectedCompArray;
#endif

	//Image Bitmap - Update
	CBitmap			m_ThumbnailBitmap;
	CBitmap			m_ImageBitmap;
	ImageUpdate		m_ImageUpdate;
	BITMAPINFO *	m_pBitmapInfo;

	//Image Maps
	int				m_nMap;
	int				m_iStandardMap;
	CImageMap **	m_Map;

	//Drawing
	COLORREF		m_DrawForeColor;
	COLORREF		m_DrawBackColor;
	CRect			m_DrawLimitRect;

	//Connected Components
#ifdef IMAGE_COMPONENT_SUPPORT
	CComp **		m_Component;
	int				m_nComponent;
#endif //IMAGE_COMPONENT_SUPPORT

	//Default Progress Bar (use this for all classes)
	BOOL					m_CancelRequested;
#ifdef IMAGE_PROGRESS_SUPPORT
	static	CProgressCtrl * m_pProgressBar;
	static	CEdit *			m_pProgressBarEdit;
	static	WORD			m_ProgressBarOptions;
	BOOL					m_ProgressRunning;
#ifdef IMG_ENABLE_PROGRESS_DIALOG
	CDlgProgress *			m_pProgressWnd;
#endif //IMG_ENABLE_PROGRESS_DIALOG
#endif //IMAGE_PROGRESS_SUPPORT

	//------------------------------
	//       CLASS FUNCTIONS
	//------------------------------

	//Initialization, Creation, Input-Output
public:
	CImage();
	virtual ~CImage();
	virtual BOOL		Create(int Width, int Height, COLORREF BackColour, LPCTSTR Name);
	virtual BOOL		CreateDuplicate(CImage *pImage, BOOL ReferenceArrays = FALSE);
	virtual BOOL		CreateFromImage(CImage *pImage, BYTE Flags = 0x00);
	virtual void		FreeMemory();
	virtual BOOL		ExportTIFF(LPCTSTR FileName);
	virtual BOOL		CreateFromTIFF(TIFF * Tif, LPCTSTR Name = NULL);
	virtual void		Serialize(CArchive& ar);
protected:
	virtual void		FreeMem();
	void				FreeComponents();		

	//Image Maps
public:
	int					GetImageMapCount();
	CImageMap *			AddImageMap(BYTE PreDefinedMap);
	CImageMap *			AddImageMap(LPCTSTR Name, BYTE MapType);
	CImageMap *			SafeGetImageMap(BYTE PreDefinedMap);
	CImageMap *			SafeGetImageMap(LPCTSTR Name, BYTE MapType);
	CImageMap *			GetImageMap(int index);
	CImageMap *			GetImageMap(BYTE PreDefinedMap);
	CImageMap *			GetImageMap(LPCTSTR Name);
	int					GetImageMapIndex(BYTE PreDefinedMap);
	int					GetImageMapIndex(LPCTSTR Name);
	BOOL				SwapMapArrays(CImage * pImage);
	BOOL				CopyMapArrays(CImage *pImage);
	virtual void		InvertMap();
protected:
	inline void			MapSetByte(CImageMap * pMap, int x, int y, BYTE val);
	inline BYTE			MapGetByte(CImageMap * pMap, int x, int y);
	inline void			MapByteAddAttribute(CImageMap * pMap, int x, int y, BYTE val);
	inline void			MapByteRemoveAttribute(CImageMap * pMap, int x, int y, BYTE val);
	inline BYTE*		MapGetLine(CImageMap * pMap, int y);

	//Drawing Functions
public:
	void				DrawSetLimits(CRect &Rect);
	void				DrawSetForeColor(COLORREF ForeColor);
	void				DrawSetBackColor(COLORREF BackColor);

	//Operations
public:
	inline void			SetDirty() {m_IsDirty = TRUE;}
	virtual void		HistogramEqualization(RECT &Rectangle = CRect(0,0,0,0) );
	virtual void		Threshold(int T = 128, CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL);
	virtual void		Threshold_Niblack(int Size = 5, CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL, BYTE Flags = THR_OUTPUT_MAP|THR_OUTPUT_IMAGE);
	virtual void		Threshold_WeszkaRosenfeld(CRect& rect = CRect(-1,-1,-1,-1), CImage * pOutputImage = NULL);
	virtual void		Rotate(double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE);
	virtual void		Rotate(CRect Rect, double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE );
	virtual void		Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL);
	virtual void		Crop(int Left, int Top, int Width, int Height);
	virtual void		EdgePreservingSmoothing();
	virtual COLORREF	GetAverageColour(int MapIndex = -1, BYTE PixelType = 0x00);
	virtual void		ClearSimilar(BYTE Red, BYTE Green, BYTE Blue, int Threshold);
	virtual void		Clear(int R = 255, int G = 255, int B = 255);
	virtual BOOL		BBAnalysis(RECT ** pRect, int * nRect, CRect SearchRect = CRect(-1,-1,-1,-1), int Connectivity = FCC_C8);
	virtual void		GetNegative();
	virtual BOOL		ReSample(int NewWidth, int NewHeight, int NewDPI);
	virtual void		RecalculateNumberOfPixels();
	virtual BOOL		GetHistograms(long ** Histograms, DWORD ColorComponents = HCC_RGB|HCC_HLS, RECT& Rectangle = CRect(0,0,0,0), BYTE Action = GH_DEFAULT_ACTION, int MapIndex = -1, BYTE PixelType = 0x00);
	virtual BOOL		HistogramTransformation(int Transformation[256], DWORD ColorComponents = HCC_RGB);
	virtual void		GetProjection(int Left, int Top, int Width, int Height, long * VertProfile = NULL, long * HorzProfile = NULL, BYTE Flags = 0x00);
	virtual void		ColorReduction();
	virtual void		ApplyRankOrderFilter(BYTE Type, int Size = 3, BYTE Channel = CH_RGB);
	virtual void		ApplyFilter(FilterStruct *Filter);
	void				ApplyMorphologicalFilterToMap(int MapIndex, BYTE Type, int Size = 3, BYTE PixelType = 0x00, BYTE PreservePixelType = 0x00);
#ifdef IMAGE_COLOURSPACE_SUPPORT
	static double		ComputeColourDistance(int r1, int g1, int b1, int r2, int g2, int b2);
	virtual void		EdgeDetector(int Directions);
	virtual void		ApplyAdvancedTreshold( int * Range);
#endif //colour space

	//Component Operations
#ifdef IMAGE_COMPONENT_SUPPORT
	virtual void		CCAnalysisFromMap(int MapIndex, BYTE PixelType, WORD CompType, BOOL Invert = FALSE, int Connectivity = FCC_C8);
	virtual BOOL		CCAnalysis(int Connectivity = FCC_C8);
	CComp *				GetComponent(int index);
	int					GetMaxComponent();
	void				SelectComponent(int Index, WORD Flags);
	BOOL				IsComponentSelected(int Index);
	int					RemoveComponent(int *IndexArray, int Max);
	int					RemoveComponent(int Index);
	int					CombineComponents(int * IndexArray, int Max);
	int					CombineComponents(int Index1, int Index2);
	BOOL				GetNeighbours(CComp * pBaseComponent, int * pReturnUBound, CComp ** pReturnArray);
	BOOL				GetNeighbours(CComp *pBaseComponent, int * pReturnUBound, int ** ReturnArray);
	CComp *				CreateNewComponent();
	virtual void		ShowPhysicalStructure(CBitmap * pImage = NULL);
	void				OverlayComponent(int Index = -1, CBitmap * pBitmap = NULL, BYTE Flags = OC_COMP|OC_VEXED);
	void				OverlayComponent(int * Indices = NULL, int nIndex = -1, CBitmap * pBitmap = NULL, BYTE Flags = OC_COMP|OC_VEXED);
	int					ComponentFromPoint(CPoint HitPoint);
	virtual void		DrawComponents(COLORREF Colour = RGB(0,0,0));
public:
#ifdef IMAGE_COLOURSPACE_SUPPORT
	virtual void		ColorCC(int Connectivity = FCC_C8);
	virtual void		ExColorCC(int Connectivity = FCC_C8, double LabTolerance = 20.0);
#endif //IMAGE_COLOURSPACE_SUPPORT
#endif //IMAGE_COMPONENT_SUPPORT

	//Backup - Restore
public:
	virtual BOOL		DiscardBackup();
	virtual void		RestoreBackup();
	virtual BOOL		CreateBackup();

	//Image Bitmap - Update
protected:
	virtual void		FillBitmapInfo() {}
private:
	virtual void		RequestUpdate();
public:
	virtual void		UpdateColourMap(int MapIndex);
			void		UpdateColourMap(LPCTSTR MapName);
	virtual void		UpdateThumbnail();
	void				GetBitmap(CBitmap * pBitmap);
	inline CBitmap *	GetBitmap();
	virtual BYTE*		UpdateNormal() {return NULL;}
	virtual BYTE*		UpdateDimmed() {return NULL;}

	//Attributes - Get/Set
public:
	BOOL				IsForeground(int x, int y);
	BOOL				IsBackground(int x, int y);
	inline BOOL			IsType(DWORD Check) {return (GetType() & Check) == Check;}
	static inline BOOL	IsType(DWORD Base, DWORD Check) {return (Base & Check) == Check;}

	virtual CString		GetTitle(WORD Flags = IM_TITLE_NAME|IM_TITLE_TYPE);
	inline LPCTSTR		GetName() {return m_Name;}
	void				SetName(LPCTSTR Name = NULL);
	virtual void		SetReferenceImage(CImage * pImage);
	inline BOOL			GetArraysByReference() {return m_ArraysByReference;}
	virtual BYTE		GetRed(int x, int y);
	virtual BYTE		GetGreen(int x, int y);
	virtual BYTE		GetBlue(int x, int y);
	virtual void		SetRed(int x, int y, BYTE val);
	virtual void		SetGreen(int x, int y, BYTE val);
	virtual void		SetBlue(int x, int y, BYTE val);
	virtual COLORREF	GetRGBColor(int x, int y);
	virtual void		SetRGBColor(int x, int y, COLORREF col);
	inline DWORD		AddType(DWORD Type) {m_Type |= Type; return m_Type;}
	inline DWORD		RemoveType(DWORD Type) {m_Type &= (~Type); return m_Type;}
	void				SetDPI(int NewResX, int NewResY, int NewUnit=RU_INCH);
	void				SetDPI(int NewDPI);
	inline int			GetDPI() {return m_ResX;} //to be changed
	inline DWORD		GetType() {return m_Type;}
	inline CBitmap *	GetThumbnail() {return &m_ThumbnailBitmap;}
	inline int			GetWidth() {return m_Width;}
	inline int			GetHeight() {return m_Height;}
	inline WORD			GetUpdateFlags() {return m_ImageUpdate.Flags;}
	void				SetUpdateFlags(WORD UpdateFlags);
	inline float		GetMagnification() {return m_ImageUpdate.Magnification;}
	void				SetMagnification(float Magnification);
	inline long			GetNumberOfPixels() {return m_NumberOfPixels;}
	void				GetImageUpdate(ImageUpdate * pImageUpdate);
	void				SetImageUpdate(ImageUpdate * pImageUpdate);
	inline BOOL			GetShowComponents() {return (m_ImageUpdate.Flags & UD_SHOW_COMPONENTS);}
	void				SetShowComponents(BOOL ShowComponents);
	inline BOOL			GetShowBW() {return (m_ImageUpdate.Flags & UD_BW_TEXT);}
	void				SetShowBW(BOOL ShowBW);
protected:
	inline int	CImage::GetColorFromBitArray(BYTE *array, int x, int y, int WidthBytes)
		{return ((*(array+y*WidthBytes+(int)(x/8))) & (1<<(7-(x-8*(int)(x/8)))) )>>(7-(x-8*(int)(x/8)));	}
	inline void	CImage::SetColorToBitArray(BYTE *array, int x, int y, int WidthBytes, int color)
		{color==0? *(array+y*WidthBytes+(int)(x/8))&=~(1<<(7-(x-8*(int)(x/8)))) : *(array+y*WidthBytes+(int)(x/8))|=(1<<(7-(x-8*(int)(x/8))));	}

	inline int	CImage::GetColorFromByteArray(BYTE *array, int x, int y, int WidthBytes)
		{return  *(array+y*WidthBytes+x);}
	inline void CImage::SetColorToByteArray(BYTE *array, int x, int y, int WidthBytes, int color)
		{color==0? *(array+y*WidthBytes+x) = 0x00 : *(array+y*WidthBytes+x) = 0x01;	}

	inline void CImage::SetRedColorToArray(BYTE *array, int x, int y, int WidthBytes, int color, int Bpp = 32)
		{	*(array+y*WidthBytes+x*Bpp/8+2)=color;	}	
	inline void CImage::SetGreenColorToArray(BYTE *array, int x, int y, int WidthBytes, int color, int Bpp = 32)
		{	*(array+y*WidthBytes+x*Bpp/8+1)=color;	}
	inline void CImage::SetBlueColorToArray(BYTE *array, int x, int y, int WidthBytes, int color, int Bpp = 32)
		{	*(array+y*WidthBytes+x*Bpp/8)=color;	}
	inline int	CImage::GetRedColorFromArray(BYTE *array, int x, int y, int WidthBytes, int Bpp = 32)
		{	return *(array+y*WidthBytes+x*Bpp/8+2); }
	inline int	CImage::GetGreenColorFromArray(BYTE *array, int x, int y, int WidthBytes, int Bpp = 32)
		{	return *(array+y*WidthBytes+x*Bpp/8+1); }
	inline int	CImage::GetBlueColorFromArray(BYTE *array, int x, int y, int WidthBytes, int Bpp = 32)
		{	return *(array+y*WidthBytes+x*Bpp/8);	}

	//Utility functions
protected:
	virtual inline int	GetBytesPerLine(int Width) {return ((Width * m_BitsPerPixel + 31) & ~31) /8;} //{return (Width * m_BitsPerPixel + 15) / 16 * 2;}
	virtual void		SetupLineArray(BYTE ** pLineArray, BYTE * pArray, int Height, int BytesPerLine);

	//Progress Bar Specific
public:
#ifdef IMAGE_PROGRESS_SUPPORT
	void				SetProgressBarOptions(WORD flags);
	WORD				GetProgressBarOptions();
	static void			SetProgressBarCtrl(CProgressCtrl * pControl);
	static CProgressCtrl * GetProgressBarCtrl();
#endif //IMAGE_PROGRESS_SUPPORT

protected:
	BOOL	Progress_KeepAlive(); //Keep this outside the progress support context, since we may use it independantly
#ifdef IMAGE_PROGRESS_SUPPORT
#ifdef IMG_ENABLE_PROGRESS_DIALOG
	int		Progress_UnCheckTask(int index = -1);
	int		Progress_CheckTask(int index = -1);
	void	Progress_SetTasks(LPCTSTR * Task = NULL, int nTask = 0);
	void	Progress_SetTasks(CString * Task, int nTask);
	void	Progress_MainBarSetTitle(LPCTSTR string);
	void	Progress_MainBarSetDescription(LPCTSTR string);
	void	Progress_StepBarSetTitle(LPCTSTR string);
	void	Progress_StepBarSetDescription(LPCTSTR string);
public:
	CDlgProgress *	Progress_SetProgressWnd(CDlgProgress * pProgressWnd);
#endif //IMG_ENABLE_PROGRESS_DIALOG
protected:
	void	Progress_SetTitle(LPCTSTR string);
	void	Progress_MainBarSetRange(int low = -1, int high = -1);
	void	Progress_MainBarSetPos(int pos = -1);
	int		Progress_MainBarIncrease(int value = 1);
	void	Progress_StepBarSetRange(int low = -1, int high = -1);
	void	Progress_StepBarSetPos(int pos = -1);
	int		Progress_StepBarIncrease(int value = 1);
	void	Progress_Dismiss();
	void	Progress_Start(WORD wStyle = PROGDLG_STEP_BAR|PROGDLG_CANCEL_BUTTON|PROGDLG_TASK_LIST);
	BOOL	Progress_CancelRequested();
	void	Progress_SetStyle(WORD wStyle = PROGDLG_STEP_BAR|PROGDLG_CANCEL_BUTTON|PROGDLG_TASK_LIST);
#endif //IMAGE_PROGRESS_SUPPORT

	//FineReader Support
private:
#ifdef IMAGE_FINEREADER_SUPPORT
	static FREngine::IEnginePtr	m_pFREngine;// FineReader Engine object.
	static HMODULE		m_FRLibraryHandle;	// HANDLE to FREngine.dll

	static BOOL			LoadFREngine();
	static BOOL			UnloadFREngine();
	virtual void		FR_Threshold();
#endif

};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

//Global Functions, preceded with Img...
IMG_DECL_CLASS CImage * ImgLoadFromFile(LPCTSTR lpszPathName, CRuntimeClass* Image32 = NULL, CRuntimeClass* Image8 = NULL, CRuntimeClass* Image8g = NULL, CRuntimeClass* Image1 = NULL);

#endif // !defined(AFX_IMAGE_H__DFA1CBC9_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
