// ImageMap.h: interface for the CImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IMAGEMAP_H__INCLUDED_)
#define AFX_IMAGEMAP_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Image.h"

//predefined string sizes
#define IMGMAP_PIXELTYPE_NAME_SIZE 15

//Pre Defined Maps
#define PDM_STANDARD	(BYTE)0x00
#define PDM_MEMORIAL	(BYTE)0x01

//MapTypes
#define MT_UNDEFINED	(BYTE)0x00
#define	MT_ATTRIBUTE	(BYTE)0x01	//Attribute (8 values (on off) only, multiple attributes allowed)
#define MT_LABEL		(BYTE)0x02	//Label (256 values, single label for each pixel)

//Standard Text Descriptions for common Pixel Types
#define PT_STR_FREE			_T("*Free*")
#define PT_STR_ALLOCATED	_T("*Allocated*")
#define PT_STR_FOREGROUND	_T("Foreground")
#define PT_STR_BACKGROUND	_T("Background")

//PreservePixelType attribute of ApplyMorphologicalFilter has a meaning dependent on the type of map
//for Attribute maps it defines the bits that should not be removed during erosion. For label maps
//it defines whether the foreground value or a value from the backup should be used during erosion
//accordingly to the following values
#define PPT_USE_FOREGROUND	0x00
#define PPT_USE_BACKUP		0x01

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CImageMap : public CObject
{
    DECLARE_SERIAL(CImageMap)

friend class CImage;
friend class CHiColorImage;
friend class CBiLevelImage;
friend class CLoColorImage;
friend class CGreyScaleImage;

public:
	void	MapByteRemoveAttribute(int x, int y, BYTE val);
	void	MapByteAddAttribute(int x, int y, BYTE val);
	BYTE	GetMapByte(int x, int y);
	void	SetMapByte(int x, int y, BYTE val);
	inline BOOL	GetMapColour(int x, int y, BYTE *Red, BYTE *Green, BYTE *Blue);
	BOOL	GetMapColour(BYTE MapValue, BYTE *Red, BYTE *Green, BYTE *Blue);
	//Attributes
	CString GetName();
	BOOL	SetPixelType(int index, LPCTSTR Name, BYTE Red, BYTE Green, BYTE Blue, BOOL Visible);
	CImage * GetImage();
	void	SetImage(CImage * pImage);
	//Pixel Type Attributes
	int		GetPTCount();
	BYTE	GetPTBlue(LPCTSTR Name);
	BYTE	GetPTBlue(int index);
	void	SetPTBlue(LPCTSTR Name, BYTE Blue);
	void	SetPTBlue(int index, BYTE Blue);
	BYTE	GetPTGreen(LPCTSTR Name);
	BYTE	GetPTGreen(int index);
	void	SetPTGreen(LPCTSTR Name, BYTE Green);
	void	SetPTGreen(int index, BYTE Green);
	BYTE	GetPTRed(LPCTSTR Name);
	BYTE	GetPTRed(int index);
	void	SetPTRed(LPCTSTR Name, BYTE Red);
	void	SetPTRed(int index, BYTE Red);
	BOOL	GetPTVisible(LPCTSTR Name);
	BOOL	GetPTVisible(int index);
	void	SetPTVisible(LPCTSTR Name, BOOL Visible);
	void	SetPTVisible(int index, BOOL Visible);
	int		GetPTIndex(LPCTSTR PixelTypeName);
	LPCTSTR GetPTName(BYTE ByteLabel);
	LPCTSTR GetPTName(int index);
	void	SetPTName(BYTE ByteLabel, LPCTSTR Name);
	void	SetPTName(int index, LPCTSTR Name);
	BYTE	GetPTByteLabel(LPCTSTR PixelTypeName);
	void	SetPTColour(int index, BYTE Red, BYTE Green, BYTE Blue);
	void	SetPTColour(LPCTSTR Name, BYTE Red, BYTE Green, BYTE Blue);
	//Operations
	void	GetProjection(BYTE PTByte, int Left, int Top, int Width, int Height, long * VertProj = NULL, long * HorProj = NULL, BOOL UseNOTMask = FALSE, BYTE Flags = PRJ_PIXEL_COUNT);
	void	Crop(int Left, int Top, int Width, int Height);
	void	Rotate(double Angle, BYTE NewPixelLabel = 0x00);
	void	Rotate(CRect Rect, double Angle, BYTE NewPixelLabel = 0x00, BOOL CropResultToOriginalSize = FALSE);
	void	Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL, BYTE NewPixelLabel = 0x00);
	BOOL	IsPixelType(int x, int y, LPCTSTR Name);
	BOOL	IsPixelType(int x, int y, BYTE value);
	BOOL	IsPreDefinedMap(BYTE PreDefinedMap);
	BOOL	AllocateLabels(int nLabels, int * LabelArray, LPCTSTR Name = NULL);
	int		GetNextFreeLabel();
	void	ApplyMorphologicalFilterToMap(BYTE Type, int Size = 3, BYTE PixelType = 0x01, BYTE PreservePixelType = 0x00); //PreservePixelType 0x00 equals PPT_USE_FOREGROUND (for label type maps)
	void	ClearMap(BYTE Value = NULL);
	void	SetToPreDefinedMap(BYTE PreDefinedMap = PDM_STANDARD);
	BOOL	ReSample(int NewWidth, int NewHeight);
	//Creation / Serialization
	BOOL CreateDuplicate(CImageMap * pMap, CImage * pImage = NULL);
	BOOL Create(CImage * pImage, LPCTSTR Name, BYTE MapType = MT_ATTRIBUTE);
	BOOL Create(int Width, int Height, LPCTSTR Name, BYTE MapType = MT_ATTRIBUTE);
	BOOL Create(CImage * pImage, BYTE PreDefinedMap = PDM_STANDARD);
	BOOL Create(int Width, int Height, BYTE PreDefinedMap = PDM_STANDARD);
	virtual void	Serialize(CArchive& ar);
	//Backup Functions
	CImageMap * CreateBackup();
	CImageMap * GetBackup();
	CImageMap * RestoreBackup();
	void DiscardBackup();
	//Utility Functions
	void		SetupLineArray(BYTE ** pLineArray, BYTE * pArray, int Height, int BytesPerLine);
	//Initialization
	CImageMap();
	virtual ~CImageMap();
	//Variables
private:
	inline void PrivateSetMapByte(int x, int y, BYTE val)
		{m_MapLineArray[y][x] = val;}
	inline BYTE PrivateGetMapByte(int x, int y)
		{return m_MapLineArray[y][x];}
	inline void PrivateMapByteAddAttribute(int x, int y, BYTE val)
		{m_MapLineArray[y][x] |= val;}
	inline void PrivateMapByteRemoveAttribute(int x, int y, BYTE val)
		{m_MapLineArray[y][x] &= (~val);}

private:
	void FreeMem();
	BYTE *		m_MapArray;
	BYTE **		m_MapLineArray;
	int			m_Width;
	int			m_Height;
	CImage *	m_pImage;
	CString		m_Name;
	CImageMap * m_Backup;
	typedef struct tagPixelType
	{
		TCHAR		Name[IMGMAP_PIXELTYPE_NAME_SIZE];
		BYTE		Red;
		BYTE		Green;
		BYTE		Blue;
		BOOL		Visible;
	} PixelType;
	PixelType * m_PixelType;
	BYTE		m_MapType; //Attribute (8 values (on off) only, multiple attributes allowed) /Label (256 values, single label for each pixel)
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_IMAGEMAP_H__INCLUDED_)
