// LoColorImage.h: interface for the CLoColorImage class.
//
//////////////////////////////////////////////////////////////////////
//Delevoped by Dimosthenis Karatzas: karatzas@csc.liv.ac.uk
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOCOLORIMAGE_H__DFA1CBCB_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
#define AFX_LOCOLORIMAGE_H__DFA1CBCB_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Image.h"

#include "LibraryDeclaration.h"

//Make sure that functions declared using AFX_API, i.e. friend operator >> for serialization, are exported correctly
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API AFX_EXT_CLASS
#endif

class IMG_DECL_CLASS CLoColorImage : virtual public CImage  
{
	DECLARE_SERIAL(CLoColorImage)

	friend class CHiColorImage;
	friend class CGreyScaleImage;
	friend class CBiLevelImage;
public:
	//Initialization
	CLoColorImage();
	virtual ~CLoColorImage();
	virtual BOOL		CreateFromImage(CImage *pImage, BYTE Flags = 0x00);
	virtual BOOL		CreateDuplicate(CImage *pImage, BOOL ReferenceArrays = FALSE);
	//Drawing Operations
protected:
	virtual void		FillBitmapInfo();
public:
	virtual BYTE*		UpdateNormal();
	virtual BYTE*		UpdateDimmed();
	//Operations
	virtual void		Rotate(double Angle, COLORREF BackColour = RGB(255, 255, 255), BOOL CropResultToOriginalSize = FALSE);
	virtual void		Rotate(CRect Rect, double Angle, COLORREF BackColour = ROT_PRESERVE_BACKGROUND, BOOL CropResultToOriginalSize = FALSE);
	virtual void		Tilt(double Angle, BYTE Flags = TILT_HORIZONTAL);
	virtual void		Crop(int Left, int Top, int Width, int Height);
	virtual void		Serialize(CArchive& ar);
	//Attributes
	virtual BYTE		GetRed(int x, int y);
	virtual BYTE		GetGreen(int x, int y);
	virtual BYTE		GetBlue(int x, int y);
	virtual void		SetRed(int x, int y, BYTE val);
	virtual void		SetGreen(int x, int y, BYTE val);
	virtual void		SetBlue(int x, int y, BYTE val);
	virtual void		SetRGBColor(int x, int y, COLORREF col);
	virtual COLORREF	GetRGBColor(int x, int y);
	virtual BOOL		DiscardBackup();
	virtual void		RestoreBackup();
	virtual BOOL		CreateBackup();

protected:
	//Initialization
	void				FreeMem();
	//Operations
	//Attributes
	inline int	GetClosestPaletteColor(COLORREF col);
	virtual int	GetClosestPaletteColor(BYTE R, BYTE G, BYTE B);
	inline void PrivateSetByte(int x, int y, BYTE val)
		{m_LineArray[y][x] = val;}

	inline BYTE PrivateGetByte(int x, int y)
		{return m_LineArray[y][x];}

	inline BYTE GetPaletteRed(int i)
	{	return (m_Palette == NULL?i:GetRValue(m_Palette[i]) );	}

	inline BYTE GetPaletteGreen(int i)
	{	return (m_Palette == NULL?i:GetGValue(m_Palette[i]) );	}

	inline BYTE GetPaletteBlue(int i)
	{	return (m_Palette == NULL?i:GetBValue(m_Palette[i]) );	}

	//Variables
	BYTE *		m_Array;
	BYTE **		m_LineArray;
	COLORREF *	m_Palette;
	BYTE *		m_BackupArray;
	BYTE **		m_BackupLineArray;
	COLORREF *	m_BackupPalette;
};

//Restore AFX_API
#ifndef IMAGE_CLASSES_NORMAL_USE
	#undef AFX_API
	#define AFX_API
#endif

#endif // !defined(AFX_LOCOLORIMAGE_H__DFA1CBCB_CF29_11D2_95E9_0060B0F07B9A__INCLUDED_)
