#include "ConnectedComponent.h"

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "Run.h"

// CONSTRUCTION

CConnectedComponent::CConnectedComponent()
{
	m_nRuns = 0;
	m_pRuns = new CRun * [RUN_INIT];
	m_nRunsAlloc = RUN_INIT;
	m_nNeighbours = 0;
	m_pNeighbours = NULL;
}

CConnectedComponent::~CConnectedComponent()
{
	delete [] m_pNeighbours;
	delete [] m_pRuns;
}

// METHODS

bool CConnectedComponent::AddRun(CRun * Run)
{
	while(m_nRuns + 1 > m_nRunsAlloc)
	{
		CRun ** temp = new CRun * [m_nRunsAlloc + RUN_INCR];
		memcpy(temp,m_pRuns,sizeof(CRun *) * m_nRunsAlloc);
		delete [] m_pRuns;
		m_pRuns = temp;
		m_nRunsAlloc += RUN_INCR;
//		printf("CConnectedComponent: Increasing Size of Run Array to %d\n",m_nRunsAlloc);
	}
	
	m_pRuns[m_nRuns] = Run;
	m_nRuns++;
	
	if(m_nRuns == 1)
	{
		m_nX1 = Run->GetX1();
		m_nX2 = Run->GetX2();
		m_nY1 = Run->GetY();
		m_nY2 = Run->GetY();
	}
	else
	{
//		printf("m_nX1 = %d\n",m_nX1);
//		printf("Run->GetX1() = %d\n",Run->GetX1());
		if(Run->GetX1() < m_nX1)
			m_nX1 = Run->GetX1();

		if(Run->GetX2() > m_nX2)
			m_nX2 = Run->GetX2();
		
		if(Run->GetY() < m_nY1)
			m_nY1 = Run->GetY();
		else if(Run->GetY() > m_nY2)
			m_nY2 = Run->GetY();
	}
	
	Run->SetCC(this);
	
	return true;
}

/*
bool CConnectedComponent::Create(CRun * Run)
{
	delete [] m_pRuns;
	m_nRuns = 0;
	m_pRuns = new CRun * [RUN_INIT];
	if(m_pRuns == NULL)
	{
		fprintf(stderr,"Create: Couldn't Allocate Memory for m_pRuns\n");
		return false;
	}
	m_nRunsAlloc = RUN_INIT;
	
	m_pRuns[0] = Run;
	m_nRuns = 1;
	m_nX1 = Run->GetX1();
	m_nX2 = Run->GetX2();
	m_nY1 = Run->GetY();
	m_nY2 = Run->GetY();

	Run->SetCC(this);
	
	return true;
}
*/

int CConnectedComponent::GetCentroidX()
{
	return (m_nX2 + m_nX1) / 2;
}

int CConnectedComponent::GetCentroidY()
{
	return (m_nY2 + m_nY1) / 2;
}

CConnectedComponent * CConnectedComponent::GetNeighbour(int Index)
{
	if(Index < 0)
	{
		fprintf(stderr,"GetNeighbour(%d): Index < 0\n",Index);
		return NULL;
	}
	else if(Index >= m_nNeighbours)
	{
		fprintf(stderr,"GetNeighbour(%d): Index >= %d\n",Index,m_nNeighbours);
		return NULL;
	}
	return m_pNeighbours[Index];
}

int CConnectedComponent::GetNoNeighbours()
{
	return m_nNeighbours;
}

int CConnectedComponent::GetNoRuns()
{
	return m_nRuns;
}

CRun * CConnectedComponent::GetRun(int Index)
{
	if(Index < 0 || Index >= m_nRuns)
		return NULL;
	else
		return m_pRuns[Index];
}

int CConnectedComponent::GetSize()
{
	return int(sqrt(double((1 + m_nX2 - m_nX1) * (1 + m_nY2 - m_nY1))));
}

int CConnectedComponent::GetX1()
{
	return m_nX1;
}

int CConnectedComponent::GetX2()
{
	return m_nX2;
}

int CConnectedComponent::GetY1()
{
	return m_nY1;
}

int CConnectedComponent::GetY2()
{
	return m_nY2;
}

bool CConnectedComponent::IsIn(int x, int y)
{
	int i;

	for(i = 0; i < m_nRuns; i++)
	{
		if
		(
			GetRun(i)->GetY() == y
			&&
			GetRun(i)->GetX1() <= x
			&&
			GetRun(i)->GetX2() >= x
		)
			return true;
	}
	return false;
}

bool CConnectedComponent::Merge(CConnectedComponent * CC)
{
	int i;
	CRun * Run;
	
	for(i = 0; i < CC->GetNoRuns(); i++)
	{
		Run = CC->GetRun(i);
		Run->SetCC(this);
		AddRun(Run);
	}
}

bool CConnectedComponent::SetNeighbour(int Index,CConnectedComponent * Neighbour)
{
	if(Index < 0 || Index >= m_nNeighbours)
		return false;
	else
	{
		m_pNeighbours[Index] = Neighbour;
		return true;
	}
}


bool CConnectedComponent::SetNoNeighbours(int NoNeighbours)
{
	delete [] m_pNeighbours;
	m_pNeighbours = new CConnectedComponent * [NoNeighbours];
	
	if(m_pNeighbours == NULL)
	{
		fprintf(stderr,"SetNoNeighbours: Couldn't Allocate Memory for m_pNeighbours\n");
		return false;
	}
	
	m_nNeighbours = NoNeighbours;
	return true;
}

bool CConnectedComponent::SetNoRuns(int Number)
{
	delete [] m_pRuns;
	m_pRuns = new CRun * [Number];
	if(m_pRuns == NULL)
	{
		fprintf(stderr,"SetNoRuns: Couldn't Allocate Memory for m_pRuns\n");
		return false;
	}
	m_nRuns = Number;
	return true;
}

