#ifndef CONNECTEDCOMPONENTS_H
#define CONNECTEDCOMPONENTS_H

#include "BiLevelImage.h"

#include "Histogram.h"

class CRun;
class CConnectedComponent;

class CConnectedComponents
{
	// CONSTRUCTION
public:
	CConnectedComponents();
	~CConnectedComponents();

	// METHODS
public:
	bool AddCC(CRun * Run);
	bool AddRun(int y, int x1, int x2);
	CHistogram * CreateBetweenLineNNDistHistogram(double Angle,double Tolerance);
	CHistogram * CreateComponentSizeHistogram();
	bool CreateFromImage(CBiLevelImage * Image, bool FourConnected = true, bool LookForBlack = true);
	CHistogram * CreateNNAngleHist(double BinAngle);
	bool CreateSubSet(CConnectedComponents * CC,int LowerBound, int UpperBound);
	CHistogram * CreateWithinLineNNDistHistogram(double Angle,double Tolerance);
	void DeleteCC(CConnectedComponent * CC);
	bool FindNearestNeighbours(int K);
	CConnectedComponent * GetCC(int Index);
	bool GetFourConnected();
	int  GetHeight();
	int  GetNoCCs();
	int  GetNoRuns(int Y);
	CRun * GetRun(int Y, int Index);
	int  GetWidth();
	void SetHeight(int NewHeight);
	void SortComponentsX();

	// DATA ITEMS
private:
	// m_bFourConnected = true if 4-Connected, false if 8-Connected
	bool m_bFourConnected;
	int m_nHeight;
	CRun *** m_pRuns;
	int  *   m_nRuns;
	int  *   m_nRunsAlloc;
	CConnectedComponent ** m_pCCs;
	int     m_nCCs;
	int     m_nCCsAlloc;
	int     m_nWidth;

//	static const int RUN_INIT = 100;
//	static const int RUN_INCR = 100;
//	static const int CC_INIT = 100;
//	static const int CC_INCR = 100;
	static const int RUN_INIT = 0;
	static const int RUN_INCR = 1;
	static const int CC_INIT = 0;
	static const int CC_INCR = 1;
};

#endif // CONNECTEDCOMPONENTS_H

